/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** Main Module
 ** 
 ** \brief This example shows how to use the ADC range compare function.
 **
 ** This example set the ADC0 channel 0 in repeat conversion mode with interrupt
 ** enabled and enable the ADC compare function.
 ** After input a voltage grearer than AVCC*1/4 or less than AVCC*3/4, break 
 ** the code and check if the flag "bAdcInRange" is set to true.
 **
 ** History:
 **   - 2014-03-03  1.0  Edison Zhang    First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

#define ADC_COMPARE_LOWER_VALUE     (0x400u)
#define ADC_COMPARE_UPPER_VALUE     (0xC00u)

volatile uint32_t  u32AdcResult = 0u;		// Global ADC result variable
volatile uint16_t  u16AdcValue;             // Result value
volatile uint8_t   u8AdcChannel;            // Result channel  
volatile boolean_t bAdcInRange = FALSE;	    // Interrupt notification flag

/**
 ******************************************************************************
 ** \brief  Error callback function
 ******************************************************************************/
void Main_adc_error_callback(void)
{
    // Error handling ... Should not happen.
}

/**
 ******************************************************************************
 ** \brief  Result callback function
 **
 ** \param  u32AdcArgument  ADC FIFO value
 ******************************************************************************/
void Main_adc_result_callback(uint32_t* pu32AdcArgument)
{
    stc_adcn_t*      pstcAdc = (stc_adcn_t*) &ADC0;
    stc_scan_fifo_info_t stcFifoInfo;
    u32AdcResult = (uint32_t)*pu32AdcArgument;             // Store result 
    Adc_ExtractScanFifo(pstcAdc, u32AdcResult, &stcFifoInfo);   // Retrieve channel from result
    u8AdcChannel = stcFifoInfo.u8Ch;    // Get sample channel
    u16AdcValue  = stcFifoInfo.u16Data; // Get result value
}

/**
 ******************************************************************************
 ** \brief  ADC compare interrupt callback function
 ******************************************************************************/
void Main_adc_compare_callback(void)
{
    bAdcInRange = TRUE; 
}


/**
 ******************************************************************************
 ** \brief  ADC0 initialization and single conversion start
 ******************************************************************************/
void Main_adc_irq(void)
{
    stc_adcn_t*      pstcAdc = (stc_adcn_t*) &ADC0;
    stc_adc_config_t stcConfig;
    stc_adc_int_sel_t stcIntSel;
    stc_adc_int_callback_t stcIntCallback;

    PDL_ZERO_STRUCT(stcConfig);		// Clear local configuration to zero.

    /* Common setting */
    stcConfig.bLsbAlignment = TRUE;
    stcConfig.u32SamplingTimeSelect.AD_CH_0 = 1u;
    stcConfig.enSamplingTimeN0 = Value32;
    stcConfig.u8SamplingTime0 = 30u;
    stcConfig.enSamplingTimeN1 = Value32;
    stcConfig.u8SamplingTime1 = 30u;
    stcConfig.u8SamplingMultiplier = 5u;
    stcConfig.u8EnableTime = 10u;   
    /* Scan conversion configuration */
    stcConfig.bScanInit = TRUE;
    stcConfig.stcScanInit.u32ScanCannelSelect.AD_CH_0 = 1u;
    stcConfig.stcScanInit.enScanMode = ScanRepeatConversion;
    stcConfig.stcScanInit.bScanTimerStartEnable = FALSE;
    stcConfig.stcScanInit.u8ScanFifoDepth = 0u;       // Only one conversion
    stcConfig.stcScanInit.enScanTimerTrigger = AdcNoTimer;
    /* Priority converison is not used, so don't initialize it */
    stcConfig.bPrioInit = FALSE;
    /* Comparison function is not used, so don't initialize it */
    stcConfig.bComparisonEnable = FALSE;
    /* Initialize range comparison function */
    stcConfig.bRangeComparisonEnable = TRUE;
    stcConfig.stcRangeComparisonInit.bRangeCompareAllChannels = FALSE;
    stcConfig.stcRangeComparisonInit.bWithinRange = TRUE;
    stcConfig.stcRangeComparisonInit.u16LowerLimitRangeValue = ADC_COMPARE_LOWER_VALUE;
    stcConfig.stcRangeComparisonInit.u16UpperLimitRangeValue = ADC_COMPARE_UPPER_VALUE;
    stcConfig.stcRangeComparisonInit.u8RangeComapreChannel = 0u;
    stcConfig.stcRangeComparisonInit.u8RangeCountValue = 3u; // Continuous detection state: 3 times
    
    if (Ok == Adc_Init(pstcAdc, &stcConfig))        // Init ADC0
    {
        /* Configure interrupt  */
        PDL_ZERO_STRUCT(stcIntSel);
        PDL_ZERO_STRUCT(stcIntCallback);
        stcIntSel.bScanIrq = 1;
        stcIntSel.bRangeComparisonIrq = 1;
        stcIntCallback.pfnScanCallback = (func_ptr_adc_parg32_t)Main_adc_result_callback;
        stcIntCallback.pfnRangeComparisonCallback = Main_adc_compare_callback;
        Adc_EnableInt(pstcAdc, &stcIntSel, &stcIntCallback);
        
        Adc_Enable(pstcAdc);               // Enable ADC0 and wait for ready
        Adc_SwTriggerScan(pstcAdc);        // Trigger ADC0
    }
}

/**
 ******************************************************************************
 ** \brief  Main function
 **
 ** \return int32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{
    Main_adc_irq();
    
    while(bAdcInRange != TRUE)
    {};

    Adc_DeInit(&ADC0);
    
    while(1)
    {};
  
}
