/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** Main Module
 ** 
 ** \brief This example shows how to use the ADC start compare unit.
 **
 ** Before using this exampel, user can connect P10/AN00 with AVCC and 
 ** P11/AN01 with GND. ADC samples the analog signal from AN00 and AN11.
 ** The ADC result data will output to terminal window.
 **
 ** Three mode are domenstrated as following:
 ** - normal mode
 ** - offset mode
 ** - FM3 compatible mode
 **
 ** In the normal mode, example 4 of FM0P peripheral manual [timer part] is 
 ** implemented. The ADCMP will compare the FRT count with ADCMP compare
 ** register of ADCMP ch.0 when FRT is counting up and compare the FRT count 
 ** ADC compare register of ADCMP ch.1 when FRT is counting down. Once the match
 ** occurs, ADC scan conversion will start.
 **
 ** In the offset mode, example 7 of FM0P peripheral manual [timer part] is 
 ** implemented. A compare value is first set in the ADCMP compare 
 ** register and the reload value of down counter is set in the ADCMP compare 
 ** register, when FRT count matchs OCU compare value, the down counter starts, 
 ** when the down counter underflows, ADC scan conversion will start.
 **
 ** In the FM3 compatible mode, it implements same function with that of normal
 ** mode.
 **
 ** History:
 **   - 2014-01-15  0.1  Edison Zhang        First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/* ADC */
/*! \brief ADC unit  */ 
#define USER_ADC_UNIT                    ADC0
/*! \brief Scan channel number  */ 
#define USER_ADC_SCAN_CH_NUM             (2)
/*! \brief ADC scan channel 0  */ 
#define USER_ADC_SCAN_CH0                 0
/*! \brief ADC scan channel 1  */ 
#define USER_ADC_SCAN_CH1                 1
/* FRT */
/*! \brief User MFT unit */
#define USER_MFT_FRT_CH      MFT0_FRT 
/*! \brief User free run timer channel */
#define USER_FRT_CH      MFT_FRT_CH2 
/*! \brief User free run timer mode */
#define FRT_MODE         FrtUpDownCount 
/* ADCMP */
/*! \brief User ADC compare unit */
#define USER_MFT_ADCMP_CH          MFT0_ADCMP
/*! \brief User ADC compare channel 0 */
#define USER_ADCMP_CH0                 0
/*! \brief User ADC compare channel 1 */
#define USER_ADCMP_CH1                 1
/*! \brief Couple channel */
#define USER_ADCMP_COUPLE_CH           MFT_ADCMP_CH10
/*! \brief User ADC compare unit to trigger ADC scan */
#define USER_ADCMP_SEL_SOURCE   (AdcmpTrigAdc0Scan) 
/* OCU */
/*! \brief User ADC compare unit */
#define USER_MFT_OCU_CH           MFT0_OCU
/*! \brief User OCU channel 0 */
#define USER_OCU_CH0              USER_ADCMP_CH0
/*! \brief User OCU channel 1 */
#define USER_OCU_CH1              USER_ADCMP_CH1   

/*---------------------------------------------------------------------------*/
/* global data                                                               */
/*---------------------------------------------------------------------------*/   
/*! \brief Read finish flag  */ 
boolean_t bReadFinished = FALSE;
/*! \brief Scan data  */ 
uint16_t ScanData[20][USER_ADC_SCAN_CH_NUM];
/*! \brief Scan data  */ 
uint16_t ScanDataCnt = 0;


/**
 ******************************************************************************
 ** \brief  Result callback function
 **
 ** \param  u32AdcArgument  ADC FIFO value
 ******************************************************************************/
void AdcScanIntCallback(uint32_t* pu32AdcArgument)
{
    uint32_t AdcData, Cnt;
    stc_scan_fifo_info_t stcFifoInfo;
    Cnt = USER_ADC_SCAN_CH_NUM;

    if(ScanDataCnt >= sizeof(ScanData)/sizeof(ScanData[0]))
    {
        ScanDataCnt = 0;
        bReadFinished = 1; /* Finish timer trigger */
        Mft_Frt_Stop(&USER_MFT_FRT_CH ,USER_FRT_CH);
        Adc_ClrScanFifo((stc_adcn_t*)&USER_ADC_UNIT); /* Give up FIFO data */

        return;
    }

    while(Cnt--)
    {
        AdcData = *pu32AdcArgument;
        Adc_ExtractScanFifo((stc_adcn_t*)&USER_ADC_UNIT, AdcData, &stcFifoInfo);

        if(stcFifoInfo.u8Ch == USER_ADC_SCAN_CH0)
        {
            ScanData[ScanDataCnt][0] = stcFifoInfo.u16Data;
        }
        else if(stcFifoInfo.u8Ch == USER_ADC_SCAN_CH1)
        {
            ScanData[ScanDataCnt][1] = stcFifoInfo.u16Data;
        }
    }
    ScanDataCnt++;
}   
   
/**
 ******************************************************************************
 ** \brief  Error callback function
 ******************************************************************************/
void AdcErrorCallback(void)
{
  // Error handling ... Should not happen.
}

/**
 ******************************************************************************
 ** \brief  Result callback function
 **
 ** \param  u32AdcArgument  ADC FIFO value
 ******************************************************************************/
void InitAdc(void)
{
    stc_adcn_t*      pstcAdc = (stc_adcn_t*) &USER_ADC_UNIT;
    stc_adc_config_t stcConfig;
    stc_adc_int_sel_t stcIntSel;
    stc_adc_int_callback_t stcIntCallback;

    PDL_ZERO_STRUCT(stcConfig);		// Clear local configuration to zero.

    /* Common setting */
    stcConfig.bLsbAlignment = TRUE;
    stcConfig.u32SamplingTimeSelect.u32AD_CHn = 0xFFFFFFFFu;
    stcConfig.enSamplingTimeN0 = Value32;
    stcConfig.u8SamplingTime0 = 30u;
    stcConfig.enSamplingTimeN1 = Value32;
    stcConfig.u8SamplingTime1 = 30u;
    stcConfig.u8SamplingMultiplier = 5u;
    stcConfig.u8EnableTime = 10u;
    /* Scan conversion configuration */
    stcConfig.bScanInit = TRUE;
    stcConfig.stcScanInit.u32ScanCannelSelect.u32AD_CHn = ((1u << USER_ADC_SCAN_CH0) |
                                                           (1u << USER_ADC_SCAN_CH1) );
    stcConfig.stcScanInit.enScanMode = ScanSingleConversion;
    stcConfig.stcScanInit.bScanTimerStartEnable = TRUE;
    stcConfig.stcScanInit.u8ScanFifoDepth = (USER_ADC_SCAN_CH_NUM - 1);       // Only one conversion
    stcConfig.stcScanInit.enScanTimerTrigger = AdcMft;
    /* Priority converison is not used, so don't initialize it */
    stcConfig.bPrioInit = FALSE;
    /* Comparison function is not used, so don't initialize it */
    stcConfig.bComparisonEnable = FALSE;
    /* Range comparison function is not used, so don't initialize it */
    stcConfig.bRangeComparisonEnable = FALSE;

    if (Ok == Adc_Init(pstcAdc, &stcConfig))        // Init ADC0
    {
        /* Configure interrupt  */
        PDL_ZERO_STRUCT(stcIntSel);
        PDL_ZERO_STRUCT(stcIntCallback);
        stcIntSel.bScanIrq = 1;
        stcIntSel.bFifoOverrunIrq = 1;
        stcIntCallback.pfnScanCallback = (func_ptr_adc_parg32_t)AdcScanIntCallback;
        stcIntCallback.pfnScanErrorCallback = AdcErrorCallback;
        Adc_EnableInt(pstcAdc, &stcIntSel, &stcIntCallback);

        Adc_Enable(pstcAdc);               // Enable ADC0 and wait for ready
        Adc_SwTriggerScan(pstcAdc);        // Trigger ADC0
    }

}

/**
 ******************************************************************************
 ** \brief  Initialize ADC start compare unit with normal mode
 ******************************************************************************/
void InitAdcmpNormalMode(void)
{
    stc_mft_adcmp_config_t stcAdcmpConfig;
    stc_mft_adcmp_func_t stcAdcmpFunc0, stcAdcmpFunc1;
      
    PDL_ZERO_STRUCT(stcAdcmpConfig);      // Clear local configuration to zero.
    PDL_ZERO_STRUCT(stcAdcmpFunc0);
    PDL_ZERO_STRUCT(stcAdcmpFunc1);
    
    stcAdcmpConfig.enFrt = (en_adcmp_frt_t)USER_FRT_CH;
    stcAdcmpConfig.enMode = AdcmpNormalMode;
    stcAdcmpConfig.enBuf = AdcmpBufFrtZero;  
    stcAdcmpConfig.enTrigSel = USER_ADCMP_SEL_SOURCE;
    stcAdcmpFunc0.bZeroEn = TRUE;
    stcAdcmpFunc0.bUpEn = TRUE;
    stcAdcmpFunc0.bPeakEn = FALSE;
    stcAdcmpFunc0.bDownEn = FALSE;
    stcAdcmpFunc1.bZeroEn = FALSE;
    stcAdcmpFunc1.bUpEn = FALSE;
    stcAdcmpFunc1.bPeakEn = TRUE;
    stcAdcmpFunc1.bDownEn = TRUE;
        
    if( Ok == Mft_Adcmp_Init(&USER_MFT_ADCMP_CH ,USER_ADCMP_CH0, &stcAdcmpConfig))
    {
        // (1/(40M/64) * 20000 = 32ms (first trigger)
        Mft_Adcmp_WriteAcmp(&USER_MFT_ADCMP_CH, USER_ADCMP_CH0, 20000); 
        Mft_Adcmp_EnableOperation(&USER_MFT_ADCMP_CH, USER_ADCMP_CH0, &stcAdcmpFunc0);
    }
    
    if( Ok == Mft_Adcmp_Init(&USER_MFT_ADCMP_CH ,USER_ADCMP_CH1, &stcAdcmpConfig))
    {
        // (1/(40M/64) * (60000 + (60000-40000)) = 128ms (second trigger)
        Mft_Adcmp_WriteAcmp(&USER_MFT_ADCMP_CH, USER_ADCMP_CH1, 40000);
        Mft_Adcmp_EnableOperation(&USER_MFT_ADCMP_CH, USER_ADCMP_CH1, &stcAdcmpFunc1);
    }
}

/**
 ******************************************************************************
 ** \brief  Initialize ADC start compare unit with offset mode
 ******************************************************************************/
void InitAdcmpOffsetMode(void)
{
    stc_mft_adcmp_config_t stcAdcmpConfig;
    stc_mft_adcmp_func_t stcAdcmpFunc0, stcAdcmpFunc1;
    stc_mft_ocu_config_t stcOcuConfig;
     
    // Clear local configuration to zero.
    PDL_ZERO_STRUCT(stcAdcmpConfig);
    PDL_ZERO_STRUCT(stcAdcmpFunc0);
    PDL_ZERO_STRUCT(stcAdcmpFunc1);
    PDL_ZERO_STRUCT(stcOcuConfig);
    
    stcAdcmpConfig.enFrt = (en_adcmp_frt_t)USER_FRT_CH;
    stcAdcmpConfig.enMode = AdcmpOffsetMode;
    stcAdcmpConfig.enBuf = AdcmpBufDisable;  
    stcAdcmpConfig.enTrigSel = USER_ADCMP_SEL_SOURCE;
    stcAdcmpConfig.enOccpSel = AdcmpSelOccp0;
    
    stcAdcmpFunc0.bZeroEn = TRUE;
    stcAdcmpFunc0.bUpEn = TRUE;
    stcAdcmpFunc0.bPeakEn = FALSE;
    stcAdcmpFunc0.bDownEn = FALSE;
    
    stcAdcmpFunc1.bZeroEn = TRUE;
    stcAdcmpFunc1.bUpEn = TRUE;
    stcAdcmpFunc1.bPeakEn = FALSE;
    stcAdcmpFunc1.bDownEn = FALSE;
       
    stcOcuConfig.bFm4 = TRUE;
    stcOcuConfig.enFrtConnect = (en_mft_ocu_frt_t)USER_FRT_CH;
    
    // Count from 10000
    if(Ok == Mft_Ocu_Init(&USER_MFT_OCU_CH, USER_OCU_CH0, &stcOcuConfig))
    {
        Mft_Ocu_WriteOccp(&USER_MFT_OCU_CH, USER_OCU_CH0, 10000);
    }
    
    if( Ok == Mft_Adcmp_Init(&USER_MFT_ADCMP_CH ,USER_ADCMP_CH0, &stcAdcmpConfig))
    {
        // (1/(40M/64) * (10000+10000) = 32ms (first trigger)
        Mft_Adcmp_WriteAcmp(&USER_MFT_ADCMP_CH, USER_ADCMP_CH0, 10000); 
        Mft_Adcmp_EnableOperation(&USER_MFT_ADCMP_CH, USER_ADCMP_CH0, &stcAdcmpFunc0);
    }   
    if( Ok == Mft_Adcmp_Init(&USER_MFT_ADCMP_CH ,USER_ADCMP_CH1, &stcAdcmpConfig))
    {
        // (1/(40M/64) * (30000+10000) = 64ms (second trigger)
        Mft_Adcmp_WriteAcmp(&USER_MFT_ADCMP_CH, USER_ADCMP_CH1, 30000);
        Mft_Adcmp_EnableOperation(&USER_MFT_ADCMP_CH, USER_ADCMP_CH1, &stcAdcmpFunc1);
    } 
}

/**
 ******************************************************************************
 ** \brief  Initialize ADC start compare unit with FM3 mode
 ******************************************************************************/
void InitAdcmpFm3Mode(void)
{
    stc_mft_adcmp_fm3_config_t stcAdcmpFm3Config;
    
    // Clear local configuration to zero.
    PDL_ZERO_STRUCT(stcAdcmpFm3Config);
    
    stcAdcmpFm3Config.enFrt = (en_adcmp_fm3_frt_t)USER_FRT_CH;
    stcAdcmpFm3Config.enBuf = AdcmpBufFrtZero;
    stcAdcmpFm3Config.enMode = AdcmpFm3AccpUpAccpdnDown;
    stcAdcmpFm3Config.enTrigSel = USER_ADCMP_SEL_SOURCE;
    
    if(Ok == Mft_Adcmp_Fm3_Init(&USER_MFT_ADCMP_CH, USER_ADCMP_COUPLE_CH, &stcAdcmpFm3Config))
    {
        Mft_Adcmp_Fm3_WriteAccp(&USER_MFT_ADCMP_CH, USER_ADCMP_COUPLE_CH, 20000);
        Mft_Adcmp_Fm3_WriteAccpdn(&USER_MFT_ADCMP_CH, USER_ADCMP_COUPLE_CH, 40000);
        Mft_Adcmp_Fm3_EnableOperation(&USER_MFT_ADCMP_CH, USER_ADCMP_COUPLE_CH);
    }
}

/**
 ******************************************************************************
 ** \brief  Disable ADCMP
 ******************************************************************************/
void DeInitAdcmp(void)
{
    stc_mft_adcmp_func_t stcAdcmpFunc0;
    PDL_ZERO_STRUCT(stcAdcmpFunc0);

    stcAdcmpFunc0.bZeroEn = TRUE;
    stcAdcmpFunc0.bUpEn = TRUE;
    stcAdcmpFunc0.bPeakEn = TRUE;
    stcAdcmpFunc0.bDownEn = TRUE;
    
    Mft_Adcmp_DisableOperation(&USER_MFT_ADCMP_CH, USER_ADCMP_CH0, &stcAdcmpFunc0);
    Mft_Adcmp_Fm3_DisableOperation(&USER_MFT_ADCMP_CH, USER_ADCMP_COUPLE_CH);
    
}

/**
 ******************************************************************************
 ** \brief  Initialize FRT
 ******************************************************************************/
void InitFrt(void)
{
    stc_mft_frt_config_t stcFrtConfig;
    
    PDL_ZERO_STRUCT(stcFrtConfig);      // Clear local configuration to zero.
    
    //configure the FRT parameter
    stcFrtConfig.enFrtMode = FRT_MODE;
    stcFrtConfig.enFrtClockDiv = FrtPclkDiv64;
    stcFrtConfig.bEnExtClock = FALSE;
    stcFrtConfig.bEnBuffer = TRUE;

    Mft_Frt_Stop(&USER_MFT_FRT_CH ,USER_FRT_CH);
    //init frt module
    if( Ok == Mft_Frt_Init(&USER_MFT_FRT_CH ,USER_FRT_CH,&stcFrtConfig) )
    {
        // (1/(40M/64) * 60000 = 96ms
        Mft_Frt_SetCountCycle(&USER_MFT_FRT_CH ,USER_FRT_CH, 60000); 
        
    }
}

/**
 ******************************************************************************
 ** \brief  Print ADC result data in the terminal window
 ******************************************************************************/
void PrintAdcData(void)
{
    uint16_t i , j, u16Times, u16Size;
    uint8_t au8Ch[USER_ADC_SCAN_CH_NUM] = {USER_ADC_SCAN_CH0, USER_ADC_SCAN_CH1};
    
    u16Size = USER_ADC_SCAN_CH_NUM;
    u16Times = sizeof(ScanData)/sizeof(ScanData[0]);
    i = 0;
    j = 0;
    while(i != u16Times)
    {
    #ifdef DEBUG_PRINT
        printf("------------ Sample %d-------------\n\r", (i+1));
    #endif
        while(j != u16Size)
        {
            /*Print Result */
        #ifdef DEBUG_PRINT
            printf( "<Ch%d> ADC value:%d\n\r", au8Ch[j], ScanData[i][j]);
        #endif
            j++;
        }
    #ifdef DEBUG_PRINT
        printf("\n\r");
    #endif
        i++;
        j = 0;
    }
}

/**
 ******************************************************************************
 ** \brief  Main function
 **
 ** \return int32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{
    InitAdc();
    InitFrt();
    
    /* 1. Demo normal mode of adc start compare unit */
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("          Normal mode of ADC start compare unit \n");
    printf("==================================================\n");
#endif
    InitAdcmpNormalMode();
    Mft_Frt_Start(&USER_MFT_FRT_CH ,USER_FRT_CH); 
    while(1)
    {
        if(bReadFinished == TRUE)
        {
            bReadFinished = FALSE;
            PrintAdcData();
            break;
        }
    }
    DeInitAdcmp();
    
    /* 2. Demo offset mode of adc start compare unit */
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("          Offset mode of ADC start compare unit \n");
    printf("==================================================\n");
#endif    
    InitAdcmpOffsetMode();
    Mft_Frt_Start(&USER_MFT_FRT_CH ,USER_FRT_CH); 
    while(1)
    {
        if(bReadFinished == TRUE)
        {
            bReadFinished = FALSE;
            PrintAdcData();
            break;
        }
    }
    DeInitAdcmp();
    /* 3. Demo FM3 compatible mode of adc start compare unit */
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("    FM3 compatible mode of ADC start compare unit \n");
    printf("==================================================\n");
#endif     
    InitAdcmpFm3Mode();
    Mft_Frt_Start(&USER_MFT_FRT_CH ,USER_FRT_CH); 
    while(1)
    {
        if(bReadFinished == TRUE)
        {
            bReadFinished = FALSE;
            PrintAdcData();
            break;
        }
    }
    DeInitAdcmp();
    
    Adc_DeInit((stc_adcn_t*) &ADC0);

    while(1)
    {}
  
}
