/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved.
*
* This software is owned and published by:
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion
* components. This software is licensed by Spansion to be adapted only
* for use in systems utilizing Spansion components. Spansion shall not be
* responsible for misuse or illegal use of this software for devices not
* supported herein.  Spansion is providing this software "AS IS" and will
* not be responsible for issues arising from incorrect user implementation
* of the software.
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS),
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING,
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED
* WARRANTY OF NONINFRINGEMENT.
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT,
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION,
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA,
* SAVINGS OR PROFITS,
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED
* FROM, THE SOFTWARE.
*
* This software may be replicated in part or whole for the licensed use,
* with the restriction that this Disclaimer and Copyright notice must be
* included with each copy of this software, whether used in part or whole,
* at all times.
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates BT IO mode 1~7.
 **
 ** History:
 **   - 2013-07-23  0.0.1  Edison Zhang        First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
/*! \brief Demo BT IO mode 1 */
#define  DEMO_BT_IOMODE_1               0
/*! \brief Demo BT IO mode 2,3  */
#define  DEMO_BT_IOMODE_2_3             1
/*! \brief Demo BT IO mode 2,8  */
#define  DEMO_BT_IOMODE_2_8             2
/*! \brief Demo BT IO mode 4  */
#define  DEMO_BT_IOMODE_4               3
/*! \brief Demo BT IO mode 5  */
#define  DEMO_BT_IOMODE_5               4
/*! \brief Demo BT IO mode 6  */
#define  DEMO_BT_IOMODE_6               5
/*! \brief Demo BT IO mode 7  */
#define  DEMO_BT_IOMODE_7               6

/*! \brief Select a demo mode  */
#define  DEMO_MODE                      DEMO_BT_IOMODE_1

/*---------------------------------------------------------------------------*/
/* global data                                                               */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global functions                                                          */
/*---------------------------------------------------------------------------*/

#if (DEMO_MODE == DEMO_BT_IOMODE_1)
/*!
 ******************************************************************************
 ** \brief BT IO mode 1 demo code
 ******************************************************************************
 */
static void DemoBtIoMode1(void)
{
    stc_bt_pwm_config_t stcPwmConfig;
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Base Timer IO Mode 1 \n");
    printf("==================================================\n");
    printf("ECK  ------ TIOB1\n");
    printf("TGIN ------ TIOA1\n");
    printf("TIN  ------ TIOB0\n");
    printf("TOUT ------ TIOA0\n");
    printf("==================================================\n");
#endif

    // Set Basetimer IO port
    SetPinFunc_TIOA0_0_OUT();
    SetPinFunc_TIOB0_0_IN();
    SetPinFunc_TIOA1_0_IN();
    SetPinFunc_TIOB1_0_IN();

    // Set requested I/O mode
    Bt_ConfigIOMode(&BT0, BtIoMode1);

    // Set PWM mode
    Bt_SelTimerMode(&BT0, BtPwmMode);

    // Initialize PWM timer
    stcPwmConfig.enPres = PwmPres1Div4;  // PWM clock = 5MHz @ PCLK = 20MHz
    stcPwmConfig.enMode = PwmContinuous;
    stcPwmConfig.enExtTrig = PwmExtTrigBoth;
    stcPwmConfig.enOutputMask = PwmOutputNormal;
    stcPwmConfig.enOutputPolarity = PwmPolarityLow;
    stcPwmConfig.enRestartEn = PwmRestartDisable;
    Bt_Pwm_Init(&BT0, &stcPwmConfig);

    // Set cycle and duty value
    Bt_Pwm_WriteCycleVal(&BT0, 999); // Cycle = (1+m)*PWM clock cycle = 200us
    Bt_Pwm_WriteDutyVal(&BT0, 199);  // Duty = (1+m)*PWM clock cycle = 40us
#ifdef DEBUG_PRINT
    printf("Connect P3F/TIOA1_0 with P0F\n");
    printf("Press SW2\n");
    printf("PWM wave output from P3E/TIOA0_0\n");
#endif

    // Enable count operatoin
    Bt_Pwm_EnableCount(&BT0);

    while(1);
}
#endif

#if (DEMO_MODE == DEMO_BT_IOMODE_2_3)
/*!
 ******************************************************************************
 ** \brief BT IO mode 2,3 demo code
 ******************************************************************************
 */
static void DemoBtIoMode23(void)
{
    stc_bt_pwm_config_t stcPwmConfig;
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Base Timer IO Mode 2 \n");
    printf("==================================================\n");
    printf("ECK  ------ TIOB1\n");
    printf("TGIN ------ TIOA1\n");
    printf("TIN  ------ TIOB0\n");
    printf("TOUT ------ TIOA0\n");
    printf("==================================================\n");
    printf("Base Timer IO Mode 3 \n");
    printf("==================================================\n");
    printf("ECK  ------ TIOB1\n");
    printf("TGIN ------ TIOA1\n");
    printf("TIN  ------ TIOB0\n");
    printf("TOUT ------ TIOA0\n");
#endif

    // Set Basetimer IO port
    SetPinFunc_TIOA0_0_OUT();
    SetPinFunc_TIOB0_0_IN();
    SetPinFunc_TIOA1_0_OUT();
    SetPinFunc_TIOB1_0_IN();

    SetPinFunc_TIOA2_0_OUT();
    SetPinFunc_TIOB2_0_IN();
    SetPinFunc_TIOA3_1_OUT();
    //SetPinFunc_TIOB3_0_IN();

    // Set requested I/O mode
    Bt_ConfigIOMode(&BT0, BtIoMode2);
    Bt_ConfigIOMode(&BT2, BtIoMode3);
    // Set PWM mode
    Bt_SelTimerMode(&BT0, BtPwmMode);
    Bt_SelTimerMode(&BT1, BtPwmMode);
    Bt_SelTimerMode(&BT2, BtPwmMode);
    Bt_SelTimerMode(&BT3, BtPwmMode);
    // Initialize PWM timer
    stcPwmConfig.enPres = PwmPres1Div4;  // PWM clock = 5MHz @ PCLK = 20MHz
    stcPwmConfig.enMode = PwmContinuous;
    stcPwmConfig.enExtTrig = PwmExtTrigBoth;
    stcPwmConfig.enOutputMask = PwmOutputNormal;
    stcPwmConfig.enOutputPolarity = PwmPolarityLow;
    stcPwmConfig.enRestartEn = PwmRestartDisable;

    Bt_Pwm_Init(&BT0, &stcPwmConfig);
    Bt_Pwm_Init(&BT1, &stcPwmConfig);
    Bt_Pwm_Init(&BT2, &stcPwmConfig);
    Bt_Pwm_Init(&BT3, &stcPwmConfig);

    // Set cycle and duty value
    Bt_Pwm_WriteCycleVal(&BT0, 999); // Cycle = (1+m)*PWM clock cycle = 200us
    Bt_Pwm_WriteDutyVal(&BT0, 199);  // Duty = (1+m)*PWM clock cycle = 40us
    Bt_Pwm_WriteCycleVal(&BT1, 999); // Cycle = (1+m)*PWM clock cycle = 200us
    Bt_Pwm_WriteDutyVal(&BT1, 199);  // Duty = (1+m)*PWM clock cycle = 40us
    Bt_Pwm_WriteCycleVal(&BT2, 999); // Cycle = (1+m)*PWM clock cycle = 200us
    Bt_Pwm_WriteDutyVal(&BT2, 199);  // Duty = (1+m)*PWM clock cycle = 40us
    Bt_Pwm_WriteCycleVal(&BT3, 999); // Cycle = (1+m)*PWM clock cycle = 200us
    Bt_Pwm_WriteDutyVal(&BT3, 199);  // Duty = (1+m)*PWM clock cycle = 40us
#ifdef DEBUG_PRINT
    printf("TIOA0~TIOA3 shares same trigger.\n");
    printf("Trigger->TIOB0->COUT->CIN(BT2)->COUT(BT2)\n");
    printf("Connect P49/TIOB0_0 with P0F\n");
    printf("Press SW2\n");
    printf("PWM wave output from P3E/TIOA0_0,P3F/TIOA1_0,P23/TIOA2_0,P3D/TIOA3_1\n");
#endif
    // Enable count operatoin
    Bt_Pwm_EnableCount(&BT0);
    Bt_Pwm_EnableCount(&BT1);
    Bt_Pwm_EnableCount(&BT2);
    Bt_Pwm_EnableCount(&BT3);

    while(1);

}
#endif

#if (DEMO_MODE == DEMO_BT_IOMODE_2_8)
/*!
 ******************************************************************************
 ** \brief BT IO mode 2,3,8 demo code
 ******************************************************************************
 */
static void DemoBtIoMode28(void)
{
    stc_bt_pwm_config_t stcPwmConfig;
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Base Timer IO Mode 2 \n");
    printf("==================================================\n");
    printf("ECK  ------ TIOB1\n");
    printf("TGIN ------ TIOA1\n");
    printf("TIN  ------ TIOB0\n");
    printf("TOUT ------ TIOA0\n");
    printf("==================================================\n");
    printf("Base Timer IO Mode 8 \n");
    printf("==================================================\n");
    printf("ECK  ------ TIOB3\n");
    printf("TGIN ------ TIOA3\n");
    printf("TIN  ------ TIOB2\n");
    printf("TOUT ------ TIOA2\n");
    printf("==================================================\n");
#endif

    // Set Basetimer IO port
    SetPinFunc_TIOA0_0_OUT();
    SetPinFunc_TIOB0_0_IN();
    SetPinFunc_TIOA1_0_OUT();
    SetPinFunc_TIOB1_0_IN();

    SetPinFunc_TIOA2_0_OUT();
    SetPinFunc_TIOB2_0_IN();
    SetPinFunc_TIOA3_1_OUT();
    //SetPinFunc_TIOB3_0_IN();

    // Set requested I/O mode
    Bt_ConfigIOMode(&BT0, BtIoMode2);
    Bt_ConfigIOMode(&BT2, BtIoMode8);
    // Set PWM mode
    Bt_SelTimerMode(&BT0, BtPwmMode);
    Bt_SelTimerMode(&BT1, BtPwmMode);
    Bt_SelTimerMode(&BT2, BtPwmMode);
    Bt_SelTimerMode(&BT3, BtPwmMode);
    // Initialize PWM timer
    stcPwmConfig.enPres = PwmPres1Div4;  // PWM clock = 5MHz @ PCLK = 20MHz
    stcPwmConfig.enMode = PwmContinuous;
    stcPwmConfig.enExtTrig = PwmExtTrigRising;
    stcPwmConfig.enOutputMask = PwmOutputNormal;
    stcPwmConfig.enOutputPolarity = PwmPolarityLow;
    stcPwmConfig.enRestartEn = PwmRestartEnable;

    Bt_Pwm_Init(&BT0, &stcPwmConfig);
    Bt_Pwm_Init(&BT1, &stcPwmConfig);
    Bt_Pwm_Init(&BT2, &stcPwmConfig);
    Bt_Pwm_Init(&BT3, &stcPwmConfig);

    // Set cycle and duty value
    Bt_Pwm_WriteCycleVal(&BT0, 5999); // Cycle = (1+m)*PWM clock cycle = 1.2ms
    Bt_Pwm_WriteDutyVal(&BT0, 2999);  // Duty = (1+m)*PWM clock cycle = 0.6ms
    Bt_Pwm_WriteCycleVal(&BT1, 5999); // Cycle = (1+m)*PWM clock cycle = 1.2ms
    Bt_Pwm_WriteDutyVal(&BT1, 2999);  // Duty = (1+m)*PWM clock cycle = 0.6ms
    Bt_Pwm_WriteCycleVal(&BT2, 999); // Cycle = (1+m)*PWM clock cycle = 200us
    Bt_Pwm_WriteDutyVal(&BT2, 199);  // Duty = (1+m)*PWM clock cycle = 40us
    Bt_Pwm_WriteCycleVal(&BT3, 999); // Cycle = (1+m)*PWM clock cycle = 200us
    Bt_Pwm_WriteDutyVal(&BT3, 199);  // Duty = (1+m)*PWM clock cycle = 40us
#ifdef DEBUG_PRINT
    printf("Connect TIOB0 with P0F. \n");
    printf("TIOA2~TIOA7 shares same trigger.\n");
    printf("Trigger->TIOB0->COUT->CIN(BT2)->COUT(BT2)\n");
    printf("1) Press SW2 down\n");
    printf("2) PWM output from TIOA0~TIOA3\n");
    printf("3) Press SW2 up\n");
#endif
    // Enable count operatoin
    Bt_Pwm_EnableCount(&BT0);
    Bt_Pwm_EnableCount(&BT1);
    Bt_Pwm_EnableCount(&BT2);
    Bt_Pwm_EnableCount(&BT3);

    while(1);

}
#endif

#if (DEMO_MODE == DEMO_BT_IOMODE_4)
/*!
 ******************************************************************************
 ** \brief BT IO mode 4 demo code
 ******************************************************************************
 */
static void DemoBtIoMode4(void)
{
    stc_bt_pwm_config_t stcPwmConfig;
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Base Timer IO Mode 4 \n");
    printf("==================================================\n");
#endif

    // Set Basetimer IO port
    SetPinFunc_TIOA0_0_OUT();
    SetPinFunc_TIOB0_0_IN();
    SetPinFunc_TIOA1_0_OUT();

    // Set requested I/O mode
    Bt_ConfigIOMode(&BT0, BtIoMode4);

    // Set PWM mode
    Bt_SelTimerMode(&BT0, BtPwmMode);
    Bt_SelTimerMode(&BT1, BtPwmMode);

    // Initialize PWM timer
    stcPwmConfig.enPres = PwmPres1Div4;  // PWM clock = 5MHz @ PCLK = 20MHz
    stcPwmConfig.enMode = PwmContinuous;
    stcPwmConfig.enExtTrig = PwmExtTrigRising;
    stcPwmConfig.enOutputMask = PwmOutputNormal;
    stcPwmConfig.enOutputPolarity = PwmPolarityLow;
    stcPwmConfig.enRestartEn = PwmRestartEnable;
    Bt_Pwm_Init(&BT0, &stcPwmConfig);
    Bt_Pwm_Init(&BT1, &stcPwmConfig);

    // Set cycle and duty value
    Bt_Pwm_WriteCycleVal(&BT0, 59999); // Cycle = (1+m)*PWM clock cycle = 12ms
    Bt_Pwm_WriteDutyVal(&BT0, 29999);  // Duty = (1+m)*PWM clock cycle = 6ms
    Bt_Pwm_WriteCycleVal(&BT1, 399); // Cycle = (1+m)*PWM clock cycle = 80us
    Bt_Pwm_WriteDutyVal(&BT1, 199);  // Duty = (1+m)*PWM clock cycle = 40us

    // Enable count operatoin
    Bt_Pwm_EnableCount(&BT0);
    Bt_Pwm_EnableCount(&BT1);

    // Software trigger
    Bt_Pwm_EnableSwTrig(&BT0);

    while(1);
}
#endif

#if (DEMO_MODE == DEMO_BT_IOMODE_5)
/*!
 ******************************************************************************
 ** \brief BT IO mode 5 demo code
 ******************************************************************************
 */
static void DemoBtIoMode5(void)
{
    stc_bt_pwm_config_t stcPwmConfig;
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Base Timer IO Mode 5 \n");
    printf("=================================================\n");
#endif

    // Set Basetimer IO port
    SetPinFunc_TIOA0_0_OUT();
    SetPinFunc_TIOA1_0_OUT();

    // Set requested I/O mode
    Bt_ConfigIOMode(&BT0, BtIoMode5);

    // Set PWM mode
    Bt_SelTimerMode(&BT0, BtPwmMode);
    Bt_SelTimerMode(&BT1, BtPwmMode);

    // Initialize PWM timer
    stcPwmConfig.enPres = PwmPres1Div4;  // PWM clock = 20MHz @ PCLK = 80MHz
    stcPwmConfig.enMode = PwmContinuous;
    stcPwmConfig.enExtTrig = PwmExtTrigRising;
    stcPwmConfig.enOutputMask = PwmOutputNormal;
    stcPwmConfig.enOutputPolarity = PwmPolarityLow;
    stcPwmConfig.enRestartEn = PwmRestartEnable;
    Bt_Pwm_Init(&BT0, &stcPwmConfig);
    Bt_Pwm_Init(&BT1, &stcPwmConfig);

    // Set cycle and duty value
    Bt_Pwm_WriteCycleVal(&BT0, 399); // Cycle = (1+m)*PWM clock cycle = 20us
    Bt_Pwm_WriteDutyVal(&BT0, 199);  // Duty = (1+m)*PWM clock cycle = 10us
    Bt_Pwm_WriteCycleVal(&BT1, 399); // Cycle = (1+m)*PWM clock cycle = 20us
    Bt_Pwm_WriteDutyVal(&BT1, 199);  // Duty = (1+m)*PWM clock cycle = 10us

    // Enable count operatoin
    Bt_Pwm_EnableCount(&BT0);
    Bt_Pwm_EnableCount(&BT1);

    // Enable Simultaneously
    Bt_SetSimultaneousStart(0x0003);
    while(1);
}
#endif

#if (DEMO_MODE == DEMO_BT_IOMODE_6)
/*!
 ******************************************************************************
 ** \brief BT IO mode 6 demo code
 ******************************************************************************
 */
static void DemoBtIoMode6(void)
{
    stc_bt_pwm_config_t stcPwmConfig;
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Base Timer IO Mode 6 \n");
    printf("==================================================\n");
#endif

    // Set Basetimer IO port
    SetPinFunc_TIOA0_0_OUT();
    SetPinFunc_TIOA1_0_OUT();

    // Set requested I/O mode
    Bt_ConfigIOMode(&BT0, BtIoMode6);

    // Set PWM mode
    Bt_SelTimerMode(&BT0, BtPwmMode);
    Bt_SelTimerMode(&BT1, BtPwmMode);

    // Initialize PWM timer
    stcPwmConfig.enPres = PwmPres1Div4;  // PWM clock = 5MHz @ PCLK = 20MHz
    stcPwmConfig.enMode = PwmContinuous;
    stcPwmConfig.enExtTrig = PwmExtTrigRising;
    stcPwmConfig.enOutputMask = PwmOutputNormal;
    stcPwmConfig.enOutputPolarity = PwmPolarityLow;
    stcPwmConfig.enRestartEn = PwmRestartEnable;
    Bt_Pwm_Init(&BT0, &stcPwmConfig);
    Bt_Pwm_Init(&BT1, &stcPwmConfig);

    // Set cycle and duty value
    Bt_Pwm_WriteCycleVal(&BT0, 59999); // Cycle = (1+m)*PWM clock cycle = 12ms
    Bt_Pwm_WriteDutyVal(&BT0, 29999);  // Duty = (1+m)*PWM clock cycle = 6ms
    Bt_Pwm_WriteCycleVal(&BT1, 399); // Cycle = (1+m)*PWM clock cycle = 80us
    Bt_Pwm_WriteDutyVal(&BT1, 199);  // Duty = (1+m)*PWM clock cycle = 40us

    // Enable count operatoin
    Bt_Pwm_EnableCount(&BT0);
    Bt_Pwm_EnableCount(&BT1);

    // Software trigger
    Bt_SetSimultaneousStart(0x0001);

    while(1);

}
#endif

#if (DEMO_MODE == DEMO_BT_IOMODE_7)
/*!
 ******************************************************************************
 ** \brief BT IO mode 7 demo code
 ******************************************************************************
 */
static void DemoBtIoMode7(void)
{
    stc_bt_pwm_config_t stcPwmConfig0, stcPwmConfig1;
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Base Timer IO Mode 7 \n");
    printf("==================================================\n");
#endif

    // Set Basetimer IO port
    SetPinFunc_TIOA0_0_OUT();
    SetPinFunc_TIOA1_0_OUT();

    // Set requested I/O mode
    Bt_ConfigIOMode(&BT0, BtIoMode7);

    // Set PWM mode
    Bt_SelTimerMode(&BT0, BtPwmMode);
    Bt_SelTimerMode(&BT1, BtPwmMode);

    // Initialize PWM timer
    stcPwmConfig0.enPres = PwmPres1Div4;  // PWM clock = 5MHz @ PCLK = 20MHz
    stcPwmConfig0.enMode = PwmContinuous;
    stcPwmConfig0.enExtTrig = PwmExtTrigRising;
    stcPwmConfig0.enOutputMask = PwmOutputNormal;
    stcPwmConfig0.enOutputPolarity = PwmPolarityLow;
    stcPwmConfig0.enRestartEn = PwmRestartEnable;

    stcPwmConfig1.enPres = PwmPres1Div4;  // PWM clock = 5MHz @ PCLK = 20MHz
    stcPwmConfig1.enMode = PwmOneshot;
    stcPwmConfig1.enExtTrig = PwmExtTrigDisable;
    stcPwmConfig1.enOutputMask = PwmOutputNormal;
    stcPwmConfig1.enOutputPolarity = PwmPolarityLow;
    stcPwmConfig1.enRestartEn = PwmRestartEnable;

    Bt_Pwm_Init(&BT0, &stcPwmConfig1);
    Bt_Pwm_Init(&BT1, &stcPwmConfig0);

    // Set cycle and duty value
    Bt_Pwm_WriteCycleVal(&BT0, 59999); // Cycle = (1+m)*PWM clock cycle = 12ms
    Bt_Pwm_WriteDutyVal(&BT0, 29999);  // Duty = (1+m)*PWM clock cycle = 6ms
    Bt_Pwm_WriteCycleVal(&BT1, 399); // Cycle = (1+m)*PWM clock cycle = 80us
    Bt_Pwm_WriteDutyVal(&BT1, 199);  // Duty = (1+m)*PWM clock cycle = 40us

    // Enable count operatoin
    Bt_Pwm_EnableCount(&BT0);
    Bt_Pwm_EnableCount(&BT1);

    // Software trigger
    Bt_Pwm_EnableSwTrig(&BT0); // trigger start after 1.5ms

    while(1);
}
#endif

/*!
 ******************************************************************************
 ** \brief BT IO mode example code
 ******************************************************************************
 */
int32_t main(void)
{
#if (DEMO_MODE == DEMO_BT_IOMODE_1)
    DemoBtIoMode1();
#endif
#if (DEMO_MODE == DEMO_BT_IOMODE_2_3)
    DemoBtIoMode23();
#endif
#if (DEMO_MODE == DEMO_BT_IOMODE_2_8)
    DemoBtIoMode28();
#endif
#if (DEMO_MODE == DEMO_BT_IOMODE_4)
    DemoBtIoMode4();
#endif
#if (DEMO_MODE == DEMO_BT_IOMODE_5)
    DemoBtIoMode5();
#endif
#if (DEMO_MODE == DEMO_BT_IOMODE_6)
    DemoBtIoMode6();
#endif
#if (DEMO_MODE == DEMO_BT_IOMODE_7)
    DemoBtIoMode7();
#endif

    while(1);
}

/*****************************************************************************/
/* END OF FILE */
