/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates both interrupt and polling access mode of Reload timer.
 **
 ** History:
 **   - 2013-05-20  0.0.1  Edison Zhang        First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
#define  USER_BT             BT2
#define  USER_TIMER_SIZE     RtSize32Bit

/*---------------------------------------------------------------------------*/
/* local datatypes                                                           */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
static uint32_t m_u32CntIntTrg, m_u32CntIntUnder;

/*---------------------------------------------------------------------------*/
/* local functions prototypes                                                */
/*---------------------------------------------------------------------------*/
static void RtUnderflowIntHandler(void);
static void RtTrigIntHandler(void);
static void Delay(void);
/*---------------------------------------------------------------------------*/
/* global data                                                               */
/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/
/* global functions                                                          */
/*---------------------------------------------------------------------------*/
/*!
 ******************************************************************************
 ** \brief Reload timer example code
 **
 ** 1. Register initialization
 ** 3. Enable interrupt.
 ** 4. Enable count operation.
 ** 5. Start software trigger.
 ** 6. Disable interrupt.
 ** 7. ReStart software trigger.
 ** 8. Disable count operation.
 ******************************************************************************
 */
int32_t main(void)
{
    stc_bt_rt_config_t stcRtConfig;
    stc_rt_int_sel_t   stcRtIntSel;
    stc_rt_int_cb_t    stcRtIntCallback;
    uint32_t u32Cnt=0;
   
#ifdef DEBUG_PRINT 
    printf("==================================================\n");
    printf("Reload timer Example Program Start \n");
    printf("==================================================\n");
#endif
    // Set BT IO mode  
    Bt_ConfigIOMode(&USER_BT, BtIoMode0);
    
    // Select reload timer function
    Bt_SelTimerMode(&USER_BT, BtRtMode);
    
    // Initialize BT 
    stcRtConfig.enPres = RtPres1Div4; // BT_CLK_DIV_4: T = 0.2us @ PCLK = 20MHz
    stcRtConfig.enSize = USER_TIMER_SIZE;
    stcRtConfig.enMode = RtReload;
    stcRtConfig.enExtTrig = RtExtTiggerDisable;
    stcRtConfig.enOutputPolarity = RtPolarityLow;

    Bt_Rt_Init(&USER_BT, &stcRtConfig);
    
    // Write cycle value
    Bt_Rt_WriteCycleVal(&USER_BT, 19999);  // Cycle = T*(m+1)=4ms @ T = 0.2us
#ifdef DEBUG_PRINT 
    printf("Cycle: %d\n", 19999);
#endif 
    
    // Initialize interrupt 
    stcRtIntSel.bRtTrigInt = 1;
    stcRtIntSel.bRtUnderflowInt = 1;
    stcRtIntCallback.pfnRtTrigIntCallback = RtTrigIntHandler ;
    stcRtIntCallback.pfnRtUnderflowIntCallback = RtUnderflowIntHandler;
    Bt_Rt_EnableInt(&USER_BT, &stcRtIntSel, &stcRtIntCallback);
    
    // Start Reload timer by software trigger    
    Bt_Rt_EnableCount(&USER_BT);
    Bt_Rt_EnableSwTrig(&USER_BT);
   
    // Delay some time
    Delay(); 
    
#ifdef DEBUG_PRINT 
    printf("Interrupt times:\n"); 
    printf("Trigger interrupt  : %d\n", m_u32CntIntTrg);       
    printf("Underflow interrupt: %d\n", m_u32CntIntUnder);  
#endif
   
    Bt_Rt_DisableInt(&USER_BT, &stcRtIntSel);

#ifdef DEBUG_PRINT 
    printf("==================================================\n");
    printf("Disable Interrupt mode, waiting several seconds approximately\n");
#endif
    
    m_u32CntIntTrg = 0;
    m_u32CntIntUnder = 0;     
    
    // Delay some time
    Delay(); 
    
#ifdef DEBUG_PRINT 
    printf("Interrupt times:\n"); 
    printf("Trigger interrupt  : %d\n", m_u32CntIntTrg);       
    printf("Underflow interrupt: %d\n", m_u32CntIntUnder);  
#endif 
     
#ifdef DEBUG_PRINT 
    printf("==================================================\n");
    printf("Enable query mode, waiting several seconds approximately\n");
#endif
 
    m_u32CntIntTrg = 0;
    m_u32CntIntUnder = 0;   
    
    // Restart triggered by software 
    Bt_Rt_EnableSwTrig(&USER_BT);
    
    // Interrupt query mode 
    do 
    {   
        if (PdlSet == Bt_Rt_GetIntFlag(&USER_BT, RtTrigInt))
        {
            Bt_Rt_ClrIntFlag(&USER_BT, RtTrigInt);
            m_u32CntIntTrg++;
        }
        else if (PdlSet == Bt_Rt_GetIntFlag(&USER_BT, RtUnderflowInt))
        {
            Bt_Rt_ClrIntFlag(&USER_BT, RtUnderflowInt);
            m_u32CntIntUnder++;
        }
        u32Cnt++;
    } while(u32Cnt < 2500000ul);
    
    Bt_Rt_DisableCount(&USER_BT);
    
#ifdef DEBUG_PRINT
    printf("Interrupt request times:\n"); 
    printf("Trigger interrupt  : %d\n", m_u32CntIntTrg);       
    printf("Underflow interrupt: %d\n", m_u32CntIntUnder);
#endif   
      
#ifdef DEBUG_PRINT 
    printf("==================================================\n");
    printf("Reload timer Example Program End \n");
    printf("==================================================\n");
#endif
    
    while(1);
}
/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

/*
 ******************************************************************************
 ** \brief RT underflow interrupt handler
 ******************************************************************************
 */
static void RtUnderflowIntHandler(void)
{
    m_u32CntIntUnder++;
}

/*
 ******************************************************************************
 ** \brief RT trigger interrupt handler
 ******************************************************************************
 */
static void RtTrigIntHandler(void)
{
    m_u32CntIntTrg++;
}

/*
 ******************************************************************************
 ** \brief Delay requested secondes approximately
 ******************************************************************************
 */
static void Delay(void)
{
    int32_t u8Cnt1,u8Cnt2;
   
    u8Cnt2 = SystemCoreClock;
     
    u8Cnt1 = 0;
    while (1) 
    {
        u8Cnt1++;
        if (u8Cnt1 > u8Cnt2) 
        {
            break;
        }
    }   
}
