/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved.
*
* This software is owned and published by:
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion
* components. This software is licensed by Spansion to be adapted only
* for use in systems utilizing Spansion components. Spansion shall not be
* responsible for misuse or illegal use of this software for devices not
* supported herein.  Spansion is providing this software "AS IS" and will
* not be responsible for issues arising from incorrect user implementation
* of the software.
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS),
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING,
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED
* WARRANTY OF NONINFRINGEMENT.
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT,
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION,
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA,
* SAVINGS OR PROFITS,
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED
* FROM, THE SOFTWARE.
*
* This software may be replicated in part or whole for the licensed use,
* with the restriction that this Disclaimer and Copyright notice must be
* included with each copy of this software, whether used in part or whole,
* at all times.
*/
/******************************************************************************/
/** \file main.c
 **
 ** Main Module
 **
 ** \brief This example shows how to use the system clock initialization by
 ** PDL functions. For this the definition CLOCK_SETUP must be set to
 ** CLOCK_SETTING_NONE in system_s6xxxx.h.
 ** First the reset cause will be determined, judging if there is a system
 ** clock setup needed or not.
 ** Afterwards the main, pll clock is setup with user configuration value.
 **
 ** History:
 **   - 2014-02-15  1.0  Edison Zhang        First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/**
 ******************************************************************************
 ** \brief  PDL clock initialization
 ******************************************************************************/
void Main_ClkInit(void)
{
  static stc_reset_result_t stcResetCause;
  stc_clk_config_t          stcClockConfig;

  Reset_GetCause(&stcResetCause);

    do
    {
        if (TRUE == stcResetCause.bSoftware)
        {
            // Place code here
        }

        if (TRUE == stcResetCause.bAnomalousFrequency)
        {
            // Place code here
            break;
        }

        if (TRUE == stcResetCause.bClockSupervisor)
        {
            // Place code here
            break;
        }

        if (TRUE == stcResetCause.bHardwareWatchdog)
        {
            // Place code here
            break;
        }

        if (TRUE == stcResetCause.bSoftwareWatchdog)
        {
            // Place code here
            break;
        }

        if (TRUE == stcResetCause.bInitx)
        {
            // Place code here
            break;
        }

        if (TRUE == stcResetCause.bPowerOn)
        {
            // Place code here
        }

        PDL_ZERO_STRUCT(stcClockConfig);

        stcClockConfig.enBaseClkDiv     = BaseClkDiv1;
        stcClockConfig.enAPB0Div        = Apb0Div1;
        stcClockConfig.enAPB1Div        = Apb1Div1;
        stcClockConfig.bAPB1Disable     = FALSE;
        stcClockConfig.enMCOWaitTime    = McoWaitExp117;
        stcClockConfig.enSCOWaitTime    = ScoWaitExp10;
        stcClockConfig.enPLLOWaitTime   = PlloWaitExp19;
        stcClockConfig.u8PllK           = 1;
        stcClockConfig.u8PllN           = 10;
        stcClockConfig.u8PllM           = 2;

        /* Initialize clock */
        if(Ok != Clk_Init(&stcClockConfig))
        {
            while(1);
        }

        Clk_EnableMainClock(TRUE);                          // ... now enable Main oscillator
        Clk_EnablePllClock(TRUE);                           // ... now enable PLL oscillator

        Clk_SetSource(ClkPll);                              // ... and transit to Main Clock

    }while(0);
}

/**
 ******************************************************************************
 ** \brief  Main function of PDL
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{
  Main_ClkInit();

  while(1)
  {}
}
