/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved.
*
* This software is owned and published by:
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion
* components. This software is licensed by Spansion to be adapted only
* for use in systems utilizing Spansion components. Spansion shall not be
* responsible for misuse or illegal use of this software for devices not
* supported herein.  Spansion is providing this software "AS IS" and will
* not be responsible for issues arising from incorrect user implementation
* of the software.
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS),
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING,
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED
* WARRANTY OF NONINFRINGEMENT.
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT,
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION,
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA,
* SAVINGS OR PROFITS,
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED
* FROM, THE SOFTWARE.
*
* This software may be replicated in part or whole for the licensed use,
* with the restriction that this Disclaimer and Copyright notice must be
* included with each copy of this software, whether used in part or whole,
* at all times.
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates how to access to AT24C02 with I2C interface.
 **
 ** Set the "PDL_UTILITY_ENABLE_I2C_POLLING_AT24CXX" to "PDL_ON" in the 
 ** pdl_user.h to use I2C polling method
 ** Set the "PDL_UTILITY_ENABLE_I2C_INT_AT24CXX" to in the pdl_user.h to 
 ** use I2C interrupt method
 **
 ** Never set both definitions to "PDL_ON" at the same time.
 **
 ** History:
 **   - 2014-01-10  1.0  Edison Zhang        First release version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/******************************************************************************/
/* Global variable                                                            */
/******************************************************************************/
uint8_t au8WrBuf[256], au8RdBuf[256]; 
   
/**
 ******************************************************************************
 ** \brief  Main function of PDL
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{  
    uint8_t u8DevAddr;
    uint8_t u8i;
    uint16_t u16j;
    
    u8DevAddr = AT24CXX_7BIT_DEVICE_ADDR;
    
    At24cxx_Init();
    
    /* Byte write and Random read */
    At24cxx_ByteWrite(u8DevAddr, 0x0000u, 0x55);
    At24cxx_Delayms(10);
    
    At24cxx_ByteWrite(u8DevAddr, 0x0080u, 0xAA);
    At24cxx_Delayms(10);
      
    At24cxx_RandomRead(u8DevAddr, 0x0000u, &au8RdBuf[0]);
    At24cxx_Delayms(10);
    At24cxx_RandomRead(u8DevAddr, 0x0080u, &au8RdBuf[1]);
    At24cxx_Delayms(10);
    
    /* Page write, Random read and squence read */
    for(u8i=0;u8i<AT24CXX_PAGE_SIZE;u8i++)
    {
        au8WrBuf[u8i] = 0x12+0x22*u8i;
    }
    
    At24cxx_PageWrite(u8DevAddr, 0x0008u, au8WrBuf, AT24CXX_PAGE_SIZE);
    At24cxx_Delayms(10);

    At24cxx_RandomRead(u8DevAddr, 0x0008u, &au8RdBuf[0]); // read first data 
    At24cxx_Delayms(10);
    
    At24cxx_SequentialRead(u8DevAddr, &au8RdBuf[1], (AT24CXX_PAGE_SIZE-1)); // read following data
    At24cxx_Delayms(10);
    
    /* Page write (only write 4bytes, from random address), Random read and squence read */
    for(u8i=0;u8i<4;u8i++)
    {
        au8WrBuf[u8i] = 0x12+0x11*u8i;
    }
    At24cxx_PageWrite(u8DevAddr, 0x0015u, au8WrBuf, 4);
    At24cxx_Delayms(10);
    
    At24cxx_RandomRead(u8DevAddr, 0x0015u, au8RdBuf); // read first data
    At24cxx_Delayms(10);
    
    At24cxx_SequentialRead(u8DevAddr, &au8RdBuf[1], 3); // read following data
    At24cxx_Delayms(10);
      
    /* Write whole chip of AT2402C, Random read and squence read  */
    for(u16j=0;u16j<256;u16j++)
    {
        au8WrBuf[u16j] = u16j;
    }
    
    for(u8i = 0; u8i<32; u8i++)
    {
        At24cxx_PageWrite(u8DevAddr, u8i*AT24CXX_PAGE_SIZE, &au8WrBuf[u8i*AT24CXX_PAGE_SIZE], AT24CXX_PAGE_SIZE);
        At24cxx_Delayms(10);
    }
    
    At24cxx_RandomRead(u8DevAddr, 0x0000u, &au8RdBuf[0]); // read first data 
    At24cxx_Delayms(10);
    
    At24cxx_SequentialRead(u8DevAddr, &au8RdBuf[1], 255); // read following data
    At24cxx_Delayms(10);
    
    At24cxx_CurrentAddrRead(u8DevAddr, &au8RdBuf[255]);  // should be the first data (at 0x0000)
    
    /* Erase whole chip */
    for(u16j = 0; u16j<256; u16j++)
    {
        At24cxx_ByteWrite(u8DevAddr, u16j, 0x00u);
        At24cxx_Delayms(10);
    }
    
    while(1)
    {}
}
