/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates single channel mode of OCU.
 ** Ch.0 and Ch.1 implement 1-change mode
 ** Ch.2 and Ch.3 implement active-high mode
 ** Ch.4 and Ch.5 implement active-low mode
 **
 ** History:
 **   - 2014-01-07  0.0.1  Edison Zhang        First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
/*  FRT module  */
// MFT channel
#define USER_MFT_FRT          MFT0_FRT
// FRT 0
#define USER_FRT_CH0          MFT_FRT_CH0
#define USER_FRT_CH0_MOD      FrtUpCount
// FRT 1
#define USER_FRT_CH1          MFT_FRT_CH1 
#define USER_FRT_CH1_MOD      FrtUpDownCount
// FRT cycle
#define FRT_CYCLE       60000  

/*  OCU module  */
// MFT channel
#define USER_MFT_OCU        MFT0_OCU
// OCU channel
#define USER_OCU_CH0    MFT_OCU_CH0
#define USER_OCU_CH1    MFT_OCU_CH1
#define USER_OCU_CH2    MFT_OCU_CH2
#define USER_OCU_CH3    MFT_OCU_CH3
#define USER_OCU_CH4    MFT_OCU_CH4
#define USER_OCU_CH5    MFT_OCU_CH5
// OCU compare value
#define USER_OCU_CH0_CMPVAL1 (5000)
#define USER_OCU_CH1_CMPVAL1 (15000)
#define USER_OCU_CH2_CMPVAL1 (20000)
#define USER_OCU_CH3_CMPVAL1 (30000)
#define USER_OCU_CH4_CMPVAL1 (40000)
#define USER_OCU_CH5_CMPVAL1 (50000)

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
static uint8_t m_u8OcuMatchFlag[6] = {0};

/*---------------------------------------------------------------------------*/
/* local functions prototypes                                                */
/*---------------------------------------------------------------------------*/
/**
 ******************************************************************************
 ** \brief  OCU channel 0 interrupt callback function   .
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
static void Ocu0IntCallback(void)
{
    m_u8OcuMatchFlag[0] = 1;
}

/**
 ******************************************************************************
 ** \brief  OCU channel 1 interrupt callback function  .
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
static void Ocu1IntCallback(void)
{
    m_u8OcuMatchFlag[1] = 1;
}

/**
 ******************************************************************************
 ** \brief  OCU channel 2 interrupt callback function   .
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
static void Ocu2IntCallback(void)
{
    m_u8OcuMatchFlag[2] = 1;
}

/**
 ******************************************************************************
 ** \brief  OCU channel 3 interrupt callback function   .
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
static void Ocu3IntCallback(void)
{
    m_u8OcuMatchFlag[3] = 1;
}

/**
 ******************************************************************************
 ** \brief  OCU channel 4 interrupt callback function   .
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
static void Ocu4IntCallback(void)
{
    m_u8OcuMatchFlag[4] = 1;
}

/**
 ******************************************************************************
 ** \brief  OCU channel 5 interrupt callback function   .
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
static void Ocu5IntCallback(void)
{
    m_u8OcuMatchFlag[5] = 1;
}

/**
 ******************************************************************************
 ** \brief  Init user free run timer   .
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
static void InitFrt(void)
{
    stc_mft_frt_config_t stcFrtConfig0; //channel 1 configure structure
    stc_mft_frt_config_t stcFrtConfig1; //channel 1 configure structure
    
    //configure the FRT channel 0 parameter
    stcFrtConfig0.enFrtMode = USER_FRT_CH0_MOD;
    stcFrtConfig0.enFrtClockDiv = FrtPclkDiv1024;
    stcFrtConfig0.bEnExtClock = FALSE;
    stcFrtConfig0.bEnBuffer = TRUE;
    
    //configure the FRT channel 1 parameter
    stcFrtConfig1.enFrtMode = USER_FRT_CH1_MOD;
    stcFrtConfig1.enFrtClockDiv = FrtPclkDiv512;
    stcFrtConfig1.bEnExtClock = FALSE;
    stcFrtConfig1.bEnBuffer = TRUE;

    //stop frt
    Mft_Frt_Stop(&USER_MFT_FRT,USER_FRT_CH0);
    Mft_Frt_Stop(&USER_MFT_FRT,USER_FRT_CH1);
    //init frt module
    Mft_Frt_Init(&USER_MFT_FRT,USER_FRT_CH0,&stcFrtConfig0);
    Mft_Frt_Init(&USER_MFT_FRT,USER_FRT_CH1,&stcFrtConfig1);
    //set frt cycle value
    Mft_Frt_SetCountCycle(&USER_MFT_FRT,USER_FRT_CH0,FRT_CYCLE);
    Mft_Frt_SetCountCycle(&USER_MFT_FRT,USER_FRT_CH1,FRT_CYCLE);
}

/**
 ******************************************************************************
 ** \brief  Init user output compare unit  .
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
static void InitOcu(void)
{
    stc_mft_ocu_config_t stcOcuConfig0;
    stc_mft_ocu_config_t stcOcuConfig1;
    stc_mft_ocu_config_t stcOcuConfig2;
    
    stc_even_compare_config_t stcCh0CompareConfig, stcCh2CompareConfig, stcCh4CompareConfig;
    stc_odd_compare_config_t  stcCh1CompareConfig, stcCh3CompareConfig, stcCh5CompareConfig;
    
    /*configure the OCU parameter*/
    stcOcuConfig0.enFrtConnect = Frt0ToOcu;
    stcOcuConfig0.bFm4 = TRUE;
    stcOcuConfig0.enStatePin = RtLowLevel;
    stcOcuConfig0.bCmod = FALSE;
    stcOcuConfig0.u8Mod = 0;
    stcOcuConfig0.enOccpBufMode = OccpBufDisable;
    stcOcuConfig0.enOcseBufMode = OcseBufDisable;
    
    stcOcuConfig1.enFrtConnect = Frt1ToOcu;
    stcOcuConfig1.bFm4 = TRUE;
    stcOcuConfig1.enStatePin = RtLowLevel;
    stcOcuConfig1.bCmod = FALSE;
    stcOcuConfig1.u8Mod = 0;
    stcOcuConfig1.enOccpBufMode = OccpBufTrsfByFrtZero;
    stcOcuConfig1.enOcseBufMode = OcseBufDisable;
    
    stcOcuConfig2.enFrtConnect = Frt1ToOcu;
    stcOcuConfig2.bFm4 = TRUE;
    stcOcuConfig2.enStatePin = RtHighLevel;
    stcOcuConfig2.bCmod = FALSE;
    stcOcuConfig2.u8Mod = 0;
    stcOcuConfig2.enOccpBufMode = OccpBufTrsfByFrtZero;
    stcOcuConfig2.enOcseBufMode = OcseBufDisable;

    //init OCU
    Mft_Ocu_Init(&USER_MFT_OCU,USER_OCU_CH0,&stcOcuConfig0);
    Mft_Ocu_Init(&USER_MFT_OCU,USER_OCU_CH1,&stcOcuConfig0);
    
    Mft_Ocu_Init(&USER_MFT_OCU,USER_OCU_CH2,&stcOcuConfig1);
    Mft_Ocu_Init(&USER_MFT_OCU,USER_OCU_CH3,&stcOcuConfig1);
    
    Mft_Ocu_Init(&USER_MFT_OCU,USER_OCU_CH4,&stcOcuConfig2);
    Mft_Ocu_Init(&USER_MFT_OCU,USER_OCU_CH5,&stcOcuConfig2);

    // OCU1&OCU0: 1 change mode
    // OCSE0:        0x00000FFF
    // OCSE1:        0x0FF00FFF
    stcCh0CompareConfig.enFrtZeroEvenMatchEvenChRtStatus = RtOutputReverse;
    stcCh0CompareConfig.enFrtZeroEvenNotMatchEvenChRtStatus = RtOutputHold;
    stcCh0CompareConfig.enFrtUpCntEvenMatchEvenChRtStatus = RtOutputReverse;
    stcCh0CompareConfig.enFrtPeakEvenMatchEvenChRtStatus = RtOutputReverse;
    stcCh0CompareConfig.enFrtPeakEvenNotMatchEvenChStatus = RtOutputHold;
    stcCh0CompareConfig.enFrtDownCntEvenMatchEvenChRtStatus = RtOutputReverse;
    stcCh0CompareConfig.enIopFlagWhenFrtZeroEvenMatch = IopFlagSet;
    stcCh0CompareConfig.enIopFlagWhenFrtUpCntEvenMatch = IopFlagSet;
    stcCh0CompareConfig.enIopFlagWhenFrtPeakEvenMatch = IopFlagSet;
    stcCh0CompareConfig.enIopFlagWhenFrtDownCntEvenMatch = IopFlagSet;
    
    stcCh1CompareConfig.enFrtZeroOddMatchEvenMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtZeroOddMatchEvenNotMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtZeroOddNotMatchEvenMatchOddChRtStatus = RtOutputHold;
    stcCh1CompareConfig.enFrtZeroOddNotMatchEvenNotMatchOddChRtStatus = RtOutputHold;
    stcCh1CompareConfig.enFrtUpCntOddMatchEvenMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtUpCntOddMatchEvenNotMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtUpCntOddNotMatchEvenMatchOddChRtStatus = RtOutputHold;
    stcCh1CompareConfig.enFrtPeakOddMatchEvenMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtPeakOddMatchEvenNotMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtPeakOddNotMatchEvenMatchOddChRtStatus = RtOutputHold;
    stcCh1CompareConfig.enFrtPeakOddNotMatchEvenNotMatchOddChRtStatus = RtOutputHold;
    stcCh1CompareConfig.enFrtDownOddMatchEvenMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtDownOddMatchEvenNotMatchOddChRtStatus = RtOutputReverse;
    stcCh1CompareConfig.enFrtDownOddNotMatchEvenMatchOddChRtStatus = RtOutputHold;
    stcCh1CompareConfig.enIopFlagWhenFrtZeroOddMatch = IopFlagSet;
    stcCh1CompareConfig.enIopFlagWhenFrtUpCntOddMatch = IopFlagSet;
    stcCh1CompareConfig.enIopFlagWhenFrtPeakOddMatch = IopFlagSet;
    stcCh1CompareConfig.enIopFlagWhenFrtDownCntOddMatch = IopFlagSet;
    
    Mft_Ocu_SetEvenChCompareMode(&USER_MFT_OCU, USER_OCU_CH0, &stcCh0CompareConfig);
    Mft_Ocu_SetOddChCompareMode(&USER_MFT_OCU, USER_OCU_CH1, &stcCh1CompareConfig);
    
    // OCU3&OCU2: Active-High mode
    // OCSE2:        0x0000852D
    // OCSE3:        0x8520852D
    stcCh2CompareConfig.enFrtZeroEvenMatchEvenChRtStatus = RtOutputHigh;
    stcCh2CompareConfig.enFrtZeroEvenNotMatchEvenChRtStatus = RtOutputLow;
    stcCh2CompareConfig.enFrtUpCntEvenMatchEvenChRtStatus = RtOutputHigh;
    stcCh2CompareConfig.enFrtPeakEvenMatchEvenChRtStatus = RtOutputHold;
    stcCh2CompareConfig.enFrtPeakEvenNotMatchEvenChStatus = RtOutputHold;
    stcCh2CompareConfig.enFrtDownCntEvenMatchEvenChRtStatus = RtOutputLow;
    stcCh2CompareConfig.enIopFlagWhenFrtZeroEvenMatch = IopFlagSet;
    stcCh2CompareConfig.enIopFlagWhenFrtUpCntEvenMatch = IopFlagSet;
    stcCh2CompareConfig.enIopFlagWhenFrtPeakEvenMatch = IopFlagHold;
    stcCh2CompareConfig.enIopFlagWhenFrtDownCntEvenMatch = IopFlagSet;
    
    stcCh3CompareConfig.enFrtZeroOddMatchEvenMatchOddChRtStatus = RtOutputHigh;
    stcCh3CompareConfig.enFrtZeroOddMatchEvenNotMatchOddChRtStatus = RtOutputHigh;
    stcCh3CompareConfig.enFrtZeroOddNotMatchEvenMatchOddChRtStatus = RtOutputLow;
    stcCh3CompareConfig.enFrtZeroOddNotMatchEvenNotMatchOddChRtStatus = RtOutputLow;
    stcCh3CompareConfig.enFrtUpCntOddMatchEvenMatchOddChRtStatus = RtOutputHigh;
    stcCh3CompareConfig.enFrtUpCntOddMatchEvenNotMatchOddChRtStatus = RtOutputHigh;
    stcCh3CompareConfig.enFrtUpCntOddNotMatchEvenMatchOddChRtStatus = RtOutputHold;
    stcCh3CompareConfig.enFrtPeakOddMatchEvenMatchOddChRtStatus = RtOutputHold;
    stcCh3CompareConfig.enFrtPeakOddMatchEvenNotMatchOddChRtStatus = RtOutputHold;
    stcCh3CompareConfig.enFrtPeakOddNotMatchEvenMatchOddChRtStatus = RtOutputHold;
    stcCh3CompareConfig.enFrtPeakOddNotMatchEvenNotMatchOddChRtStatus = RtOutputHold;
    stcCh3CompareConfig.enFrtDownOddMatchEvenMatchOddChRtStatus = RtOutputLow;
    stcCh3CompareConfig.enFrtDownOddMatchEvenNotMatchOddChRtStatus = RtOutputLow;
    stcCh3CompareConfig.enFrtDownOddNotMatchEvenMatchOddChRtStatus = RtOutputHold;
    stcCh3CompareConfig.enIopFlagWhenFrtZeroOddMatch = IopFlagSet;
    stcCh3CompareConfig.enIopFlagWhenFrtUpCntOddMatch = IopFlagSet;
    stcCh3CompareConfig.enIopFlagWhenFrtPeakOddMatch = IopFlagHold;
    stcCh3CompareConfig.enIopFlagWhenFrtDownCntOddMatch = IopFlagSet;
    
    Mft_Ocu_SetEvenChCompareMode(&USER_MFT_OCU, USER_OCU_CH2, &stcCh2CompareConfig);
    Mft_Ocu_SetOddChCompareMode(&USER_MFT_OCU, USER_OCU_CH3, &stcCh3CompareConfig);
    
    // OCU5&OCU4: Active-Low mode
    // OCSE4        0x00004A1D
    // OCSE5        0x4A104A1D
    stcCh4CompareConfig.enFrtZeroEvenMatchEvenChRtStatus = RtOutputLow;
    stcCh4CompareConfig.enFrtZeroEvenNotMatchEvenChRtStatus = RtOutputHigh;
    stcCh4CompareConfig.enFrtUpCntEvenMatchEvenChRtStatus = RtOutputLow;
    stcCh4CompareConfig.enFrtPeakEvenMatchEvenChRtStatus = RtOutputHold;
    stcCh4CompareConfig.enFrtPeakEvenNotMatchEvenChStatus = RtOutputHold;
    stcCh4CompareConfig.enFrtDownCntEvenMatchEvenChRtStatus = RtOutputHigh;
    stcCh4CompareConfig.enIopFlagWhenFrtZeroEvenMatch = IopFlagSet;
    stcCh4CompareConfig.enIopFlagWhenFrtUpCntEvenMatch = IopFlagSet;
    stcCh4CompareConfig.enIopFlagWhenFrtPeakEvenMatch = IopFlagHold;
    stcCh4CompareConfig.enIopFlagWhenFrtDownCntEvenMatch = IopFlagSet;
    
    stcCh5CompareConfig.enFrtZeroOddMatchEvenMatchOddChRtStatus = RtOutputLow;
    stcCh5CompareConfig.enFrtZeroOddMatchEvenNotMatchOddChRtStatus = RtOutputLow;
    stcCh5CompareConfig.enFrtZeroOddNotMatchEvenMatchOddChRtStatus = RtOutputHigh;
    stcCh5CompareConfig.enFrtZeroOddNotMatchEvenNotMatchOddChRtStatus = RtOutputHigh;
    stcCh5CompareConfig.enFrtUpCntOddMatchEvenMatchOddChRtStatus = RtOutputLow;
    stcCh5CompareConfig.enFrtUpCntOddMatchEvenNotMatchOddChRtStatus = RtOutputLow;
    stcCh5CompareConfig.enFrtUpCntOddNotMatchEvenMatchOddChRtStatus = RtOutputHold;
    stcCh5CompareConfig.enFrtPeakOddMatchEvenMatchOddChRtStatus = RtOutputHold;
    stcCh5CompareConfig.enFrtPeakOddMatchEvenNotMatchOddChRtStatus = RtOutputHold;
    stcCh5CompareConfig.enFrtPeakOddNotMatchEvenMatchOddChRtStatus = RtOutputHold;
    stcCh5CompareConfig.enFrtPeakOddNotMatchEvenNotMatchOddChRtStatus = RtOutputHold;
    stcCh5CompareConfig.enFrtDownOddMatchEvenMatchOddChRtStatus = RtOutputHigh;
    stcCh5CompareConfig.enFrtDownOddMatchEvenNotMatchOddChRtStatus = RtOutputHigh;
    stcCh5CompareConfig.enFrtDownOddNotMatchEvenMatchOddChRtStatus = RtOutputHold;
    stcCh5CompareConfig.enIopFlagWhenFrtZeroOddMatch = IopFlagSet;
    stcCh5CompareConfig.enIopFlagWhenFrtUpCntOddMatch = IopFlagSet;
    stcCh5CompareConfig.enIopFlagWhenFrtPeakOddMatch = IopFlagHold;
    stcCh5CompareConfig.enIopFlagWhenFrtDownCntOddMatch = IopFlagSet;
    
    Mft_Ocu_SetEvenChCompareMode(&USER_MFT_OCU, USER_OCU_CH4, &stcCh4CompareConfig);
    Mft_Ocu_SetOddChCompareMode(&USER_MFT_OCU, USER_OCU_CH5, &stcCh5CompareConfig);
    
    //write OCU compare value
    Mft_Ocu_WriteOccp(&USER_MFT_OCU,USER_OCU_CH0,USER_OCU_CH0_CMPVAL1);
    Mft_Ocu_WriteOccp(&USER_MFT_OCU,USER_OCU_CH1,USER_OCU_CH1_CMPVAL1);
    Mft_Ocu_WriteOccp(&USER_MFT_OCU,USER_OCU_CH2,USER_OCU_CH2_CMPVAL1);
    Mft_Ocu_WriteOccp(&USER_MFT_OCU,USER_OCU_CH3,USER_OCU_CH3_CMPVAL1);
    Mft_Ocu_WriteOccp(&USER_MFT_OCU,USER_OCU_CH4,USER_OCU_CH4_CMPVAL1);
    Mft_Ocu_WriteOccp(&USER_MFT_OCU,USER_OCU_CH5,USER_OCU_CH5_CMPVAL1);

    //Enable OCU interrupt
    Mft_Ocu_EnableInt(&USER_MFT_OCU,USER_OCU_CH0,Ocu0IntCallback);
    Mft_Ocu_EnableInt(&USER_MFT_OCU,USER_OCU_CH1,Ocu1IntCallback);
    Mft_Ocu_EnableInt(&USER_MFT_OCU,USER_OCU_CH2,Ocu2IntCallback);
    Mft_Ocu_EnableInt(&USER_MFT_OCU,USER_OCU_CH3,Ocu3IntCallback);
    Mft_Ocu_EnableInt(&USER_MFT_OCU,USER_OCU_CH4,Ocu4IntCallback);
    Mft_Ocu_EnableInt(&USER_MFT_OCU,USER_OCU_CH5,Ocu5IntCallback);
}
/*---------------------------------------------------------------------------*/
/* global data                                                               */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global functions                                                          */
/*---------------------------------------------------------------------------*/

/**
 ******************************************************************************
 ** \brief  Main function of project for S6E1A1 series.
 **
 ** \return uint32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("          OCU_CH_SINGLE Sample Start \n");
    printf("OCU0 and OCU1 connect to FRT0 and running with 1-change mode\n");
    printf("OCU2 and OCU3 connect to FRT1 and running with Active-High mode\n");
    printf("OCU4 and OCU5 connect to FRT1 and running with Active-Low mode\n");
    printf("==================================================\n");
#endif
    //init FRT
    InitFrt();
    //init OCU
    InitOcu();

    //enable OCU operation
    Mft_Ocu_EnableOperation(&USER_MFT_OCU,USER_OCU_CH0);
    Mft_Ocu_EnableOperation(&USER_MFT_OCU,USER_OCU_CH1);
    Mft_Ocu_EnableOperation(&USER_MFT_OCU,USER_OCU_CH2);
    Mft_Ocu_EnableOperation(&USER_MFT_OCU,USER_OCU_CH3);
    Mft_Ocu_EnableOperation(&USER_MFT_OCU,USER_OCU_CH4);
    Mft_Ocu_EnableOperation(&USER_MFT_OCU,USER_OCU_CH5);
    
    //start FRT
    Mft_Frt_Start(&USER_MFT_FRT,USER_FRT_CH0);
    Mft_Frt_Start(&USER_MFT_FRT,USER_FRT_CH1);

    while(1)
    {
        if(m_u8OcuMatchFlag[0] == 1)
        { 
            m_u8OcuMatchFlag[0] = 0;
        #ifdef DEBUG_PRINT
            printf("OCU%d match occurs! (FRT0 counts at %d)\n", USER_OCU_CH0, USER_OCU_CH0_CMPVAL1);
        #endif
        }

        if(m_u8OcuMatchFlag[1] == 1)
        {
            m_u8OcuMatchFlag[1] = 0;
        #ifdef DEBUG_PRINT
            printf("OCU%d match occurs! (FRT0 counts at %d)\n", USER_OCU_CH1, USER_OCU_CH1_CMPVAL1);
        #endif
        }
        
        if(m_u8OcuMatchFlag[2] == 1)
        {
            m_u8OcuMatchFlag[2] = 0;
        #ifdef DEBUG_PRINT
            printf("OCU%d match occurs! (FRT1 counts at %d)\n", USER_OCU_CH2, USER_OCU_CH2_CMPVAL1);
        #endif
        }
        
        if(m_u8OcuMatchFlag[3] == 1)
        {
            m_u8OcuMatchFlag[3] = 0;
        #ifdef DEBUG_PRINT
            printf("OCU%d match occurs! (FRT1 counts at %d)\n", USER_OCU_CH3, USER_OCU_CH3_CMPVAL1);
        #endif
        }
        
        if(m_u8OcuMatchFlag[4] == 1)
        {
            m_u8OcuMatchFlag[4] = 0;
        #ifdef DEBUG_PRINT
            printf("OCU%d match occurs! (FRT1 counts at %d)\n", USER_OCU_CH4, USER_OCU_CH4_CMPVAL1);
        #endif
        }
        
        if(m_u8OcuMatchFlag[5] == 1)
        {
            m_u8OcuMatchFlag[5] = 0;
        #ifdef DEBUG_PRINT
            printf("OCU%d match occurs! (FRT1 counts at %d)\n", USER_OCU_CH5, USER_OCU_CH5_CMPVAL1);
        #endif
        }

    }
}
