/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates watch counter interrupt .
 ** IO polling wave can be observed at P61 during demonstration process.
 **
 ** History:
 **   - 2014-02-20  0.0.1  Edison Zhang        First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/*---------------------------------------------------------------------------*/
/* local functions prototypes                                                */
/*---------------------------------------------------------------------------*/
static void IoInit(void);
static void IoPolling(void);
static void Delay(void);
static void WcDemoUseMainOsc(void);
static void WcDemoUseSubOsc(void);
static void WcDemoUseMainCr(void);
static void WcDemoUseSubCr(void);

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
static uint16_t m_u16ValidFlag = 0;

/*---------------------------------------------------------------------------*/
/* global functions                                                          */
/*---------------------------------------------------------------------------*/

/*!
 ******************************************************************************
 ** \brief Watch counter interrupt example code
 **
 ** 1. Demo Watch counter with Main Osc.
 ** 2. Demo Watch counter with Sub Osc.
 ** 3. Demo Watch counter with Main Cr.
 ** 4. Demo Watch counter with Sub Osc.
 ******************************************************************************
 */
int32_t main(void)
{
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Watch Counter Interrupt Example Program Start \n");
    printf("==================================================\n");
#endif
    WcDemoUseMainOsc();
    WcDemoUseSubOsc();
    WcDemoUseMainCr();
    WcDemoUseSubCr();
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Watch Counter Interrupt Example Program End \n");
    printf("==================================================\n");
#endif
    while(1);
}

/*!
 ******************************************************************************
 ** \brief  Initialize the IO (P38) to be tested
 **
 ** \param  none
 **
 ** \return none
 ******************************************************************************
 */
void IoInit(void)
{
    Gpio1pin_InitOut(GPIO1PIN_P61, Gpio1pin_InitDirectionOutput);
    return;
}

/*!
 ******************************************************************************
 ** \brief  Polling the IO (P38)
 **
 ** \param  none
 **
 ** \return none
 ******************************************************************************
 */
void IoPolling(void)
{
    /* ignore first trigger in order to synchronize with PCLK*/
    if(!m_u16ValidFlag)
    {
        m_u16ValidFlag++;
        return;
    }
    m_u16ValidFlag++;
    Gpio1pin_Put(GPIO1PIN_P61, (m_u16ValidFlag%2));
    return;
}

/*
 ******************************************************************************
 ** \brief Delay requested seconds approximately
 ******************************************************************************
 */
static void Delay(void)
{
    int32_t u8Cnt1,u8Cnt2;

    u8Cnt2 = SystemCoreClock*2;

    u8Cnt1 = 0;
    while (1)
    {
        u8Cnt1++;
        if (u8Cnt1 > u8Cnt2)
        {
            break;
        }
    }
}
/*!
 ******************************************************************************
 ** \brief Watch counter interrupt example code(Use Main OSC)
 **
 ** 1. Initialization.
 ** 2. Set counter clock as MainOsc.
 ** 3. Enable interrupt.
 ** 4. Enable count operation and delay some time.
 ** 5. Stop counter.
 ** 6. Disable interrupt.
 ******************************************************************************
 */
static void WcDemoUseMainOsc(void)
{
    stc_wc_config_t stcWcConfig;
    stc_wc_pres_clk_t stcWcPresClk;

    // Initialize GPIO for watch counter demo
    IoInit();
#ifdef DEBUG_PRINT
    printf("1st demo\n");
    printf("Source clock = Main OSC \n");
    m_u16ValidFlag = 0;
#endif
    // Select watch counter prescaler source clock as Main OSC
    stcWcPresClk.enInputClk = WcPresInClkMainOsc;
    // Select watch counter source clock WCCK0
    stcWcConfig.enCntClk = WcCntClkWcck0;
    // Set watch counter reload value
    stcWcConfig.u8ReloadValue = 1;
    // Set counter value, 2^20/4000000 = 0.26S, 1 step = 0.26S
    stcWcPresClk.enOutputClk = WcPresOutClkArray6;

    Wc_Pres_SelClk(&stcWcPresClk);
    Wc_Pres_EnableDiv();

    // Enable watch counter interrupt
    Wc_EnableInt(IoPolling);
    Wc_Init(&stcWcConfig);
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Start counter, waiting several seconds approximately\n");
#endif
    // Start counter
    Wc_EnableCount();
    // Delay some time
    Delay();
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Stop counter, waiting several seconds approximately\n");
    printf("\n");
#endif
    // Stop counter
    Wc_DisableCount();
    // Disable watch counter interrupt
    Wc_DisableInt();
    // Delay some time
    Delay();
}

/*!
 ******************************************************************************
 ** \brief Watch counter interrupt example code(Use Sub OSC)
 **
 ** 1. Initialization.
 ** 2. Set counter clock as SubOsc.
 ** 3. Enable interrupt.
 ** 4. Enable count operation and delay some time.
 ** 5. Stop counter.
 ** 6. Disable interrupt.
 ******************************************************************************
 */
static void WcDemoUseSubOsc(void)
{
    stc_wc_config_t stcWcConfig;
    stc_wc_pres_clk_t stcWcPresClk;

    // Initialize GPIO for watch counter demo
    IoInit();
#ifdef DEBUG_PRINT
    printf("2nd demo\n"); // TODO_Lv
    printf("Source clock = Sub OSC \n");
    m_u16ValidFlag = 0;
#endif
    // Select watch counter prescaler source clock as Sub OSC
    stcWcPresClk.enInputClk = WcPresInClkSubOsc;
    // Select watch counter source clock WCCK3
    stcWcConfig.enCntClk = WcCntClkWcck3;
    // Set watch counter reload value
    stcWcConfig.u8ReloadValue = 1;
    // Set counter value, 2^15/32768, 1 step = 1S
    stcWcPresClk.enOutputClk = WcPresOutClkArray0;

    Wc_Pres_SelClk(&stcWcPresClk);
    Wc_Pres_EnableDiv();

    // Enable watch counter interrupt
    Wc_EnableInt(IoPolling);
    Wc_Init(&stcWcConfig);
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Start counter, waiting several seconds approximately\n");
#endif
    // Start counter
    Wc_EnableCount();
    // Delay some time
    Delay();
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Stop counter, waiting several seconds approximately\n");
    printf("\n");
#endif
    // Stop counter
    Wc_DisableCount();
    // Disable watch counter interrupt
    Wc_DisableInt();
    // Delay some time
    Delay();
}
/*!
 ******************************************************************************
 ** \brief Watch counter interrupt example code(Use Main CR)
 **
 ** 1. Initialization.
 ** 2. Set counter clock as MainCr.
 ** 3. Enable interrupt.
 ** 4. Enable count operation and delay some time.
 ** 5. Stop counter.
 ** 6. Disable interrupt.
 ******************************************************************************
 */
static void WcDemoUseMainCr(void)
{
    stc_wc_config_t stcWcConfig;
    stc_wc_pres_clk_t stcWcPresClk;

    // Initialize GPIO for watch counter demo
    IoInit();
#ifdef DEBUG_PRINT
    printf("3rd demo\n");
    printf("Source clock = High CR \n");
    m_u16ValidFlag = 0;
#endif
    // Select watch counter prescaler source clock as High CR
    stcWcPresClk.enInputClk = WcPresInClkHighCr;
    // Select watch counter source clock WCCK0
    stcWcConfig.enCntClk = WcCntClkWcck0;
    // Set watch counter reload value
    stcWcConfig.u8ReloadValue = 1;
    // Set counter value, 2^20/4000000 = 0.26S, 1 step = 0.26S
    stcWcPresClk.enOutputClk = WcPresOutClkArray6;

    Wc_Pres_SelClk(&stcWcPresClk);
    Wc_Pres_EnableDiv();

    // Enable watch counter interrupt
    Wc_EnableInt(IoPolling);
    Wc_Init(&stcWcConfig);
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Start counter, waiting several seconds approximately\n");
#endif
    // Start counter
    Wc_EnableCount();
    // Delay some time
    Delay();
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Stop counter, waiting several seconds approximately\n");
    printf("\n");
#endif
    // Stop counter
    Wc_DisableCount();
    // Disable watch counter interrupt
    Wc_DisableInt();
    // Delay some time
    Delay();
}
/*!
 ******************************************************************************
 ** \brief Watch counter interrupt example code(Use Sub Cr)
 **
 ** 1. Initialization.
 ** 2. Set counter clock as SubCr.
 ** 3. Enable interrupt.
 ** 4. Enable count operation and delay some time.
 ** 5. Stop counter.
 ** 6. Disable interrupt.
 ******************************************************************************
 */
static void WcDemoUseSubCr(void)
{
    stc_wc_config_t stcWcConfig;
    stc_wc_pres_clk_t stcWcPresClk;

    // Initialize GPIO for watch counter demo
    IoInit();
#ifdef DEBUG_PRINT
    printf("4th demo\n");
    printf("Source clock = Low CR \n");
    m_u16ValidFlag = 0;
#endif
    // Select watch counter prescaler source clock as Low CR
    stcWcPresClk.enInputClk = WcPresInClkLowCr;
    // Select watch counter source clock WCCK0
    stcWcConfig.enCntClk = WcCntClkWcck1;
    // Set watch counter reload value
    stcWcConfig.u8ReloadValue = 1;
    // Set counter value, 2^10/100000, 1 step = 10.24mS
    stcWcPresClk.enOutputClk = WcPresOutClkArray4;

    Wc_Pres_SelClk(&stcWcPresClk);
    Wc_Pres_EnableDiv();

    // Enable watch counter interrupt
    Wc_EnableInt(IoPolling);
    Wc_Init(&stcWcConfig);
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Start counter, waiting several seconds approximately\n");
#endif
    // Start counter
    Wc_EnableCount();
    // Delay some time
    Delay();
#ifdef DEBUG_PRINT
    printf("==================================================\n");
    printf("Stop counter, waiting several seconds approximately\n");
    printf("\n");
#endif
    // Stop counter
    Wc_DisableCount();
    // Disable watch counter interrupt
    Wc_DisableInt();
    // Delay some time
    Delay();
}
/*****************************************************************************/
/* END OF FILE */
