/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file exint.c
 **
 ** A detailed description is available at 
 ** @link ExintGroup Clock Module description @endlink
 **
 ** History:
 **   - 2014-02-21  1.0  Edison Zhang  First version.
 **   - 2014-04-21  1.1  Edison Zhang  Clear INT flag before enable INT in 
 **                                    Exint_Enable() to avoid mis-set of INT
 **                                    flag at the first time.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "exint.h"

/**
 ******************************************************************************
 ** \addtogroup ExintGroup
 ******************************************************************************/
//@{

/******************************************************************************/
/* Local pre-processor symbols/macros ('#define')                             */
/******************************************************************************/

/******************************************************************************/
/* Global variable definitions (declared in header file with 'extern')        */
/******************************************************************************/
#if (defined(PDL_PERIPHERAL_NMI_ACTIVE))
func_ptr_t pfnNMICallback;  ///< NMI Callback pointer
#endif

#if (defined(PDL_PERIPHERAL_EXINT_ACTIVE))
///< External interrupts callback pointer array
func_ptr_t apfnExintCallback[EXINT_CHANNEL_NUM] = 
{
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
};

/******************************************************************************/
/* Local type definitions ('typedef')                                         */
/******************************************************************************/

/******************************************************************************/
/* Local function prototypes ('static')                                       */
/******************************************************************************/

/******************************************************************************/
/* Local variable definitions ('static')                                      */
/******************************************************************************/

/******************************************************************************/
/* Function implementation - global ('extern') and local ('static')           */
/******************************************************************************/

/**
 ******************************************************************************
 ** \brief Interrupt handlers for External interrupts
 **
 ** This function clears the cause bit and calls the callback function
 ******************************************************************************/
void Exint_IrqHandler(uint8_t u8Channel)
{  
    FM0P_EXTI->EICL &= (0xFFFFFFFFu ^ (1u << u8Channel));

    if (0 != apfnExintCallback[u8Channel])
    {
        apfnExintCallback[u8Channel]();
    }
  
} // Exint_IrqHandler

/*!
 ******************************************************************************
 ** \brief Enable external interrupt 
 **
 ** \param u8Ch 0~31
 ** \param pfnIntCb Interrupt callback function

 ******************************************************************************
 */
en_result_t Exint_Enable(uint8_t u8Ch, func_ptr_t pfnIntCb)
{
    if((u8Ch > EXINT_MAX_CH_INDEX) ||  (pfnIntCb == NULL))
    {
        return ErrorInvalidParameter;
    }
  
    FM0P_EXTI->EICL &= ~(1ul << u8Ch); /* Clear interrupt flag */
    FM0P_EXTI->ENIR |= (1ul<< u8Ch);   /* Enable interrupt */
    
    apfnExintCallback[u8Ch] = pfnIntCb;

    if(u8Ch <= 7u)
    {
        NVIC_ClearPendingIRQ(EXINT0_7_IRQn);
        NVIC_EnableIRQ(EXINT0_7_IRQn);
        NVIC_SetPriority(EXINT0_7_IRQn, PDL_IRQ_LEVEL_EXINT0_7);   
    }
    else
    {
        NVIC_ClearPendingIRQ(EXINT8_31_IRQn);
        NVIC_EnableIRQ(EXINT8_31_IRQn);
        NVIC_SetPriority(EXINT0_7_IRQn, PDL_IRQ_LEVEL_EXINT0_7);    
    }
    
    return Ok;
}

/*!
 ******************************************************************************
 ** \brief Dsiable external interrupt 
 **
 ** \param u8Ch 0~31
 **         
 ** \retval Ok                         Interreupt disabled normally
 ** \retval ErrorInvalidParameter      u8Ch > EXINT_MAX_CH_INDEX
 **
 ******************************************************************************
 */
en_result_t Exint_Disable(uint8_t u8Ch)
{
    if(u8Ch > EXINT_MAX_CH_INDEX)
    {
        return ErrorInvalidParameter;
    }
  
    FM0P_EXTI->ENIR &= ~(1ul<<u8Ch);
    
    apfnExintCallback[u8Ch] = NULL;
    
    if((FM0P_EXTI->ENIR & 0x000000FFu) ==  0x00000000u)
    {
        NVIC_ClearPendingIRQ(EXINT0_7_IRQn);
        NVIC_DisableIRQ(EXINT0_7_IRQn);
        NVIC_SetPriority(EXINT0_7_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
    }
    
    if((FM0P_EXTI->ENIR & 0xFFFFFF00u) ==  0x00000000u)
    {
        NVIC_ClearPendingIRQ(EXINT8_31_IRQn);
        NVIC_DisableIRQ(EXINT8_31_IRQn);
        NVIC_SetPriority(EXINT8_31_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
    }
    
    return Ok;
}

/*!
 ******************************************************************************
 ** \brief Set external interrupt detect mode
 **
 ** \param u8Ch   0~31
 ** \param enLevel           External interrupt active level
 ** \arg ExIntLowLevel       Low level is active edge for interrupt detection 
 ** \arg ExIntHighLevel      High level is active edge for interrupt detection 
 ** \arg ExIntRisingEdge     Rising edge is active edge for interrupt detection 
 ** \arg ExIntFallingEdge    Falling edge is active edge for interrupt detection
 **         
 ** \retval Ok                         Interrupt detection edge set normmally
 ** \retval ErrorInvalidParameter      u8Ch > EXINT_MAX_CH_INDEX
 **                                    invalid setting for enLevel
 **
 ******************************************************************************
 */
en_result_t Exint_SetIntDetectMode(uint8_t u8Ch, en_exint_level_t enLevel)
{
    if(u8Ch > EXINT_MAX_CH_INDEX)
    {
        return ErrorInvalidParameter;
    }
  
    switch (enLevel)
    {
        case ExIntLowLevel:
        case ExIntHighLevel:
        case ExIntRisingEdge:
        case ExIntFallingEdge:
            FM0P_EXTI->ELVR &= ~(3ul<<(2*u8Ch));
            FM0P_EXTI->ELVR |= ((uint8_t)enLevel<<(2*u8Ch));
            break;  
        default:    
            return ErrorInvalidParameter;
    }
    
    return Ok;
}

/*!
 ******************************************************************************
 ** \brief Get the external interrupt detection mode 
 **
 ** \param u8Ch   0~31
 **         
 ** \retval ExIntLowLevel       Low level is active edge for interrupt detection 
 ** \retval ExIntHighLevel      High level is active edge for interrupt detection 
 ** \retval ExIntRisingEdge     Rising edge is active edge for interrupt detection 
 ** \retval ExIntFallingEdge    Falling edge is active edge for interrupt detection
 **
 ******************************************************************************
 */
en_exint_level_t Exint_GetIntDetectMode(uint8_t u8Ch)
{
    uint8_t u8Level;
    u8Level = (FM0P_EXTI->ELVR & (3ul<<(2*u8Ch))) >> (2*u8Ch);
    return (en_exint_level_t)u8Level;
}

#endif // #if (defined(PDL_PERIPHERAL_EXINT_ACTIVE))

#if (defined(PDL_PERIPHERAL_NMI_ACTIVE))

/**
 ******************************************************************************
 ** \brief Interrupt handler for None-Maskable Interrupt (NMI)
 **
 ** This function clears the cause bit and calls the NMI callback function, if
 ** specified.
 ******************************************************************************/
void Nmi_IrqHandler(void)
{
    if ((FM0P_EXTI->NMIRR & 0x01) == 0x01)
    {
        FM0P_EXTI->NMICL = 0;

        if (0 != pfnNMICallback)
        {
            pfnNMICallback();
        }
    }
}


/*!
 ******************************************************************************
 ** \brief Set the interrupt callback function of NMI
 **
 ** \param pfnIntCb pointer to interrupt callback function
 **         
 ** \retval Ok                         NMI callback function set normally
 ** \retval ErrorInvalidParameter      pfnIntCb == NULL
 **
 ******************************************************************************
 */
en_result_t Exint_Nmi_SetIntCallback(func_ptr_t pfnIntCb)
{
    if(pfnIntCb == NULL)
    {
        return ErrorInvalidParameter;
    }
    pfnNMICallback = pfnIntCb;
    return Ok;
}

//@}

#endif

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/


