/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file interrupts.c
 **
 ** PDL Interrupt Handler
 **
 ** A detailed description is available at 
 ** @link PdlInterrupts PDL Interrupt handling description @endlink
 **
 ** History:
 **   - 2013-12-24  0.0.1  Edison Zhang        First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "interrupts.h"

/******************************************************************************/
/*********************************** NMI **************************************/
/******************************************************************************/
#if (PDL_INTERRUPT_ENABLE_NMI == PDL_ON) || (PDL_INTERRUPT_ENABLE_HWWDG == PDL_ON)
void NMI_Handler(void)
{
#if (PDL_INTERRUPT_ENABLE_NMI == PDL_ON)
    if(bFM0P_INTREQ_EXC02MON_NMI)
    {
        Nmi_IrqHandler();
    }
#endif
#if (PDL_INTERRUPT_ENABLE_HWWDG == PDL_ON)
    if (bFM0P_INTREQ_EXC02MON_HWINT == 1u)
    {
        HwwdgIrqHandler();
    }
#endif
}
#endif

/******************************************************************************/
/******************************* SW watchdog **********************************/
/******************************************************************************/
#if (PDL_INTERRUPT_ENABLE_SWWDG == PDL_ON)
void SWDT_IRQHandler(void)
{
    SwwdgIrqHandler();
}
#endif

/******************************************************************************/
/*********************************** ADC **************************************/
/******************************************************************************/
#if (PDL_INTERRUPT_ENABLE_ADC0 == PDL_ON) 
void ADC0_IRQHandler(void)
{
    AdcIrqHandler((volatile stc_adcn_t*)&ADC0, &(m_astcAdcInstanceDataLut[AdcInstanceIndexAdc0].stcInternData));
}
#endif

#if (PDL_INTERRUPT_ENABLE_ADC1 == PDL_ON) 
void ADC1_IRQHandler(void)
{
    AdcIrqHandler((volatile stc_adcn_t*)&ADC1, &(m_astcAdcInstanceDataLut[AdcInstanceIndexAdc1].stcInternData));
}
#endif

#if (PDL_INTERRUPT_ENABLE_LCD == PDL_ON) || (PDL_INTERRUPT_ENABLE_ADC2 == PDL_ON) 
void ADC2_LCD_IRQHandler(void)
{
#if (PDL_INTERRUPT_ENABLE_LCD == PDL_ON)
#endif
#if (PDL_INTERRUPT_ENABLE_ADC2 == PDL_ON)   
    AdcIrqHandler((volatile stc_adcn_t*)&ADC2, &(m_astcAdcInstanceDataLut[AdcInstanceIndexAdc2].stcInternData));
#endif    
}
#endif

/******************************************************************************/
/*********************************** BT/Flash *********************************/
/******************************************************************************/

#if (PDL_INTERRUPT_ENABLE_BT0 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT1 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT2 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT3 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT4 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT5 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT6 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_BT7 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_FLASH == PDL_ON)  
/**
 ******************************************************************************
 ** \brief BT ch. 0 IRQ handler
 ******************************************************************************
 */
void BT0_7_FLASH_IRQHandler(void)
{
    uint32_t u32IrqMon = FM0P_INTREQ->IRQ31MON;
  
#if (PDL_INTERRUPT_ENABLE_BT0 == PDL_ON) 
    if(0 != (u32IrqMon & 0x00000003ul))
    {
        Bt_IrqHandler((volatile stc_btn_t*)&BT0, &m_astcBtInstanceDataLut[BtInstanceIndexBt0].stcInternData);
    }
#endif     
#if (PDL_INTERRUPT_ENABLE_BT1 == PDL_ON) 
    if(0 != (u32IrqMon & 0x0000000Cul))
    {
        Bt_IrqHandler((volatile stc_btn_t*)&BT1, &m_astcBtInstanceDataLut[BtInstanceIndexBt1].stcInternData);
    }
#endif     
#if (PDL_INTERRUPT_ENABLE_BT2 == PDL_ON) 
    if(0 != (u32IrqMon & 0x00000030ul))
    {
        Bt_IrqHandler((volatile stc_btn_t*)&BT2, &m_astcBtInstanceDataLut[BtInstanceIndexBt2].stcInternData);
    }
#endif 
#if (PDL_INTERRUPT_ENABLE_BT3 == PDL_ON) 
    if(0 != (u32IrqMon & 0x000000C0ul))
    {
        Bt_IrqHandler((volatile stc_btn_t*)&BT3, &m_astcBtInstanceDataLut[BtInstanceIndexBt3].stcInternData);
    }
#endif    
#if (PDL_INTERRUPT_ENABLE_BT4 == PDL_ON) 
    if(0 != (u32IrqMon & 0x00000300ul))
    {
        Bt_IrqHandler((volatile stc_btn_t*)&BT4, &m_astcBtInstanceDataLut[BtInstanceIndexBt4].stcInternData);
    }
#endif
#if (PDL_INTERRUPT_ENABLE_BT5 == PDL_ON) 
    if(0 != (u32IrqMon & 0x00000C00ul))
    {
        Bt_IrqHandler((volatile stc_btn_t*)&BT5, &m_astcBtInstanceDataLut[BtInstanceIndexBt5].stcInternData);
    }
#endif  
#if (PDL_INTERRUPT_ENABLE_BT6 == PDL_ON) 
    if(0 != (u32IrqMon & 0x00003000ul))
    {
        Bt_IrqHandler((volatile stc_btn_t*)&BT6, &m_astcBtInstanceDataLut[BtInstanceIndexBt6].stcInternData);
    }
#endif
#if (PDL_INTERRUPT_ENABLE_BT7 == PDL_ON) 
    if(0 != (u32IrqMon & 0x0000C000ul))
    {
        Bt_IrqHandler((volatile stc_btn_t*)&BT7, &m_astcBtInstanceDataLut[BtInstanceIndexBt7].stcInternData);
    }
#endif   
#if (PDL_INTERRUPT_ENABLE_FLASH == PDL_OFF)
#endif    
}
#endif

/******************************************************************************/
/*********************************** CLK/WC/RTC *******************************/
/******************************************************************************/
#if (PDL_INTERRUPT_ENABLE_CLK == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_WC == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_RTC == PDL_ON)
void TIM_IRQHandler(void)
{
    uint32_t u32IrqMon = FM0P_INTREQ->IRQ24MON;

#if (PDL_INTERRUPT_ENABLE_RTC == PDL_ON)
    if(0 != (u32IrqMon & 0x00000020ul))
    {  
        Rtc_IrqHandler();
    }
#endif
#if (PDL_INTERRUPT_ENABLE_WC == PDL_ON)
    if(0 != (u32IrqMon & 0x00000010ul))
    {  
        Wc_IrqHandler();
    }
#endif    
#if (PDL_INTERRUPT_ENABLE_CLK == PDL_ON)
    if(0 != (u32IrqMon & 0x00000007ul))
    {  
        Clk_IrqHandler();
    }
#endif    
}
#endif

/******************************************************************************/
/*********************************** CSV **************************************/
/******************************************************************************/
#if (PDL_INTERRUPT_ENABLE_CSV == PDL_ON)
void CSV_IRQHandler(void)
{
    Csv_IrqHandler();
}
#endif

/******************************************************************************/
/******************************* Dual Timer/QPRC ******************************/
/******************************************************************************/
#if (PDL_INTERRUPT_ENABLE_DT == PDL_ON) || (PDL_INTERRUPT_ENABLE_QPRC0 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_QPRC1 == PDL_ON) || (PDL_INTERRUPT_ENABLE_QPRC2 == PDL_ON)
void DT_QPRC_Handler(void)
{
    uint32_t u32IrqMon = FM0P_INTREQ->IRQ06MON;
#if (PDL_INTERRUPT_ENABLE_DT == PDL_ON)
    if (0 != (u32IrqMon & 0x00000001ul))
    {
        DtIrqHandler(DtChannel0);
    }
    if (0 != (u32IrqMon & 0x00000002ul))
    {
        DtIrqHandler(DtChannel1);
    }
#endif    
#if (PDL_INTERRUPT_ENABLE_QPRC0 == PDL_ON)
    if(0!= (u32IrqMon & 0x000000FCul))
    {
        Qprc_IrqHandler((volatile stc_qprcn_t*)&QPRC0, &m_astcQprcInstanceDataLut[QprcInstanceIndexQprc0].stcInternData);
    }
#endif  
#if (PDL_INTERRUPT_ENABLE_QPRC1 == PDL_ON)
    if(0!= (u32IrqMon & 0x00003F00ul))
    {
        Qprc_IrqHandler((volatile stc_qprcn_t*)&QPRC1, &m_astcQprcInstanceDataLut[QprcInstanceIndexQprc1].stcInternData);
    }
#endif      
#if (PDL_INTERRUPT_ENABLE_QPRC2 == PDL_ON)    
    if(0!= (u32IrqMon & 0x000FC000ul))
    {
        Qprc_IrqHandler((volatile stc_qprcn_t*)&QPRC2, &m_astcQprcInstanceDataLut[QprcInstanceIndexQprc2].stcInternData);
    }
#endif    
}
#endif

/******************************************************************************/
/*********************************** LVD **************************************/
/******************************************************************************/
#if (PDL_INTERRUPT_ENABLE_LVD == PDL_ON)
/**
 ******************************************************************************
 ** \brief LVD IRQ handler
 ******************************************************************************
 */
void LVD_IRQHandler(void)
{
    LvdIrqHandler();
}
#endif // #if (PDL_INTERRUPT_ENABLE_LVD == PDL_ON) && (PDL_PERIPHERAL_ENABLE_LVD == PDL_ON)

/******************************************************************************/
/********************************* EXINT **************************************/
/******************************************************************************/
#if (PDL_INTERRUPT_ENABLE_EXINT0 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT1 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT2 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT3 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT4 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT5 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT6 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT7 == PDL_ON)   
/**
 ******************************************************************************
 ** \brief EXINT ch.0 ~ ch.7 IRQ handler
 ******************************************************************************
 */
void INT0_7_Handler(void)
{ 
#if(PDL_INTERRUPT_ENABLE_EXINT0 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ04MON_EXTINT0)
    {
        Exint_IrqHandler(ExintInstanceIndexExint0);
    }
#endif  
#if(PDL_INTERRUPT_ENABLE_EXINT1 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ04MON_EXTINT1)
    {
        Exint_IrqHandler(ExintInstanceIndexExint1);
    }
#endif 
#if(PDL_INTERRUPT_ENABLE_EXINT2 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ04MON_EXTINT2)
    {
        Exint_IrqHandler(ExintInstanceIndexExint2);
    }
#endif 
#if(PDL_INTERRUPT_ENABLE_EXINT3 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ04MON_EXTINT3)
    {
        Exint_IrqHandler(ExintInstanceIndexExint3);
    }
#endif 
#if(PDL_INTERRUPT_ENABLE_EXINT4 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ04MON_EXTINT4)
    {
        Exint_IrqHandler(ExintInstanceIndexExint4);
    }
#endif 
#if(PDL_INTERRUPT_ENABLE_EXINT5 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ04MON_EXTINT5)
    {
        Exint_IrqHandler(ExintInstanceIndexExint5);
    }
#endif 
#if(PDL_INTERRUPT_ENABLE_EXINT6 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ04MON_EXTINT6)
    {
        Exint_IrqHandler(ExintInstanceIndexExint6);
    }
#endif 
#if(PDL_INTERRUPT_ENABLE_EXINT7 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ04MON_EXTINT7)
    {
        Exint_IrqHandler(ExintInstanceIndexExint7);
    }
#endif     
    
}
#endif

#if (PDL_INTERRUPT_ENABLE_EXINT8 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT9 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT10 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT11 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT12 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT13 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT14 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT15 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT16 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT17 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT18 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT19 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT20 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT21 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT22 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT23 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT24 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT25 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT26 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT27 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT28 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT29 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT30 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_EXINT31 == PDL_ON)     
/**
 ******************************************************************************
 ** \brief EXINT ch.8 ~ ch.31 IRQ handler
 ******************************************************************************
 */
void INT8_31_Handler(void)
{ 
#if(PDL_INTERRUPT_ENABLE_EXINT8 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ05MON_EXTINT8)
    {
        Exint_IrqHandler(ExintInstanceIndexExint8);
    }
#endif  
#if(PDL_INTERRUPT_ENABLE_EXINT9 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ05MON_EXTINT9)
    {
        Exint_IrqHandler(ExintInstanceIndexExint9);
    }
#endif 
#if(PDL_INTERRUPT_ENABLE_EXINT10 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ05MON_EXTINT10)
    {
        Exint_IrqHandler(ExintInstanceIndexExint10);
    }
#endif 
#if(PDL_INTERRUPT_ENABLE_EXINT11 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ05MON_EXTINT11)
    {
        Exint_IrqHandler(ExintInstanceIndexExint11);
    }
#endif 
#if(PDL_INTERRUPT_ENABLE_EXINT12 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ05MON_EXTINT12)
    {
        Exint_IrqHandler(ExintInstanceIndexExint12);
    }
#endif 
#if(PDL_INTERRUPT_ENABLE_EXINT13 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ05MON_EXTINT13)
    {
        Exint_IrqHandler(ExintInstanceIndexExint13);
    }
#endif 
#if(PDL_INTERRUPT_ENABLE_EXINT14 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ05MON_EXTINT14)
    {
        Exint_IrqHandler(ExintInstanceIndexExint14);
    }
#endif 
#if(PDL_INTERRUPT_ENABLE_EXINT15 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ05MON_EXTINT15)
    {
        Exint_IrqHandler(ExintInstanceIndexExint15);
    }
#endif     
#if(PDL_INTERRUPT_ENABLE_EXINT16 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ05MON_EXTINT16)
    {
        Exint_IrqHandler(ExintInstanceIndexExint16);
    }
#endif  
#if(PDL_INTERRUPT_ENABLE_EXINT17 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ05MON_EXTINT17)
    {
        Exint_IrqHandler(ExintInstanceIndexExint17);
    }
#endif 
#if(PDL_INTERRUPT_ENABLE_EXINT18 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ05MON_EXTINT18)
    {
        Exint_IrqHandler(ExintInstanceIndexExint18);
    }
#endif 
#if(PDL_INTERRUPT_ENABLE_EXINT19 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ05MON_EXTINT19)
    {
        Exint_IrqHandler(ExintInstanceIndexExint19);
    }
#endif 
#if(PDL_INTERRUPT_ENABLE_EXINT20 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ05MON_EXTINT20)
    {
        Exint_IrqHandler(ExintInstanceIndexExint20);
    }
#endif 
#if(PDL_INTERRUPT_ENABLE_EXINT21 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ05MON_EXTINT21)
    {
        Exint_IrqHandler(ExintInstanceIndexExint21);
    }
#endif 
#if(PDL_INTERRUPT_ENABLE_EXINT22 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ05MON_EXTINT22)
    {
        Exint_IrqHandler(ExintInstanceIndexExint22);
    }
#endif 
#if(PDL_INTERRUPT_ENABLE_EXINT23 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ05MON_EXTINT23)
    {
        Exint_IrqHandler(ExintInstanceIndexExint23);
    }
#endif
#if(PDL_INTERRUPT_ENABLE_EXINT24 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ05MON_EXTINT24)
    {
        Exint_IrqHandler(ExintInstanceIndexExint24);
    }
#endif  
#if(PDL_INTERRUPT_ENABLE_EXINT25 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ05MON_EXTINT25)
    {
        Exint_IrqHandler(ExintInstanceIndexExint25);
    }
#endif 
#if(PDL_INTERRUPT_ENABLE_EXINT26 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ05MON_EXTINT26)
    {
        Exint_IrqHandler(ExintInstanceIndexExint26);
    }
#endif 
#if(PDL_INTERRUPT_ENABLE_EXINT27 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ05MON_EXTINT27)
    {
        Exint_IrqHandler(ExintInstanceIndexExint27);
    }
#endif 
#if(PDL_INTERRUPT_ENABLE_EXINT28 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ05MON_EXTINT28)
    {
        Exint_IrqHandler(ExintInstanceIndexExint28);
    }
#endif 
#if(PDL_INTERRUPT_ENABLE_EXINT29 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ05MON_EXTINT29)
    {
        Exint_IrqHandler(ExintInstanceIndexExint29);
    }
#endif 
#if(PDL_INTERRUPT_ENABLE_EXINT30 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ05MON_EXTINT30)
    {
        Exint_IrqHandler(ExintInstanceIndexExint30);
    }
#endif 
#if(PDL_INTERRUPT_ENABLE_EXINT31 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ05MON_EXTINT31)
    {
        Exint_IrqHandler(ExintInstanceIndexExint31);
    }
#endif       
}
#endif

/******************************************************************************/
/*********************************** MFT **************************************/
/******************************************************************************/
#if (PDL_INTERRUPT_ENABLE_MFT0_FRT == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT1_FRT == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT2_FRT == PDL_ON)  
/**
 ******************************************************************************
 ** \brief MFT0's FRT peak match IRQ handler
 ******************************************************************************
 */
void MFT_FRT_IRQHandler(void)
{ 
    uint32_t u32IrqMon = FM0P_INTREQ->IRQ28MON;
#if (PDL_INTERRUPT_ENABLE_MFT0_FRT == PDL_ON)    
    if(0 != (u32IrqMon & 0x0000003Fu))
    {
        Mft_Frt_IrqHandler((volatile stc_mftn_frt_t*)&MFT0_FRT, &m_astcMftFrtInstanceDataLut[FrtInstanceIndexFrt0].stcInternData);
    }
#endif   
#if (PDL_INTERRUPT_ENABLE_MFT1_FRT == PDL_ON)    
    if(0 != (u32IrqMon & 0x00000FC0u))
    {
        Mft_Frt_IrqHandler((volatile stc_mftn_frt_t*)&MFT1_FRT, &m_astcMftFrtInstanceDataLut[FrtInstanceIndexFrt1].stcInternData);
    }
#endif     
#if (PDL_INTERRUPT_ENABLE_MFT2_FRT == PDL_ON)    
    if(0 != (u32IrqMon & 0x0003F000u))
    {
        Mft_Frt_IrqHandler((volatile stc_mftn_frt_t*)&MFT2_FRT, &m_astcMftFrtInstanceDataLut[FrtInstanceIndexFrt2].stcInternData);
    }
#endif      
}
#endif

#if (PDL_INTERRUPT_ENABLE_MFT0_OCU == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT1_OCU == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT2_OCU == PDL_ON)  
/**
 ******************************************************************************
 ** \brief MFT0's OCU IRQ handler
 ******************************************************************************
 */
void MFT_OPC_IRQHandler(void)
{
    uint32_t u32IrqMon = FM0P_INTREQ->IRQ30MON;
    
#if (PDL_INTERRUPT_ENABLE_MFT0_OCU == PDL_ON)    
    if(0 != (u32IrqMon & 0x0000003Fu))
    {
        Mft_Ocu_IrqHandler((volatile stc_mftn_ocu_t*)&MFT0_OCU, &m_astcMftOcuInstanceDataLut[OcuInstanceIndexOcu0].stcInternData);
    }
#endif  
#if (PDL_INTERRUPT_ENABLE_MFT1_OCU == PDL_ON)    
    if(0 != (u32IrqMon & 0x00000FC0u))
    {
        Mft_Ocu_IrqHandler((volatile stc_mftn_ocu_t*)&MFT1_OCU, &m_astcMftOcuInstanceDataLut[OcuInstanceIndexOcu1].stcInternData);
    }
#endif     
#if (PDL_INTERRUPT_ENABLE_MFT2_OCU == PDL_ON)    
    if(0 != (u32IrqMon & 0x00000FC0u))
    {
        Mft_Ocu_IrqHandler((volatile stc_mftn_ocu_t*)&MFT2_OCU, &m_astcMftOcuInstanceDataLut[OcuInstanceIndexOcu2].stcInternData);
    }
#endif   
}
#endif

#if (PDL_INTERRUPT_ENABLE_MFT0_WFG == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT1_WFG == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT2_WFG == PDL_ON)  
/**
 ******************************************************************************
 ** \brief MFT0's WFG IRQ handler
 ******************************************************************************
 */
void MFT_WFG_IRQHandler(void)
{
    uint32_t u32IrqMon = FM0P_INTREQ->IRQ03MON;
      
#if (PDL_INTERRUPT_ENABLE_MFT0_WFG == PDL_ON)
    if(0 != (u32IrqMon & 0x0000000Fu))
    {
        Mft_Wfg_IrqHandler((volatile stc_mftn_wfg_t*)&MFT0_WFG, &m_astcMftWfgInstanceDataLut[WfgInstanceIndexWfg0].stcInternData);
    }
#endif
#if (PDL_INTERRUPT_ENABLE_MFT1_WFG == PDL_ON)
    if(0 != (u32IrqMon & 0x000000F0u))
    {
        Mft_Wfg_IrqHandler((volatile stc_mftn_wfg_t*)&MFT1_WFG, &m_astcMftWfgInstanceDataLut[WfgInstanceIndexWfg1].stcInternData);
    }
#endif    
#if (PDL_INTERRUPT_ENABLE_MFT2_WFG == PDL_ON)
    if(0 != (u32IrqMon & 0x00000F00u))
    {
        Mft_Wfg_IrqHandler((volatile stc_mftn_wfg_t*)&MFT2_WFG, &m_astcMftWfgInstanceDataLut[WfgInstanceIndexWfg2].stcInternData);
    }
#endif     
}
#endif

#if (PDL_INTERRUPT_ENABLE_MFT0_ICU == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT1_ICU == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT2_ICU == PDL_ON)
/**
 ******************************************************************************
 ** \brief MFT0's ICU IRQ handler
 ******************************************************************************
 */
void MFT_IPC_IRQHandler(void)
{
    uint32_t u32IrqMon = FM0P_INTREQ->IRQ29MON;
    
#if (PDL_INTERRUPT_ENABLE_MFT0_ICU == PDL_ON)  
    if(0 != (u32IrqMon & 0x0000000Fu))
    {
        Mft_Icu_IrqHandler((volatile stc_mftn_icu_t*)&MFT0_ICU, &m_astcMftIcuInstanceDataLut[IcuInstanceIndexIcu0].stcInternData);
    }
#endif    
#if (PDL_INTERRUPT_ENABLE_MFT1_ICU == PDL_ON)  
    if(0 != (u32IrqMon & 0x000000F0u))
    {
        Mft_Icu_IrqHandler((volatile stc_mftn_icu_t*)&MFT1_ICU, &m_astcMftIcuInstanceDataLut[IcuInstanceIndexIcu1].stcInternData);
    }
#endif    
#if (PDL_INTERRUPT_ENABLE_MFT2_ICU == PDL_ON)  
    if(0 != (u32IrqMon & 0x00000F00u))
    {
        Mft_Icu_IrqHandler((volatile stc_mftn_icu_t*)&MFT2_ICU, &m_astcMftIcuInstanceDataLut[IcuInstanceIndexIcu2].stcInternData);
    }
#endif      
}
#endif

/******************************************************************************/
/*********************************** PPG **************************************/
/******************************************************************************/
#if (PDL_INTERRUPT_ENABLE_PPG == PDL_ON)
/**
 ******************************************************************************
 ** \brief PPG ch.0,2,4,8,10,12,16,18,20 IRQ handler
 ******************************************************************************
 */
void PPG_IRQHandler(void)
{
    Ppg_IrqHandler();
}
#endif

/******************************************************************************/
/*********************************** MFS **************************************/
/******************************************************************************/

#if (PDL_INTERRUPT_ENABLE_MFS0 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS8 == PDL_ON)
/**
 ******************************************************************************
 ** \brief MFS ch.0/8 RX IRQ handler
 ******************************************************************************
 */
void MFS0_8_RX_IRQHandler(void) 
{
#if (PDL_INTERRUPT_ENABLE_MFS0 == PDL_ON)    
    if(0 != bFM0P_INTREQ_IRQ07MON_MFSINT0)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs0].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerRx((stc_mfsn_uart_t*)&UART0, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs0].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerRx((stc_mfsn_csio_t*)&CSIO0, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs0].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerRx((stc_mfsn_i2c_t*)&I2C0, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs0].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerRx((stc_mfsn_lin_t*)&LIN0, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs0].stcInternData));
                break;
            default:
                break;
        
        }
    }
#endif
#if (PDL_INTERRUPT_ENABLE_MFS8 == PDL_ON)        
    if(0 != bFM0P_INTREQ_IRQ07MON_MFSINT1)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs8].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerRx((stc_mfsn_uart_t*)&UART8, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs8].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerRx((stc_mfsn_csio_t*)&CSIO8, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs8].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerRx((stc_mfsn_i2c_t*)&I2C8, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs8].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerRx((stc_mfsn_lin_t*)&LIN8, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs8].stcInternData));
                break;
            default:
                break;
        
        }
    }
#endif    
}

/**
 ******************************************************************************
 ** \brief MFS ch.0/8 TX IRQ handler
 ******************************************************************************
 */
void MFS0_8_TX_IRQHandler(void)
{
#if (PDL_INTERRUPT_ENABLE_MFS0 == PDL_ON)    
    if(0 != bFM0P_INTREQ_IRQ08MON_MFSINT0)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs0].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerTx((stc_mfsn_uart_t*)&UART0, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs0].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerTx((stc_mfsn_csio_t*)&CSIO0, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs0].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerTx((stc_mfsn_i2c_t*)&I2C0, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs0].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerTx((stc_mfsn_lin_t*)&LIN0, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs0].stcInternData));
                break;
            default:
                break;
        
        }
    }
    if(0 != bFM0P_INTREQ_IRQ08MON_MFSINT1)
    {
       switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs0].stcInternData.enMode)
        {
            case MfsUartMode:
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerStatus((stc_mfsn_csio_t*)&CSIO0, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs0].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerStatus((stc_mfsn_i2c_t*)&I2C0, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs0].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerStatus((stc_mfsn_lin_t*)&LIN0, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs0].stcInternData));
                break;
            default:
                break;
        
        }
    }
#endif
#if (PDL_INTERRUPT_ENABLE_MFS8 == PDL_ON)        
    if(0 != bFM0P_INTREQ_IRQ08MON_MFSINT2)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs8].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerTx((stc_mfsn_uart_t*)&UART8, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs8].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerTx((stc_mfsn_csio_t*)&CSIO8, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs8].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerTx((stc_mfsn_i2c_t*)&I2C8, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs8].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerTx((stc_mfsn_lin_t*)&LIN8, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs8].stcInternData));
                break;
            default:
                break;
        
        }
    }
    if(0 != bFM0P_INTREQ_IRQ08MON_MFSINT3)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs8].stcInternData.enMode)
        {
            case MfsUartMode:
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerStatus((stc_mfsn_csio_t*)&CSIO8, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs8].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerStatus((stc_mfsn_i2c_t*)&I2C8, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs8].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerStatus((stc_mfsn_lin_t*)&LIN8, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs8].stcInternData));
                break;
            default:
                break;
        
        } 
    }
#endif    
}
#endif // #if (PDL_INTERRUPT_ENABLE_MFS0 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS8 == PDL_ON)

#if (PDL_INTERRUPT_ENABLE_MFS1 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS9 == PDL_ON)
/**
 ******************************************************************************
 ** \brief MFS ch.1/9 RX IRQ handler
 ******************************************************************************
 */
void MFS1_9_RX_IRQHandler(void)
{
#if (PDL_INTERRUPT_ENABLE_MFS1 == PDL_ON)    
    if(0 != bFM0P_INTREQ_IRQ09MON_MFSINT0)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs1].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerRx((stc_mfsn_uart_t*)&UART1, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs1].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerRx((stc_mfsn_csio_t*)&CSIO1, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs1].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerRx((stc_mfsn_i2c_t*)&I2C1, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs1].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerRx((stc_mfsn_lin_t*)&LIN1, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs1].stcInternData));
                break;
            default:
                break;
        
        }
    }
#endif
#if (PDL_INTERRUPT_ENABLE_MFS9 == PDL_ON)        
    if(0 != bFM0P_INTREQ_IRQ09MON_MFSINT1)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs9].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerRx((stc_mfsn_uart_t*)&UART9, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs9].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerRx((stc_mfsn_csio_t*)&CSIO9, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs9].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerRx((stc_mfsn_i2c_t*)&I2C9, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs9].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerRx((stc_mfsn_lin_t*)&LIN9, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs9].stcInternData));
                break;
            default:
                break;
        
        }
    }
#endif    
}

/**
 ******************************************************************************
 ** \brief MFS ch.1/9 TX IRQ handler
 ******************************************************************************
 */
void MFS1_9_TX_IRQHandler(void)
{
#if (PDL_INTERRUPT_ENABLE_MFS1 == PDL_ON)    
    if(0 != bFM0P_INTREQ_IRQ10MON_MFSINT0)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs1].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerTx((stc_mfsn_uart_t*)&UART1, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs1].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerTx((stc_mfsn_csio_t*)&CSIO1, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs1].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerTx((stc_mfsn_i2c_t*)&I2C1, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs1].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerTx((stc_mfsn_lin_t*)&LIN1, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs1].stcInternData));
                break;
            default:
                break;
        
        }
    }
    if(0 != bFM0P_INTREQ_IRQ10MON_MFSINT1)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs1].stcInternData.enMode)
        {
            case MfsUartMode:
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerStatus((stc_mfsn_csio_t*)&CSIO1, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs1].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerStatus((stc_mfsn_i2c_t*)&I2C1, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs1].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerStatus((stc_mfsn_lin_t*)&LIN1, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs1].stcInternData));
                break;
            default:
                break;
        
        } 
    }
#endif
#if (PDL_INTERRUPT_ENABLE_MFS9 == PDL_ON)        
    if(0 != bFM0P_INTREQ_IRQ10MON_MFSINT2)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs9].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerTx((stc_mfsn_uart_t*)&UART9, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs9].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerTx((stc_mfsn_csio_t*)&CSIO9, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs9].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerTx((stc_mfsn_i2c_t*)&I2C9, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs9].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerTx((stc_mfsn_lin_t*)&LIN9, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs9].stcInternData));
                break;
            default:
                break;
        
        }
    }
    if(0 != bFM0P_INTREQ_IRQ10MON_MFSINT3)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs9].stcInternData.enMode)
        {
            case MfsUartMode:
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerStatus((stc_mfsn_csio_t*)&CSIO9, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs9].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerStatus((stc_mfsn_i2c_t*)&I2C9, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs9].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerStatus((stc_mfsn_lin_t*)&LIN9, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs9].stcInternData));
                break;
            default:
                break;
        
        }
    }
#endif    
}
#endif // #if (PDL_INTERRUPT_ENABLE_MFS1 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS9 == PDL_ON)

#if (PDL_INTERRUPT_ENABLE_MFS2 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS10 == PDL_ON)
/**
 ******************************************************************************
 ** \brief MFS ch.2/10 RX IRQ handler
 ******************************************************************************
 */
void MFS2_10_RX_IRQHandler(void)
{
#if (PDL_INTERRUPT_ENABLE_MFS2 == PDL_ON)    
    if(0 != bFM0P_INTREQ_IRQ11MON_MFSINT0)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs2].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerRx((stc_mfsn_uart_t*)&UART2, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs2].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerRx((stc_mfsn_csio_t*)&CSIO2, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs2].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerRx((stc_mfsn_i2c_t*)&I2C2, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs2].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerRx((stc_mfsn_lin_t*)&LIN2, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs2].stcInternData));
                break;
            default:
                break;
        
        } 
    }
#endif
#if (PDL_INTERRUPT_ENABLE_MFS10 == PDL_ON)        
    if(0 != bFM0P_INTREQ_IRQ11MON_MFSINT1)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs10].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerRx((stc_mfsn_uart_t*)&UART10, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs10].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerRx((stc_mfsn_csio_t*)&CSIO10, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs10].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerRx((stc_mfsn_i2c_t*)&I2C10, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs10].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerRx((stc_mfsn_lin_t*)&LIN10, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs10].stcInternData));
                break;
            default:
                break;
        
        } 
    }
#endif    
}

/**
 ******************************************************************************
 ** \brief MFS ch.2/10 TX IRQ handler
 ******************************************************************************
 */
void MFS2_10_TX_IRQHandler(void)
{
#if (PDL_INTERRUPT_ENABLE_MFS2 == PDL_ON)    
    if(0 != bFM0P_INTREQ_IRQ12MON_MFSINT0)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs2].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerTx((stc_mfsn_uart_t*)&UART2, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs2].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerTx((stc_mfsn_csio_t*)&CSIO2, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs2].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerTx((stc_mfsn_i2c_t*)&I2C2, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs2].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerTx((stc_mfsn_lin_t*)&LIN2, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs2].stcInternData));
                break;
            default:
                break;
        
        }
    }
    if(0 != bFM0P_INTREQ_IRQ12MON_MFSINT1)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs2].stcInternData.enMode)
        {
            case MfsUartMode:
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerStatus((stc_mfsn_csio_t*)&CSIO2, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs2].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerStatus((stc_mfsn_i2c_t*)&I2C2, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs2].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerStatus((stc_mfsn_lin_t*)&LIN2, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs2].stcInternData));
                break;
            default:
                break;
        
        } 
    }
#endif
#if (PDL_INTERRUPT_ENABLE_MFS10 == PDL_ON)        
    if(0 != bFM0P_INTREQ_IRQ12MON_MFSINT2)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs10].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerTx((stc_mfsn_uart_t*)&UART10, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs10].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerTx((stc_mfsn_csio_t*)&CSIO10, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs10].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerTx((stc_mfsn_i2c_t*)&I2C10, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs10].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerTx((stc_mfsn_lin_t*)&LIN10, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs10].stcInternData));
                break;
            default:
                break;
        
        }
    }
    if(0 != bFM0P_INTREQ_IRQ12MON_MFSINT3)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs10].stcInternData.enMode)
        {
            case MfsUartMode:
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerStatus((stc_mfsn_csio_t*)&CSIO10, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs10].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerStatus((stc_mfsn_i2c_t*)&I2C10, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs10].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerStatus((stc_mfsn_lin_t*)&LIN10, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs10].stcInternData));
                break;
            default:
                break;
        
        }
    }
#endif    
}
#endif // #if (PDL_INTERRUPT_ENABLE_MFS2 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS10 == PDL_ON)

#if (PDL_INTERRUPT_ENABLE_MFS3 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS11 == PDL_ON)
/**
 ******************************************************************************
 ** \brief MFS ch.3/11 RX IRQ handler
 ******************************************************************************
 */
void MFS3_11_RX_IRQHandler(void)
{
#if (PDL_INTERRUPT_ENABLE_MFS3 == PDL_ON)    
    if(0 != bFM0P_INTREQ_IRQ13MON_MFSINT0)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs3].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerRx((stc_mfsn_uart_t*)&UART3, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs3].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerRx((stc_mfsn_csio_t*)&CSIO3, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs3].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerRx((stc_mfsn_i2c_t*)&I2C3, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs3].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerRx((stc_mfsn_lin_t*)&LIN3, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs3].stcInternData));
                break;
            default:
                break;
        
        } 
    }
#endif
#if (PDL_INTERRUPT_ENABLE_MFS11 == PDL_ON)        
    if(0 != bFM0P_INTREQ_IRQ13MON_MFSINT1)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs11].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerRx((stc_mfsn_uart_t*)&UART11, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs11].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerRx((stc_mfsn_csio_t*)&CSIO11, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs11].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerRx((stc_mfsn_i2c_t*)&I2C11, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs11].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerRx((stc_mfsn_lin_t*)&LIN11, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs11].stcInternData));
                break;
            default:
                break;
        
        } 
    }
#endif    
}

/**
 ******************************************************************************
 ** \brief MFS ch.3/11 TX IRQ handler
 ******************************************************************************
 */
void MFS3_11_TX_IRQHandler(void)
{
#if (PDL_INTERRUPT_ENABLE_MFS3 == PDL_ON)    
    if(0 != bFM0P_INTREQ_IRQ14MON_MFSINT0)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs3].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerTx((stc_mfsn_uart_t*)&UART3, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs3].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerTx((stc_mfsn_csio_t*)&CSIO3, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs3].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerTx((stc_mfsn_i2c_t*)&I2C3, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs3].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerTx((stc_mfsn_lin_t*)&LIN3, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs3].stcInternData));
                break;
            default:
                break;
        
        }
    }
    if(0 != bFM0P_INTREQ_IRQ14MON_MFSINT1)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs3].stcInternData.enMode)
        {
            case MfsUartMode:
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerStatus((stc_mfsn_csio_t*)&CSIO3, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs3].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerStatus((stc_mfsn_i2c_t*)&I2C3, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs3].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerStatus((stc_mfsn_lin_t*)&LIN3, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs3].stcInternData));
                break;
            default:
                break;
        
        }
    }
#endif
#if (PDL_INTERRUPT_ENABLE_MFS11 == PDL_ON)        
    if(0 != bFM0P_INTREQ_IRQ14MON_MFSINT2)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs11].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerTx((stc_mfsn_uart_t*)&UART11, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs11].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerTx((stc_mfsn_csio_t*)&CSIO11, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs11].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerTx((stc_mfsn_i2c_t*)&I2C11, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs11].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerTx((stc_mfsn_lin_t*)&LIN11, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs11].stcInternData));
                break;
            default:
                break;
        
        }
    }
    if(0 != bFM0P_INTREQ_IRQ14MON_MFSINT3)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs11].stcInternData.enMode)
        {
            case MfsUartMode:
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerStatus((stc_mfsn_csio_t*)&CSIO11, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs11].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerStatus((stc_mfsn_i2c_t*)&I2C11, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs11].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerStatus((stc_mfsn_lin_t*)&LIN11, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs11].stcInternData));
                break;
            default:
                break;
        
        }
    }
#endif    
}
#endif // #if (PDL_INTERRUPT_ENABLE_MFS3 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS11 == PDL_ON)

#if (PDL_INTERRUPT_ENABLE_MFS4 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS12 == PDL_ON)
/**
 ******************************************************************************
 ** \brief MFS ch.4/12 RX IRQ handler
 ******************************************************************************
 */
void MFS4_12_RX_IRQHandler(void)
{
#if (PDL_INTERRUPT_ENABLE_MFS4 == PDL_ON)    
    if(0 != bFM0P_INTREQ_IRQ15MON_MFSINT0)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs4].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerRx((stc_mfsn_uart_t*)&UART4, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs4].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerRx((stc_mfsn_csio_t*)&CSIO4, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs4].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerRx((stc_mfsn_i2c_t*)&I2C4, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs4].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerRx((stc_mfsn_lin_t*)&LIN4, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs4].stcInternData));
                break;
            default:
                break;
        
        } 
    }
#endif
#if (PDL_INTERRUPT_ENABLE_MFS12 == PDL_ON)        
    if(0 != bFM0P_INTREQ_IRQ15MON_MFSINT1)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs12].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerRx((stc_mfsn_uart_t*)&UART12, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs12].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerRx((stc_mfsn_csio_t*)&CSIO12, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs12].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerRx((stc_mfsn_i2c_t*)&I2C12, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs12].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerRx((stc_mfsn_lin_t*)&LIN12, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs12].stcInternData));
                break;
            default:
                break;
        
        }  
    }
#endif    
}

/**
 ******************************************************************************
 ** \brief MFS ch.4/12 TX IRQ handler
 ******************************************************************************
 */
void MFS4_12_TX_IRQHandler(void)
{
#if (PDL_INTERRUPT_ENABLE_MFS4 == PDL_ON)    
    if(0 != bFM0P_INTREQ_IRQ16MON_MFSINT0)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs4].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerTx((stc_mfsn_uart_t*)&UART4, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs4].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerTx((stc_mfsn_csio_t*)&CSIO4, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs4].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerTx((stc_mfsn_i2c_t*)&I2C4, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs4].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerTx((stc_mfsn_lin_t*)&LIN4, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs4].stcInternData));
                break;
            default:
                break;
        }
    }
    if(0 != bFM0P_INTREQ_IRQ16MON_MFSINT1)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs4].stcInternData.enMode)
        {
            case MfsUartMode:
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerStatus((stc_mfsn_csio_t*)&CSIO4, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs4].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerStatus((stc_mfsn_i2c_t*)&I2C4, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs4].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerStatus((stc_mfsn_lin_t*)&LIN4, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs4].stcInternData));
                break;
            default:
                break;
        
        }
    }
#endif
#if (PDL_INTERRUPT_ENABLE_MFS12 == PDL_ON)        
    if(0 != bFM0P_INTREQ_IRQ16MON_MFSINT2)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs12].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerTx((stc_mfsn_uart_t*)&UART12, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs12].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerTx((stc_mfsn_csio_t*)&CSIO12, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs12].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerTx((stc_mfsn_i2c_t*)&I2C12, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs12].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerTx((stc_mfsn_lin_t*)&LIN12, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs12].stcInternData));
                break;
            default:
                break;
        }
    }
    if(0 != bFM0P_INTREQ_IRQ16MON_MFSINT3)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs12].stcInternData.enMode)
        {
            case MfsUartMode:
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerStatus((stc_mfsn_csio_t*)&CSIO12, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs12].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerStatus((stc_mfsn_i2c_t*)&I2C12, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs12].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerStatus((stc_mfsn_lin_t*)&LIN12, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs12].stcInternData));
                break;
            default:
                break;
        
        }
    }
#endif    
}
#endif // #if (PDL_INTERRUPT_ENABLE_MFS4 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS12 == PDL_ON)

#if (PDL_INTERRUPT_ENABLE_MFS5 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS13 == PDL_ON)
/**
 ******************************************************************************
 ** \brief MFS ch.5/13 RX IRQ handler
 ******************************************************************************
 */
void MFS5_13_RX_IRQHandler(void)
{
#if (PDL_INTERRUPT_ENABLE_MFS5 == PDL_ON)    
    if(0 != bFM0P_INTREQ_IRQ17MON_MFSINT0)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs5].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerRx((stc_mfsn_uart_t*)&UART5, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs5].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerRx((stc_mfsn_csio_t*)&CSIO5, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs5].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerRx((stc_mfsn_i2c_t*)&I2C5, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs5].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerRx((stc_mfsn_lin_t*)&LIN5, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs5].stcInternData));
                break;
            default:
                break;
        
        }  
    }
#endif
#if (PDL_INTERRUPT_ENABLE_MFS13 == PDL_ON)        
    if(0 != bFM0P_INTREQ_IRQ17MON_MFSINT1)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs13].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerRx((stc_mfsn_uart_t*)&UART13, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs13].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerRx((stc_mfsn_csio_t*)&CSIO13, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs13].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerRx((stc_mfsn_i2c_t*)&I2C13, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs13].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerRx((stc_mfsn_lin_t*)&LIN13, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs13].stcInternData));
                break;
            default:
                break;
        
        }  
    }
#endif    
}

/**
 ******************************************************************************
 ** \brief MFS ch.5/13 TX IRQ handler
 ******************************************************************************
 */
void MFS5_13_TX_IRQHandler(void)
{
#if (PDL_INTERRUPT_ENABLE_MFS5 == PDL_ON)    
    if(0 != bFM0P_INTREQ_IRQ18MON_MFSINT0)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs5].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerTx((stc_mfsn_uart_t*)&UART5, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs5].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerTx((stc_mfsn_csio_t*)&CSIO5, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs5].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerTx((stc_mfsn_i2c_t*)&I2C5, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs5].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerTx((stc_mfsn_lin_t*)&LIN5, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs5].stcInternData));
                break;
            default:
                break;
        }
    }
    if(0 != bFM0P_INTREQ_IRQ18MON_MFSINT1)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs5].stcInternData.enMode)
        {
            case MfsUartMode:
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerStatus((stc_mfsn_csio_t*)&CSIO5, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs5].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerStatus((stc_mfsn_i2c_t*)&I2C5, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs5].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerStatus((stc_mfsn_lin_t*)&LIN5, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs5].stcInternData));
                break;
            default:
                break;
        
        } 
    }
#endif
#if (PDL_INTERRUPT_ENABLE_MFS13 == PDL_ON)        
    if(0 != bFM0P_INTREQ_IRQ18MON_MFSINT2)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs13].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerTx((stc_mfsn_uart_t*)&UART13, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs13].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerTx((stc_mfsn_csio_t*)&CSIO13, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs13].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerTx((stc_mfsn_i2c_t*)&I2C13, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs13].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerTx((stc_mfsn_lin_t*)&LIN13, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs13].stcInternData));
                break;
            default:
                break;
        }
    }
    if(0 != bFM0P_INTREQ_IRQ18MON_MFSINT3)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs13].stcInternData.enMode)
        {
            case MfsUartMode:
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerStatus((stc_mfsn_csio_t*)&CSIO13, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs13].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerStatus((stc_mfsn_i2c_t*)&I2C13, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs13].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerStatus((stc_mfsn_lin_t*)&LIN13, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs13].stcInternData));
                break;
            default:
                break;
        
        }
    }
#endif    
}
#endif // #if (PDL_INTERRUPT_ENABLE_MFS5 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS13 == PDL_ON)

#if (PDL_INTERRUPT_ENABLE_MFS6 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS14 == PDL_ON) ||  \
    (PDL_INTERRUPT_ENABLE_DMA0 == PDL_ON) || (PDL_INTERRUPT_ENABLE_DMA1 == PDL_ON)   
/**
 ******************************************************************************
 ** \brief MFS ch.6/14 RX, DMA ch. 0 IRQ handler
 ******************************************************************************
 */
void MFS6_14_RX_DMA0_IRQHandler(void)
{
#if (PDL_INTERRUPT_ENABLE_MFS6 == PDL_ON)    
    if(0 != bFM0P_INTREQ_IRQ19MON_MFSINT0)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs6].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerRx((stc_mfsn_uart_t*)&UART6, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs6].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerRx((stc_mfsn_csio_t*)&CSIO6, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs6].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerRx((stc_mfsn_i2c_t*)&I2C6, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs6].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerRx((stc_mfsn_lin_t*)&LIN6, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs6].stcInternData));
                break;
            default:
                break;
        
        }  
    }
#endif
#if (PDL_INTERRUPT_ENABLE_MFS14 == PDL_ON)        
    if(0 != bFM0P_INTREQ_IRQ19MON_MFSINT1)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs14].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerRx((stc_mfsn_uart_t*)&UART14, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs14].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerRx((stc_mfsn_csio_t*)&CSIO14, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs14].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerRx((stc_mfsn_i2c_t*)&I2C14, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs14].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerRx((stc_mfsn_lin_t*)&LIN14, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs14].stcInternData));
                break;
            default:
                break;
        
        }  
    }
#endif    
#if (PDL_INTERRUPT_ENABLE_DMA0 == PDL_ON)        
    if(0 != bFM0P_INTREQ_IRQ19MON_DMAINT0)
    {
        DmaIrqHandler(0);
    }
#endif 
}

/**
 ******************************************************************************
 ** \brief MFS ch.6/14 TX, DMA ch. 1 IRQ handler
 ******************************************************************************
 */
void MFS6_14_TX_DMA1_IRQHandler(void)
{
#if (PDL_INTERRUPT_ENABLE_MFS6 == PDL_ON)    
    if(0 != bFM0P_INTREQ_IRQ20MON_MFSINT0)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs6].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerTx((stc_mfsn_uart_t*)&UART6, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs6].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerTx((stc_mfsn_csio_t*)&CSIO6, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs6].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerTx((stc_mfsn_i2c_t*)&I2C6, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs6].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerTx((stc_mfsn_lin_t*)&LIN6, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs6].stcInternData));
                break;
            default:
                break;
        }
    }
    if(0 != bFM0P_INTREQ_IRQ20MON_MFSINT1)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs6].stcInternData.enMode)
        {
            case MfsUartMode:
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerStatus((stc_mfsn_csio_t*)&CSIO6, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs6].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerStatus((stc_mfsn_i2c_t*)&I2C6, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs6].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerStatus((stc_mfsn_lin_t*)&LIN6, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs6].stcInternData));
                break;
            default:
                break;
        
        } 
    }
#endif
#if (PDL_INTERRUPT_ENABLE_MFS14 == PDL_ON)        
    if(0 != bFM0P_INTREQ_IRQ20MON_MFSINT2)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs14].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerTx((stc_mfsn_uart_t*)&UART14, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs14].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerTx((stc_mfsn_csio_t*)&CSIO14, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs14].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerTx((stc_mfsn_i2c_t*)&I2C14, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs14].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerTx((stc_mfsn_lin_t*)&LIN14, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs14].stcInternData));
                break;
            default:
                break;
        }
    }
    if(0 != bFM0P_INTREQ_IRQ20MON_MFSINT3)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs14].stcInternData.enMode)
        {
            case MfsUartMode:
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerStatus((stc_mfsn_csio_t*)&CSIO14, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs14].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerStatus((stc_mfsn_i2c_t*)&I2C14, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs14].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerStatus((stc_mfsn_lin_t*)&LIN14, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs14].stcInternData));
                break;
            default:
                break;
        
        }
    }
#endif    
#if (PDL_INTERRUPT_ENABLE_DMA1 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ20MON_DMAINT1)
    {
        DmaIrqHandler(1);
    }    
#endif    
    
}
#endif // #if (PDL_INTERRUPT_ENABLE_MFS6 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS14 == PDL_ON)

#if (PDL_INTERRUPT_ENABLE_MFS7 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS15 == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_DMA2 == PDL_ON) || (PDL_INTERRUPT_ENABLE_DMA3 == PDL_ON) 

/**
 ******************************************************************************
 ** \brief MFS ch.7/15 RX, DMA ch. 2 IRQ handler
 ******************************************************************************
 */
void MFS7_15_RX_DMA2_IRQHandler(void)
{
#if (PDL_INTERRUPT_ENABLE_MFS7 == PDL_ON)    
    if(0 != bFM0P_INTREQ_IRQ21MON_MFSINT0)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs7].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerRx((stc_mfsn_uart_t*)&UART7, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs7].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerRx((stc_mfsn_csio_t*)&CSIO7, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs7].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerRx((stc_mfsn_i2c_t*)&I2C7, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs7].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerRx((stc_mfsn_lin_t*)&LIN7, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs7].stcInternData));
                break;
            default:
                break;
        
        }   
    }
#endif
#if (PDL_INTERRUPT_ENABLE_MFS15 == PDL_ON)        
    if(0 != bFM0P_INTREQ_IRQ21MON_MFSINT1)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs15].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerRx((stc_mfsn_uart_t*)&UART15, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs15].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerRx((stc_mfsn_csio_t*)&CSIO15, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs15].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerRx((stc_mfsn_i2c_t*)&I2C15, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs15].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerRx((stc_mfsn_lin_t*)&LIN15, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs15].stcInternData));
                break;
            default:
                break;
        
        }   
    }
#endif    
#if (PDL_INTERRUPT_ENABLE_DMA2 == PDL_ON)        
    if(0 != bFM0P_INTREQ_IRQ21MON_DMAINT2)
    {
        DmaIrqHandler(2);
    }
#endif 
}

/**
 ******************************************************************************
 ** \brief MFS ch.7/15 TX, DMA ch. 3 IRQ handler
 ******************************************************************************
 */
void MFS7_15_TX_DMA3_IRQHandler(void)
{
#if (PDL_INTERRUPT_ENABLE_MFS7 == PDL_ON)    
    if(0 != bFM0P_INTREQ_IRQ22MON_MFSINT0)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs7].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerTx((stc_mfsn_uart_t*)&UART7, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs7].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerTx((stc_mfsn_csio_t*)&CSIO7, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs7].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerTx((stc_mfsn_i2c_t*)&I2C7, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs7].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerTx((stc_mfsn_lin_t*)&LIN7, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs7].stcInternData));
                break;
            default:
                break;
        }
    }
    if(0 != bFM0P_INTREQ_IRQ22MON_MFSINT1)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs7].stcInternData.enMode)
        {
            case MfsUartMode:
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerStatus((stc_mfsn_csio_t*)&CSIO7, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs7].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerStatus((stc_mfsn_i2c_t*)&I2C7, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs7].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerStatus((stc_mfsn_lin_t*)&LIN7, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs7].stcInternData));
                break;
            default:
                break;
        
        }
    }
#endif
#if (PDL_INTERRUPT_ENABLE_MFS15 == PDL_ON)        
    if(0 != bFM0P_INTREQ_IRQ22MON_MFSINT2)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs15].stcInternData.enMode)
        {
            case MfsUartMode:
                MfsUartIrqHandlerTx((stc_mfsn_uart_t*)&UART15, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs15].stcInternData));
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerTx((stc_mfsn_csio_t*)&CSIO15, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs15].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerTx((stc_mfsn_i2c_t*)&I2C15, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs15].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerTx((stc_mfsn_lin_t*)&LIN15, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs15].stcInternData));
                break;
            default:
                break;
        }
    }
    if(0 != bFM0P_INTREQ_IRQ22MON_MFSINT3)
    {
        switch(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs15].stcInternData.enMode)
        {
            case MfsUartMode:
                break;
            case MfsCsioMode:
                MfsCsioIrqHandlerStatus((stc_mfsn_csio_t*)&CSIO15, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs15].stcInternData));
                break;          
            case MfsI2cMode:
                MfsI2cIrqHandlerStatus((stc_mfsn_i2c_t*)&I2C15, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs15].stcInternData));
                break;          
            case MfsLinMode:
                MfsLinIrqHandlerStatus((stc_mfsn_lin_t*)&LIN15, &(m_astcMfsInstanceDataLut[MfsInstanceIndexMfs15].stcInternData));
                break;
            default:
                break;
        
        }
    }
#endif    
#if (PDL_INTERRUPT_ENABLE_DMA3 == PDL_ON)
    if(0 != bFM0P_INTREQ_IRQ22MON_DMAINT3)
    {
        DmaIrqHandler(3);
    }    
#endif    
    
}
#endif // #if (PDL_INTERRUPT_ENABLE_MFS7 == PDL_ON) || (PDL_INTERRUPT_ENABLE_MFS15 == PDL_ON)

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
