/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file lvd.h
 **
 ** Headerfile for LVD functions
 **  
 ** History:
 **   - 2014-03-03  1.0  Edison Zhang   First version.
 **
 ******************************************************************************/

#ifndef __LVD_H__
#define __LVD_H__

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "mcu.h"
#include "pdl_user.h"

#if (defined(PDL_PERIPHERAL_LVD_ACTIVE))

/* C binding of definitions if building with C++ compiler */
#ifdef __cplusplus
extern "C"
{
#endif

/**
 ******************************************************************************
 ** \defgroup LvdGroup Low Voltage Detection (LVD)
 **
 ** Provided functions of LPM module:
 **
 ** - Lvd_SetIntDetectVoltage()
 ** - Lvd_SetResetDetectVoltage()
 ** - Lvd_EnableFunc()
 ** - Lvd_DisableFunc()
 ** - Lvd_GetIntStatus()
 ** - Lvd_ClearIntStatus()  
 ** - Lvd_ConfigInt()     
 ** - LvdIrqHandler()     
 **    
 ** Lvd_SetIntDetectVoltage() sets voltage threshold to generate LVD interrupt,
 ** which can be selected by the parameter Lvd_SetIntDetectVoltage#enVolt.
 **
 ** Lvd_SetResetDetectVoltage() sets voltage threshold to generate LVD reset,
 ** which can be selected by the parameter Lvd_SetResetDetectVoltage#enVolt. 
 **
 ** Lvd_EnableFunc() enables the LVD interrupt or reset function and 
 ** Lvd_DisableFunc() disables the LVD interrupt or reset function.
 **
 ** Lvd_GetIntStatus() gets the interrupt flag of LVD interrupt and  
 ** Lvd_ClearIntStatus() clears the interrupt flag of LVD interrupt.
 **
 ** Lvd_ConfigInt() enables or disables the LVD interrupt.
 **
 ** LvdIrqHandler() is LVD interrupt IRQ, which is called by interrupts.c   
 **
 ******************************************************************************/
//@{

/******************************************************************************/
/* Global pre-processor symbols/macros ('#define')                            */
/******************************************************************************/
 
/******************************************************************************
 * Global type definitions
 ******************************************************************************/

/**
 ******************************************************************************
 ** \brief Interruption voltage settings for Low Voltage Detection
 **
 ** \note The enumerated values do not correspond with the bit patterns of
 **       SVHI for upward compatibility reasons.
 ******************************************************************************/  
typedef enum en_lvd_int_volt
{
    LvdIntVoltage280 = 0,   ///< Interrupt when voltage is vicinity of 2.80 volts
    LvdIntVoltage300 = 1,   ///< Interrupt when voltage is vicinity of 3.00 volts
    LvdIntVoltage320 = 2,   ///< Interrupt when voltage is vicinity of 3.20 volts
    LvdIntVoltage360 = 3,   ///< Interrupt when voltage is vicinity of 3.60 volts
    LvdIntVoltage370 = 4,   ///< Interrupt when voltage is vicinity of 3.70 volts
    LvdIntVoltage400 = 5,   ///< Interrupt when voltage is vicinity of 4.00 volts
    LvdIntVoltage410 = 6,   ///< Interrupt when voltage is vicinity of 4.10 volts
    LvdIntVoltage420 = 7,   ///< Interrupt when voltage is vicinity of 4.20 volts
      
} en_lvd_int_volt_t;

/**
 ******************************************************************************
 ** \brief Reset voltage settings for Low Voltage Detection
 **
 ** \note The enumerated values do not correspond with the bit patterns of
 **       SVHR for upward compatibility reasons.
 ******************************************************************************/ 
typedef enum en_lvd_reset_volt
{
    LvdResetVoltage245 = 0,   ///< Reset when voltage is vicinity of 2.45 volts
    LvdResetVoltage260 = 1,   ///< Reset when voltage is vicinity of 2.60 volts
    LvdResetVoltage270 = 2,   ///< Reset when voltage is vicinity of 2.70 volts
    LvdResetVoltage280 = 3,   ///< Reset when voltage is vicinity of 2.80 volts
    LvdResetVoltage300 = 4,   ///< Reset when voltage is vicinity of 3.00 volts
    LvdResetVoltage320 = 5,   ///< Reset when voltage is vicinity of 3.20 volts
    LvdResetVoltage360 = 6,   ///< Reset when voltage is vicinity of 3.60 volts
    LvdResetVoltage370 = 7,   ///< Reset when voltage is vicinity of 3.70 volts
    LvdResetVoltage400 = 8,   ///< Reset when voltage is vicinity of 4.00 volts
    LvdResetVoltage410 = 9,   ///< Reset when voltage is vicinity of 4.10 volts
    LvdResetVoltage420 = 10,  ///< Reset when voltage is vicinity of 4.20 volts
      
} en_lvd_reset_volt_t;

/**
 ******************************************************************************
 ** \brief LVD functions
 ******************************************************************************/ 
typedef enum en_lvd_func
{
    LvdIntFunc   = 0u,   ///< Interrupt function of LVD
    LvdResetFunc = 1u,   ///< Reset function of LVD
     
}en_lvd_func_t;

/******************************************************************************/
/* Global variable definitions ('extern')                                     */
/******************************************************************************/

/******************************************************************************/
/* Global function prototypes (definition in C source)                        */
/******************************************************************************/
#if (PDL_INTERRUPT_ENABLE_LVD == PDL_ON)
void LvdIrqHandler(void);
en_result_t Lvd_ConfigInt(func_ptr_t pfnCallback);
#endif

en_result_t Lvd_SetIntDetectVoltage(en_lvd_int_volt_t enVolt);
en_result_t Lvd_SetResetDetectVoltage(en_lvd_reset_volt_t enVolt);
en_result_t Lvd_EnableFunc(en_lvd_func_t enFunc);
en_result_t Lvd_DisableFunc(en_lvd_func_t enFunc);
boolean_t Lvd_GetIntStatus(void);
void Lvd_ClearIntStatus(void);

//@} // LvdGroup

#ifdef __cplusplus
}
#endif

#endif /* #if (defined(PDL_PERIPHERAL_LVD_ACTIVE)) */

#endif /* __LVD_H__ */
/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
