/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file mft_icu.c
 **
 ** A detailed description is available at 
 ** @link IcuGroup ICU Module description @endlink
 **
 ** History:
 **   - 2014-01-10  1.0  Edison Zhang First version.
 **   - 2014-01-21  1.1  Edison Zhang Change the "MftIcuGetInternDataPtr" 
 **                                   location to avoid warning when it is unused.
 **   - 2014-05-14  1.2  Update some comments and add some parameter check.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "mft_icu.h"

#if (defined(PDL_PERIPHERAL_MFT_ICU_ACTIVE))
   
/**
 ******************************************************************************
 ** \addtogroup IcuGroup
 ******************************************************************************/
//@{   
/******************************************************************************/
/* Local pre-processor symbols/macros ('#define')                             */
/******************************************************************************/

/******************************************************************************/
/* Global variable definitions (declared in header file with 'extern')        */
/******************************************************************************/
stc_mft_icu_instance_data_t m_astcMftIcuInstanceDataLut[MFT_ICU_INSTANCE_COUNT] =
{
#if (PDL_PERIPHERAL_ENABLE_MFT0_ICU == PDL_ON)
    { 
        &MFT0_ICU,    // pstcInstance
        {NULL,NULL,NULL,NULL}          // stcInternData (not initialized yet)
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_MFT1_ICU == PDL_ON)
    { 
        &MFT1_ICU,    // pstcInstance
        {NULL,NULL,NULL,NULL}          // stcInternData (not initialized yet)
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_MFT2_ICU == PDL_ON)
    { 
        &MFT2_ICU,    // pstcInstance
        {NULL,NULL,NULL,NULL}          // stcInternData (not initialized yet)
    },
#endif
};

/******************************************************************************/
/* Local type definitions ('typedef')                                         */
/******************************************************************************/

/******************************************************************************/
/* Local function prototypes ('static')                                       */
/******************************************************************************/
#if (PDL_INTERRUPT_ENABLE_MFT0_ICU == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT1_ICU == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT2_ICU == PDL_ON)
static stc_mft_icu_intern_data_t* MftIcuGetInternDataPtr(volatile stc_mftn_icu_t *pstcMftIcu);
static void Mft_Icu_InitIrq( volatile stc_mftn_icu_t* pstcMftIcu );
static void Mft_Icu_DeInitIrq( volatile stc_mftn_icu_t* pstcMftIcu );
#endif
/******************************************************************************/
/* Local variable definitions ('static')                                      */
/******************************************************************************/

/******************************************************************************/
/* Function implementation - global ('extern') and local ('static')           */
/******************************************************************************/

#if (PDL_INTERRUPT_ENABLE_MFT0_ICU == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT1_ICU == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT2_ICU == PDL_ON)
/**
 *****************************************************************************
 ** \brief Return the internal data for a certain MFT_ICU instance.
 **
 ** \param pstcMftIcu Pointer to MftIcu instance
 **
 ** \return Pointer to internal data or NULL if instance is not enabled (or not known)
 **
 *****************************************************************************/
static stc_mft_icu_intern_data_t* MftIcuGetInternDataPtr(volatile stc_mftn_icu_t *pstcMftIcu) 
{
    uint8_t u8Instance;
   
    for (u8Instance = 0u; u8Instance < MFT_ICU_INSTANCE_COUNT; u8Instance++)
    {
        if (pstcMftIcu == m_astcMftIcuInstanceDataLut[u8Instance].pstcInstance)
        {
            return &m_astcMftIcuInstanceDataLut[u8Instance].stcInternData;
        }
    }

    return NULL;
}		
		
/**
 ******************************************************************************
 ** \brief Device Interrupt handler
 **
 ** \param [in] pstcMftIcu            Pointer to ICU instance
 ** \param [in] pstcMftIcuInternData  Pointer to intern data
 **
 ** \return none
 ******************************************************************************/
void Mft_Icu_IrqHandler( volatile stc_mftn_icu_t* pstcMftIcu, 
                         stc_mft_icu_intern_data_t* pstcMftIcuInternData)
{
    // ICU interrupt 0
    if(TRUE == pstcMftIcu->ICSA10_f.ICP0)
    {
        pstcMftIcu->ICSA10_f.ICP0 = 0u;
        if (pstcMftIcuInternData->pfnIcu0Callback != NULL)
        {
             pstcMftIcuInternData->pfnIcu0Callback();
        }
    }
    // ICU interrupt 1
    if(TRUE == pstcMftIcu->ICSA10_f.ICP1)
    {
        pstcMftIcu->ICSA10_f.ICP1 = 0u;
        if (pstcMftIcuInternData->pfnIcu1Callback != NULL)
        {
             pstcMftIcuInternData->pfnIcu1Callback();
        }
    }
    // ICU interrupt 2
    if(TRUE == pstcMftIcu->ICSA32_f.ICP0)
    {
        pstcMftIcu->ICSA32_f.ICP0 = 0u;
        if (pstcMftIcuInternData->pfnIcu2Callback != NULL)
        {
             pstcMftIcuInternData->pfnIcu2Callback();
        }
    }
    // ICU interrupt 3
    if(TRUE == pstcMftIcu->ICSA32_f.ICP1)
    {
        pstcMftIcu->ICSA32_f.ICP1 = 0u;
        if (pstcMftIcuInternData->pfnIcu3Callback != NULL)
        {
             pstcMftIcuInternData->pfnIcu3Callback();
        }
    }
}

/**
 ******************************************************************************
 ** \brief Device dependent initialization of interrupts according CMSIS with
 **        level defined in pdl.h
 **
 ** \param  pstcMftIcu Pointer to ICU instance
 ******************************************************************************/
static void Mft_Icu_InitIrq( volatile stc_mftn_icu_t* pstcMftIcu ) 
{
    if ((pstcMftIcu == (volatile stc_mftn_icu_t*)(&MFT0_ICU)) || 
        (pstcMftIcu == (volatile stc_mftn_icu_t*)(&MFT1_ICU)) ||
        (pstcMftIcu == (volatile stc_mftn_icu_t*)(&MFT2_ICU)))  
    {
        NVIC_ClearPendingIRQ(INCAP_IRQn);
        NVIC_EnableIRQ(INCAP_IRQn);
        NVIC_SetPriority(INCAP_IRQn, PDL_IRQ_LEVEL_MFT_ICU);   
    }
    return;
}

/**
 ******************************************************************************
 ** \brief Device dependent de-initialization of interrupts according CMSIS 
 **
 ** \param [in] pstcMftIcu Pointer to ICU instance
 ******************************************************************************/
static void Mft_Icu_DeInitIrq( volatile stc_mftn_icu_t* pstcMftIcu ) 
{
    if ((pstcMftIcu == (volatile stc_mftn_icu_t*)(&MFT0_ICU)) || 
        (pstcMftIcu == (volatile stc_mftn_icu_t*)(&MFT1_ICU)) ||
        (pstcMftIcu == (volatile stc_mftn_icu_t*)(&MFT2_ICU)))  
    {
        if(pstcMftIcu->ICSA10_f.ICE0 == 1u)
        {
            return;
        }
        if(pstcMftIcu->ICSA10_f.ICE1 == 1u)
        {
            return;
        }
        if(pstcMftIcu->ICSA32_f.ICE0 == 1u)
        {
            return;
        }
        if(pstcMftIcu->ICSA32_f.ICE1 == 0u)
        {
            return;
        }
        NVIC_ClearPendingIRQ(INCAP_IRQn);
        NVIC_DisableIRQ(INCAP_IRQn);
        NVIC_SetPriority(INCAP_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
    }
    
    return;
}

#endif

/**
 ******************************************************************************
 ** \brief  Select FRTx channel to connect to ICUx
 **
 ** \param [in] pstcMftIcu    Pointer to ICU instance
 ** \param [in] u8Ch       Mft Icu channel
 ** \arg        MFT_ICU_CH0 ~ MFT_ICU_CH3
 ** \param [in] enFrt      Frt channel number
 ** \arg        Frt0ToIcu,
 **             Frt1ToIcu,
 **             Frt2ToIcu,
 **             IcuFrtToExt0,
 **             IcuFrtToExt1
 ** \retval Ok                     FRT channel selected 
 ** \retval ErrorInvalidParameter  If one of following conditions are met:
 **                                - NULL == pstcMftIcu
 **                                - u8Ch > MFT_ICU_CHx_MAX
 **                                - enFrt > IcuFrtToExt1
 ******************************************************************************/
en_result_t Mft_Icu_SelFrt( volatile stc_mftn_icu_t* pstcMftIcu, uint8_t u8Ch, 
                            en_mft_icu_frt_t enFrt)
{
    volatile stc_mft_icu_icfs10_field_t* stcICFS10;
    volatile stc_mft_icu_icfs32_field_t* stcICFS32;

    // Check for NULL pointer and channel parameter
    boolean_t bAvoidSideEffects  = ((NULL == pstcMftIcu) ? 1u : 0u);
              bAvoidSideEffects |= ((u8Ch > MFT_ICU_CHx_MAX) ? 1u : 0u);
    if (TRUE == bAvoidSideEffects)
    {
        return ErrorInvalidParameter ;
    }

    // Get actual address of register list of current channel
    stcICFS10 = (volatile stc_mft_icu_icfs10_field_t*)(&(pstcMftIcu->ICFS10));
    stcICFS32 = (volatile stc_mft_icu_icfs32_field_t*)(&(pstcMftIcu->ICFS32));
    
    // FRT channel select: FRTx -> ICUx
    if(enFrt > IcuFrtToExt1)
    {
        return ErrorInvalidParameter;
    }
    // configure the Frt channel to connect to Icu
    switch(u8Ch)
    {
        // ICU channel 0
        case MFT_ICU_CH0: 
            stcICFS10->FSI0 = (uint8_t)enFrt;           
            break;
        // ICU channel 1
        case MFT_ICU_CH1:            
            stcICFS10->FSI1 = (uint8_t)enFrt;
            break;
        // ICU channel 2
        case MFT_ICU_CH2:           
            stcICFS32->FSI0 = (uint8_t)enFrt;
            break;
        // ICU channel 3
        case MFT_ICU_CH3:
            stcICFS32->FSI1 = (uint8_t)enFrt;
            break;
        default:
            return ErrorInvalidParameter;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief  Configure ICU module detection mode(different edge)
 **
 ** \param [in] pstcMftIcu    Pointer to ICU instance
 ** \param  [in] u8Ch       Mft Icu channel
 ** \arg        MFT_ICU_CH0 ~ MFT_ICU_CH3
 ** \param  [in] enMode     Icu detect mode
 ** \arg        IcuDisable,
 **             IcuRisingDetect,
 **             IcuFallingDetect,
 **             IcuBothDetect
 ** \retval Ok                     ICU detection mode selected
 ** \retval ErrorInvalidParameter  If one of following conditions are met:
 **                                - NULL == pstcMftIcu
 **                                - u8Ch > MFT_ICU_CHx_MAX
 **                                - enMode > IcuBothDetect
 ******************************************************************************/

en_result_t Mft_Icu_ConfigDetectMode( volatile stc_mftn_icu_t* pstcMftIcu, uint8_t u8Ch, 
                                      en_mft_icu_mode_t enMode)
{
    // Check for NULL pointer and channel parameter
    boolean_t bAvoidSideEffects  = ((NULL == pstcMftIcu) ? 1u : 0u);
    bAvoidSideEffects |= ((u8Ch > MFT_ICU_CHx_MAX) ? 1u : 0u);
    if (bAvoidSideEffects)
    {
        return ErrorInvalidParameter ;
    }

    // Check configuration of Mft Icu mode
    if(enMode > IcuBothDetect)
    {
        return ErrorInvalidParameter;
    }
    // Configure Icu detection mode
    switch (u8Ch)
    {
        // ICU channel 0
        case MFT_ICU_CH0:            
            pstcMftIcu->ICSA10_f.EG0 = (uint8_t)enMode;
            break;
        // ICU channel 1
        case MFT_ICU_CH1:
            pstcMftIcu->ICSA10_f.EG1 = (uint8_t)enMode;
            break;
        // ICU channel 2
        case MFT_ICU_CH2:
            pstcMftIcu->ICSA32_f.EG0 = (uint8_t)enMode;
            break;
        // ICU channel 3
        case MFT_ICU_CH3:
            pstcMftIcu->ICSA32_f.EG1 = (uint8_t)enMode;
            break;
        // ICU channel number error
        default:
            return ErrorInvalidParameter;
    }
    return Ok;
}

#if (PDL_INTERRUPT_ENABLE_MFT0_ICU == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT1_ICU == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT2_ICU == PDL_ON)
/**
 ******************************************************************************
 ** \brief  Enable Mft Icu interrupt
 **
 ** \param [in] pstcMftIcu    Pointer to ICU instance
 ** \param  [in] u8Ch         Mft Icu channel
 ** \arg        MFT_ICU_CH0 ~ MFT_ICU_CH3
 ** \param  [in] pfnCallback  register callback function
 ** \arg        user interrupt application function
 ** \retval Ok                     ICU interrupt are enabled
 ** \retval ErrorInvalidParameter  If one of following conditions are met:
 **                                - NULL == pstcMftIcu
 **                                - u8Ch > MFT_ICU_CHx_MAX
 **                                - Other invalid configuration
 ******************************************************************************/

en_result_t Mft_Icu_EnableInt( volatile stc_mftn_icu_t*pstcMftIcu, uint8_t u8Ch, 
                               func_ptr_t pfnCallback)
{
    // Pointer to internal data
    stc_mft_icu_intern_data_t* pstcMftIcuInternData ;  
    volatile stc_mftn_icu_t* pstcOrgMftIcu = pstcMftIcu;

    // Check for NULL pointer and channel parameter
    boolean_t bAvoidSideEffects  = ((NULL == pstcMftIcu) ? 1u : 0u);
    bAvoidSideEffects |= ((u8Ch > MFT_ICU_CHx_MAX) ? 1u : 0u);
    bAvoidSideEffects |= ((NULL == pfnCallback) ? 1u : 0u);
    if (bAvoidSideEffects)
    {
        return ErrorInvalidParameter ;
    }

    // Get pointer to internal data structure
    pstcMftIcuInternData = MftIcuGetInternDataPtr( pstcOrgMftIcu );
    
    switch (u8Ch)
    {
        // ICU channel 0
        case MFT_ICU_CH0:
            pstcMftIcuInternData->pfnIcu0Callback = pfnCallback;
            pstcMftIcu->ICSA10_f.ICE0 = 1u;
            break;
        // ICU channel 1
        case MFT_ICU_CH1:
            pstcMftIcuInternData->pfnIcu1Callback = pfnCallback;
            pstcMftIcu->ICSA10_f.ICE1 = 1u;
            break;
        // ICU channel 2
        case MFT_ICU_CH2:
            pstcMftIcuInternData->pfnIcu2Callback = pfnCallback;
            pstcMftIcu->ICSA32_f.ICE0 = 1u;
            break;
        // ICU channel 3
        case MFT_ICU_CH3:
            pstcMftIcuInternData->pfnIcu3Callback = pfnCallback;
            pstcMftIcu->ICSA32_f.ICE1 = 1u;
            break;
        // ICU channel number error
        default:
            return ErrorInvalidParameter;
    }

    // initialize interrupt
    Mft_Icu_InitIrq(pstcOrgMftIcu);
    return Ok;
}

/**
 ******************************************************************************
 ** \brief  Disable Mft Icu interrupt and release callback function
 **
 ** \param [in] pstcMftIcu    Pointer to ICU instance
 ** \param  [in] u8Ch         Mft Icu channel
 ** \arg        MFT_ICU_CH0 ~ MFT_ICU_CH3
 **
 ** \retval Ok                     ICU interrupt are disabled
 ** \retval ErrorInvalidParameter  If one of following conditions are met:
 **                                - NULL == pstcMftIcu
 **                                - u8Ch > MFT_ICU_CHx_MAX
 **                                - Other invalid configuration
 ******************************************************************************/

en_result_t Mft_Icu_DisableInt( volatile stc_mftn_icu_t*pstcMftIcu, uint8_t u8Ch)
{
    // Pointer to internal data
    stc_mft_icu_intern_data_t* pstcMftIcuInternData ;  
    volatile stc_mftn_icu_t* pstcOrgMftIcu = pstcMftIcu;

    // Check for NULL pointer and channel parameter
    boolean_t bAvoidSideEffects  = ((NULL == pstcMftIcu) ? 1u : 0u);
              bAvoidSideEffects |= ((u8Ch > MFT_ICU_CHx_MAX) ? 1u : 0u);
    if (bAvoidSideEffects)
    {
        return ErrorInvalidParameter ;
    }

    // Get pointer to internal data structure
    pstcMftIcuInternData = MftIcuGetInternDataPtr( pstcOrgMftIcu );
    // Dis-register callback function and disable interrupt operation
    switch (u8Ch)
    {
        // ICU channel 0
        case MFT_ICU_CH0:
            pstcMftIcuInternData->pfnIcu0Callback = NULL;
            pstcMftIcu->ICSA10_f.ICE0 = 0u;
            break;
        // ICU channel 1
        case MFT_ICU_CH1:
            pstcMftIcuInternData->pfnIcu1Callback = NULL;
            pstcMftIcu->ICSA10_f.ICE1 = 0u;
            break;
        // ICU channel 2
        case MFT_ICU_CH2:
            pstcMftIcuInternData->pfnIcu2Callback = NULL;
            pstcMftIcu->ICSA32_f.ICE0 = 0u;
            break;
        // ICU channel 3
        case MFT_ICU_CH3:
            pstcMftIcuInternData->pfnIcu3Callback = NULL;
            pstcMftIcu->ICSA32_f.ICE1 = 0u;
            break;
        // ICU channel number error
        default:
            return ErrorInvalidParameter;
    }
   
    Mft_Icu_DeInitIrq(pstcOrgMftIcu);
    return Ok;
}

#endif

/**
 ******************************************************************************
 ** \brief  Get interrupt flag
 **
 ** \param [in] pstcMftIcu    Pointer to ICU instance
 ** \param  [in] u8Ch          Mft Icu channel
 ** \arg        MFT_ICU_CH0 ~ MFT_ICU_CH3
 **
 ** \retval  FALSE ICU active edge is not detected
 ** \retval  TRUE  ICU active edge is detected
 ******************************************************************************/

en_int_flag_t Mft_Icu_GetIntFlag(volatile stc_mftn_icu_t *pstcMftIcu, uint8_t u8Ch)
{
    en_int_flag_t enIntFlagBuf;

    // Read interrupt flag
    switch (u8Ch)
    {
        // ICU channel 0
        case MFT_ICU_CH0:
            (pstcMftIcu->ICSA10_f.ICP0 == 1) ? (enIntFlagBuf = PdlSet) : (enIntFlagBuf = PdlClr);
            break;
        // ICU channel 1
        case MFT_ICU_CH1:
            (pstcMftIcu->ICSA10_f.ICP1 == 1) ? (enIntFlagBuf = PdlSet) : (enIntFlagBuf = PdlClr);
            break;
        // ICU channel 2
        case MFT_ICU_CH2:         
            (pstcMftIcu->ICSA32_f.ICP0 == 1) ? (enIntFlagBuf = PdlSet) : (enIntFlagBuf = PdlClr);
            break;
        // ICU channel 3
        case MFT_ICU_CH3:          
            (pstcMftIcu->ICSA32_f.ICP1 == 1) ? (enIntFlagBuf = PdlSet) : (enIntFlagBuf = PdlClr);
            break;
        // ICU channel number error
        default:
            break;
    }
    // return interrupt flag
    return enIntFlagBuf;
}
/**
 ******************************************************************************
 ** \brief  Clear interrupt flag
 **
 ** \param [in] pstcMftIcu    Pointer to ICU instance
 ** \param  [in] u8Ch          Mft Icu channel
 ** \arg        MFT_ICU_CH0 ~ MFT_ICU_CH3
 **
 ** \retval Ok                     Interrupt flag is clear
 ** \retval ErrorInvalidParameter  If one of following conditions are met:
 **                                - NULL == pstcMftIcu
 **                                - u8Ch > MFT_ICU_CHx_MAX
 **                                - Other invalid configuration
 ******************************************************************************/

en_result_t Mft_Icu_ClrIntFlag( volatile stc_mftn_icu_t *pstcMftIcu, uint8_t u8Ch)
{
    // Check for NULL pointer and channel parameter
    boolean_t bAvoidSideEffects  = ((NULL == pstcMftIcu) ? 1u : 0u);
              bAvoidSideEffects |= ((u8Ch > MFT_ICU_CHx_MAX) ? 1u : 0u);
    if (bAvoidSideEffects)
    {
        return ErrorInvalidParameter ;
    }

	// Clear interrupt flag
    switch (u8Ch)
    {
        // ICU channel 0
        case MFT_ICU_CH0:
            pstcMftIcu->ICSA10_f.ICP0 = 0u;
            break;
        // ICU channel 1
        case MFT_ICU_CH1:
            pstcMftIcu->ICSA10_f.ICP1 = 0u;
            break;
        // ICU channel 2
        case MFT_ICU_CH2:
            pstcMftIcu->ICSA32_f.ICP0 = 0u;
            break;
        // ICU channel 3
        case MFT_ICU_CH3:
            pstcMftIcu->ICSA32_f.ICP1 = 0u;
            break;
        // ICU channel number error
        default:
            return ErrorInvalidParameter;
    }
    
    return Ok;
}
/**
 ******************************************************************************
 ** \brief  Get the latest captured edge type
 **
 ** \param  [in] pstcMftIcu    Pointer to ICU instance
 ** \param  [in] u8Ch          Mft Icu channel
 ** \arg        MFT_ICU_CH0 ~ MFT_ICU_CH3
 **
 ** \return  Detected edge type
 ******************************************************************************/
en_mft_icu_edge_t Mft_Icu_GetLastEdge( volatile stc_mftn_icu_t *pstcMftIcu, uint8_t u8Ch)
{
    en_mft_icu_edge_t enEdgeTypeBuf;

    // Read last Edge type
    switch (u8Ch)
    {
        // ICU channel 0
        case MFT_ICU_CH0:
            (pstcMftIcu->ICSB10_f.IEI0 == 1) ? (enEdgeTypeBuf = IcuRisingEdge) : (enEdgeTypeBuf = IcuFallingEdge);
            break;
        // ICU channel 1
        case MFT_ICU_CH1:
            (pstcMftIcu->ICSB10_f.IEI1 == 1) ? (enEdgeTypeBuf = IcuRisingEdge) : (enEdgeTypeBuf = IcuFallingEdge);
            break;
        // ICU channel 2
        case MFT_ICU_CH2:
            (pstcMftIcu->ICSB32_f.IEI0 == 1) ? (enEdgeTypeBuf = IcuRisingEdge) : (enEdgeTypeBuf = IcuFallingEdge);
            break;
        // ICU channel 3
        case MFT_ICU_CH3:           
            (pstcMftIcu->ICSB32_f.IEI1 == 1) ? (enEdgeTypeBuf = IcuRisingEdge) : (enEdgeTypeBuf = IcuFallingEdge);      
            break;
        // ICU channel number error
        default:
            break;
    }
    // return last edge type
    return enEdgeTypeBuf;
}
/**
 ******************************************************************************
 ** \brief  Readout captured data value
 **
 ** \param  [in] pstcMftIcu    Pointer to ICU instance
 ** \param  [in] u8Ch          Mft Icu channel
 ** \arg        MFT_ICU_CH0 ~ MFT_ICU_CH3
 **
 ** \return  captured data value
 ******************************************************************************/
uint16_t Mft_Icu_GetCaptureData(volatile stc_mftn_icu_t *pstcMftIcu, uint8_t u8Ch)
{
    uint16_t u16CaptureVal;

    // Read captured value
    switch (u8Ch)
    {
        // ICU channel 0
        case MFT_ICU_CH0:
            u16CaptureVal = (uint16_t)(pstcMftIcu->ICCP0);
            break;
        // ICU channel 1
        case MFT_ICU_CH1:
            u16CaptureVal = (uint16_t)(pstcMftIcu->ICCP1);
            break;
        // ICU channel 2
        case MFT_ICU_CH2:
            u16CaptureVal = (uint16_t)(pstcMftIcu->ICCP2);
            break;
        // ICU channel 3
        case MFT_ICU_CH3:
            u16CaptureVal = (uint16_t)(pstcMftIcu->ICCP3);
            break;
        // ICU channel number error
        default:
            break;
    }
    
    return (uint16_t)u16CaptureVal;
}

//@} // IcuGroup

#endif // #if (defined(PDL_PERIPHERAL_MFT_ICU_ACTIVE))
