/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file mft_ocu.c
 **
 ** A detailed description is available at 
 ** @link OcuGroup OCU Module description @endlink
 **  
 ** History:
 **   - 2013-12-21  0.1  Edison Zhang  First version.
 **   - 2014-01-20  0.2  Edison Add a structure for OCSE register setting
 **
 *****************************************************************************/


/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "mft_ocu.h"

#if (defined(PDL_PERIPHERAL_MFT_OCU_ACTIVE))

/**
 ******************************************************************************
 ** \addtogroup OcuGroup
 ******************************************************************************/
//@{

/******************************************************************************/
/* Local pre-processor symbols/macros ('#define')                             */
/******************************************************************************/
/******************************************************************************/
/* Global variable definitions (declared in header file with 'extern')        */
/******************************************************************************/
/// Look-up table for all enabled FRT of MFT  instances and their internal data
stc_mft_ocu_instance_data_t m_astcMftOcuInstanceDataLut[OCU_INSTANCE_COUNT] =
{
#if (PDL_PERIPHERAL_ENABLE_MFT0_OCU == PDL_ON)
    {
        &MFT0_OCU,  // pstcInstance
        {NULL, NULL, NULL, NULL, NULL, NULL}    // stcInternData (not initialized yet)
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_MFT1_OCU == PDL_ON)
    {
        &MFT1_OCU,  // pstcInstance
        {NULL, NULL, NULL, NULL, NULL, NULL}    // stcInternData (not initialized yet)
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_MFT2_OCU == PDL_ON)
    {
        &MFT2_OCU,  // pstcInstance
        {NULL, NULL, NULL, NULL, NULL, NULL}    // stcInternData (not initialized yet)
    },
#endif 
};

/******************************************************************************/
/* Local type definitions ('typedef')                                         */
/******************************************************************************/

/******************************************************************************/
/* Local function prototypes ('static')                                       */
/******************************************************************************/

/******************************************************************************/
/* Local variable definitions ('static')                                      */
/******************************************************************************/

/******************************************************************************/
/* Function implementation - global ('extern') and local ('static')           */
/******************************************************************************/
/**
 *****************************************************************************
 ** \brief Return the internal data for a certain OCU instance.
 **
 ** \param pstcOcu Pointer to OCU instance
 **
 ** \return Pointer to internal data or NULL if instance is not enabled
 **         (or not known)
 **
 *****************************************************************************/
static stc_mft_ocu_intern_data_t* MftGetInternDataPtr(volatile stc_mftn_ocu_t* pstcOcu)
{
    uint32_t u32Instance;

    for (u32Instance = 0; u32Instance < OCU_INSTANCE_COUNT; u32Instance++)
    {
        if (pstcOcu == m_astcMftOcuInstanceDataLut[u32Instance].pstcInstance)
        {
            return &m_astcMftOcuInstanceDataLut[u32Instance].stcInternData;
        }
    }

    return NULL;
}

#if (PDL_INTERRUPT_ENABLE_MFT0_OCU == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT1_OCU == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT2_OCU == PDL_ON)  
/*!
 ******************************************************************************
 ** \brief  OCU module interrupt handler
 **
 ** \param [in] pstcOcu Pointer to OCU instance
 ** \param [in]  pstcMftOcuInternData  callback function of OCU
 *****************************************************************************
*/
void Mft_Ocu_IrqHandler( volatile stc_mftn_ocu_t* pstcOcu,
                         stc_mft_ocu_intern_data_t* pstcMftOcuInternData)
{
    func_ptr_t funCallBack;
    uint32_t* ptemp;
    uint8_t Ch;

    //lookup the handler
    for(Ch=0;Ch<MFT_OCU_MAXCH;Ch++)
    {
        if(PdlSet == Mft_Ocu_GetIntFlag(pstcOcu,Ch))
        {
            //clear the interrupt flag
            Mft_Ocu_ClrIntFlag(pstcOcu,Ch);

            //pointer callback function address
            ptemp = (uint32_t*)&(pstcMftOcuInternData->pfnOcu0Callback);
            funCallBack = (func_ptr_t)(*(ptemp + Ch));
            
            if(funCallBack != NULL)
            {
               funCallBack();
            }
        }
    }    
}

/**
 ******************************************************************************
 ** \brief Device dependent initialization of interrupts according CMSIS with
 **        level defined in pdl.h
 **
 ** \param [in] pstcOcu Pointer to OCU instance
 **
 ** \retval Ok    Successful initialization
 **
 ******************************************************************************/
static void Mft_Ocu_InitIrq( volatile stc_mftn_ocu_t* pstcOcu )
{
    if ((pstcOcu == (volatile stc_mftn_ocu_t*)(&MFT0_OCU)) ||
        (pstcOcu == (volatile stc_mftn_ocu_t*)(&MFT1_OCU)) ||
        (pstcOcu == (volatile stc_mftn_ocu_t*)(&MFT2_OCU)))  
    {
        NVIC_ClearPendingIRQ(OUTCOMP_IRQn);
        NVIC_EnableIRQ(OUTCOMP_IRQn);
        NVIC_SetPriority(OUTCOMP_IRQn, PDL_IRQ_LEVEL_MFT_OCU);
    }

    return ;
}

/**
 ******************************************************************************
 ** \brief Device dependent de-initialization of interrupts according CMSIS
 **
 ** \param [in] pstcOcu Pointer to OCU instance
 **
 ** \retval Ok    Successful initialization
 **
 ******************************************************************************/
static void Mft_Ocu_DeInitIrq( volatile stc_mftn_ocu_t* pstcOcu )
{
    if ((pstcOcu == (volatile stc_mftn_ocu_t*)(&MFT0_OCU)) ||
        (pstcOcu == (volatile stc_mftn_ocu_t*)(&MFT1_OCU)) ||
        (pstcOcu == (volatile stc_mftn_ocu_t*)(&MFT2_OCU)))
    {
        if(pstcOcu->OCSA10_f.IOE0 == 1)
        {
            return;
        }
        if(pstcOcu->OCSA10_f.IOE1 == 1)
        {
            return;
        }
        if(pstcOcu->OCSA32_f.IOE0 == 1)
        {
            return;
        }
        if(pstcOcu->OCSA32_f.IOE1 == 1)
        {
            return;
        }
        if(pstcOcu->OCSA54_f.IOE0 == 1)
        {
            return;
        }
        if(pstcOcu->OCSA54_f.IOE1 == 1)
        {
            return;
        }
        
        NVIC_ClearPendingIRQ(OUTCOMP_IRQn);
        NVIC_DisableIRQ(OUTCOMP_IRQn);
        NVIC_SetPriority(OUTCOMP_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);
    }
  
    return;
}

#endif

/******************************************************************************/
/* Local Functions                                                            */
/******************************************************************************/

/*!
 ******************************************************************************
 ** \brief  Init OCU module
 **
 ** \param [in] pstcOcu Pointer to OCU instance
 ** \param [in]  u8Ch  channel of OCU
 ** \arg    MFT_OCU_CH0 ~ MFT_OCU_CH5
 ** \param [in] pstcOcuConfig configure of OCU
 ** \arg    structure of OCU cofnigure
 **
 ** \retval Ok                    OCU module initialized
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - pstcOcuConfig == NULL
 **                               - Other invalid configuration
 **
 *****************************************************************************
*/
en_result_t Mft_Ocu_Init( volatile stc_mftn_ocu_t* pstcOcu, 
                          uint8_t u8Ch, stc_mft_ocu_config_t*  pstcOcuConfig)
{
    stc_mft_ocu_intern_data_t* pstcOcuInternData;
    volatile uint8_t* pu8OCFS;
    volatile stc_mft_ocu_ocsb10_field_t* pstcOCSB;
    volatile uint8_t* p8OCSC;
    volatile uint8_t* pu8OCSD;

    // Check for NULL pointer and configuration parameter
    if ( (pstcOcu == NULL) || 
         (u8Ch >= MFT_OCU_MAXCH) ||
         (pstcOcuConfig->enFrtConnect > OcuFrtToExt) ||
         (pstcOcuConfig->bFm4 > 1) ||
         (pstcOcuConfig->bCmod > 1) || 
         (pstcOcuConfig->enOccpBufMode > OccpBufTrsfByFrtZeroPeak ) || 
         (pstcOcuConfig->enOcseBufMode > OcseBufTrsfByFrtZeroPeak ) )
    {
        return ErrorInvalidParameter ;
    }
    // Get pointer to internal data structure ...
    pstcOcuInternData = MftGetInternDataPtr( pstcOcu ) ;
    // ... and check for NULL
    if ( pstcOcuInternData != NULL )
    {
    	pstcOcuInternData->pfnOcu0Callback = NULL;
    	pstcOcuInternData->pfnOcu1Callback = NULL;
    	pstcOcuInternData->pfnOcu2Callback = NULL;
    	pstcOcuInternData->pfnOcu3Callback = NULL;
    	pstcOcuInternData->pfnOcu4Callback = NULL;
    	pstcOcuInternData->pfnOcu5Callback = NULL;
    }

    //Get pointer of current channel OCU register address
    pu8OCFS = (volatile uint8_t*)((volatile uint8_t*)pstcOcu + 0x68UL + u8Ch/2);
    pstcOCSB = (volatile stc_mft_ocu_ocsb10_field_t*)((volatile uint8_t*)pstcOcu + 0x19UL + (u8Ch/2)*4);
    p8OCSC = (volatile uint8_t*)((volatile uint8_t*)pstcOcu + 0x25UL);
    pu8OCSD = (volatile uint8_t*)((volatile uint8_t*)pstcOcu + 0x1AUL + (u8Ch/2)*4);

    //set OCSB register, config ocu's operation
    pstcOCSB->CMOD = pstcOcuConfig->bCmod;
    pstcOCSB->FM4 = pstcOcuConfig->bFm4;

    //set OCSC register, when enable FM3 compatible mode
    *p8OCSC = pstcOcuConfig->u8Mod;

    //set OCSD register and RT output level state
    if(0 == (u8Ch % 2)) //channel 0, 2, 4
    {
        *pu8OCSD &= 0x33u; 
        *pu8OCSD |= (uint8_t)pstcOcuConfig->enOccpBufMode;
        *pu8OCSD |= (uint8_t)((uint8_t)pstcOcuConfig->enOcseBufMode << 4);
        *pu8OCFS |= (uint8_t)pstcOcuConfig->enFrtConnect;
        pstcOCSB->OTD0 = pstcOcuConfig->enStatePin;
    }
    else//channel 1, 3, 5
    {
        *pu8OCSD &= 0xCCu; 
        *pu8OCSD |= (uint8_t)((uint8_t)pstcOcuConfig->enOccpBufMode << 2);
        *pu8OCSD |= (uint8_t)((uint8_t)pstcOcuConfig->enOcseBufMode << 6);
        *pu8OCFS |= (uint8_t)((uint8_t)pstcOcuConfig->enFrtConnect << 4);
        pstcOCSB->OTD1 = pstcOcuConfig->enStatePin;
    }

    return Ok;
}

/*!
 ******************************************************************************
 ** \brief  Compare congifuration of even OCU channel
 **
 ** \param [in] pstcOcu Pointer to OCU instance
 ** \param [in]  EvenCh  even channel of OCU
 ** \arg    MFT_OCU_CH0, MFT_OCU_CH2, MFT_OCU_CH4
 ** \param [in] pstcConfig pointer to structure of compare mode
 **
 ** \retval Ok                    Even OCU channel compare mode is set
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - EvenCh out of range
 **                               - Other invalid configuration
 **
 *****************************************************************************
*/
en_result_t Mft_Ocu_SetEvenChCompareMode(volatile stc_mftn_ocu_t* pstcOcu,
                                         uint8_t EvenCh,
                                         stc_even_compare_config_t* pstcConfig)
{
    volatile uint16_t* p16OCSE0;

    // Check for NULL pointer
    if ( (pstcOcu == NULL)
         || (EvenCh%2 != 0) 
         || (pstcConfig == NULL) 
         || (pstcConfig->enFrtZeroEvenMatchEvenChRtStatus > RtOutputReverse)  
         || (pstcConfig->enFrtZeroEvenNotMatchEvenChRtStatus > RtOutputReverse)  
         || (pstcConfig->enFrtUpCntEvenMatchEvenChRtStatus > RtOutputReverse) 
         || (pstcConfig->enFrtPeakEvenMatchEvenChRtStatus > RtOutputReverse) 
         || (pstcConfig->enFrtPeakEvenNotMatchEvenChStatus > RtOutputReverse)  
         || (pstcConfig->enFrtDownCntEvenMatchEvenChRtStatus > RtOutputReverse)  
         || (pstcConfig->enIopFlagWhenFrtZeroEvenMatch > IopFlagSet)  
         || (pstcConfig->enIopFlagWhenFrtUpCntEvenMatch > IopFlagSet) 
         || (pstcConfig->enIopFlagWhenFrtPeakEvenMatch > IopFlagSet)  
         || (pstcConfig->enIopFlagWhenFrtDownCntEvenMatch > IopFlagSet)  )
    {
        return ErrorInvalidParameter ;
    }
    
    p16OCSE0 = (volatile uint16_t*)((volatile uint8_t*)pstcOcu + 0x28UL + EvenCh*4);
    *p16OCSE0 = 0;
    *p16OCSE0 |= (uint16_t)((uint16_t)pstcConfig->enFrtZeroEvenMatchEvenChRtStatus    << 10u);
    *p16OCSE0 |= (uint16_t)((uint16_t)pstcConfig->enFrtZeroEvenNotMatchEvenChRtStatus << 14u); 
    *p16OCSE0 |= (uint16_t)((uint16_t)pstcConfig->enFrtUpCntEvenMatchEvenChRtStatus   << 8u);
    *p16OCSE0 |= (uint16_t)((uint16_t)pstcConfig->enFrtPeakEvenMatchEvenChRtStatus    << 6u);                   
    *p16OCSE0 |= (uint16_t)((uint16_t)pstcConfig->enFrtPeakEvenNotMatchEvenChStatus   << 12u);
    *p16OCSE0 |= (uint16_t)((uint16_t)pstcConfig->enFrtDownCntEvenMatchEvenChRtStatus << 4u);
    *p16OCSE0 |= (uint16_t)((uint16_t)pstcConfig->enIopFlagWhenFrtZeroEvenMatch       << 3u);
    *p16OCSE0 |= (uint16_t)((uint16_t)pstcConfig->enIopFlagWhenFrtUpCntEvenMatch      << 2u);
    *p16OCSE0 |= (uint16_t)((uint16_t)pstcConfig->enIopFlagWhenFrtPeakEvenMatch       << 1u);
    *p16OCSE0 |= (uint16_t)((uint16_t)pstcConfig->enIopFlagWhenFrtDownCntEvenMatch    << 0u);
    return Ok;
}

/*!
 ******************************************************************************
 ** \brief  Compare congifuration of odd OCU channel
 **
 ** \param [in] pstcOcu Pointer to OCU instance
 ** \param [in]  OddCh  odd channel of OCU
 ** \arg    MFT_OCU_CH1, MFT_OCU_CH3, MFT_OCU_CH5
 ** \param [in] pstcConfig pointer to structure of compare mode
 **
 ** \retval Ok                    Odd OCU channel compare mode is set
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - OddCh out of range
 **                               - Other invalid configuration
 **
 *****************************************************************************
*/
en_result_t Mft_Ocu_SetOddChCompareMode (volatile stc_mftn_ocu_t*pstcOcu,
                                         uint8_t OddCh,
                                         stc_odd_compare_config_t* pstcConfig)
{
    volatile uint32_t* pu32OCSE1;
    
    // Check for NULL pointer
    if ( (pstcOcu == NULL)
         || (OddCh%2 == 0) 
         || (pstcConfig->enFrtZeroOddMatchEvenMatchOddChRtStatus > RtOutputReverse)     
         || (pstcConfig->enFrtZeroOddMatchEvenNotMatchOddChRtStatus > RtOutputReverse)     
         || (pstcConfig->enFrtZeroOddNotMatchEvenMatchOddChRtStatus > RtOutputReverse)     
         || (pstcConfig->enFrtZeroOddNotMatchEvenNotMatchOddChRtStatus > RtOutputReverse)       
         || (pstcConfig->enFrtUpCntOddMatchEvenMatchOddChRtStatus  > RtOutputReverse)    
         || (pstcConfig->enFrtUpCntOddMatchEvenNotMatchOddChRtStatus > RtOutputReverse)      
         || (pstcConfig->enFrtUpCntOddNotMatchEvenMatchOddChRtStatus > RtOutputReverse)       
         || (pstcConfig->enFrtPeakOddMatchEvenMatchOddChRtStatus     > RtOutputReverse)     
         || (pstcConfig->enFrtPeakOddMatchEvenNotMatchOddChRtStatus  > RtOutputReverse)      
         || (pstcConfig->enFrtPeakOddNotMatchEvenMatchOddChRtStatus  > RtOutputReverse)     
         || (pstcConfig->enFrtPeakOddNotMatchEvenNotMatchOddChRtStatus > RtOutputReverse)      
         || (pstcConfig->enFrtDownOddMatchEvenMatchOddChRtStatus       > RtOutputReverse)     
         || (pstcConfig->enFrtDownOddMatchEvenNotMatchOddChRtStatus    > RtOutputReverse)     
         || (pstcConfig->enFrtDownOddNotMatchEvenMatchOddChRtStatus    > RtOutputReverse)      
         || (pstcConfig->enIopFlagWhenFrtZeroOddMatch    > IopFlagSet)      
         || (pstcConfig->enIopFlagWhenFrtUpCntOddMatch   > IopFlagSet)      
         || (pstcConfig->enIopFlagWhenFrtPeakOddMatch    > IopFlagSet)     
         || (pstcConfig->enIopFlagWhenFrtDownCntOddMatch > IopFlagSet) )
    {
        return ErrorInvalidParameter ;
    }
    
    pu32OCSE1 = (volatile uint32_t*)((volatile uint8_t*)pstcOcu + 0x28UL + OddCh*4);
    *pu32OCSE1 = 0;
    *pu32OCSE1 |= (uint32_t)((uint32_t)pstcConfig->enFrtZeroOddMatchEvenMatchOddChRtStatus        << 26);
    *pu32OCSE1 |= (uint32_t)((uint32_t)pstcConfig->enFrtZeroOddMatchEvenNotMatchOddChRtStatus     << 10);
    *pu32OCSE1 |= (uint32_t)((uint32_t)pstcConfig->enFrtZeroOddNotMatchEvenMatchOddChRtStatus     << 30);
    *pu32OCSE1 |= (uint32_t)((uint32_t)pstcConfig->enFrtZeroOddNotMatchEvenNotMatchOddChRtStatus  << 14);
    *pu32OCSE1 |= (uint32_t)((uint32_t)pstcConfig->enFrtUpCntOddMatchEvenMatchOddChRtStatus       << 24);
    *pu32OCSE1 |= (uint32_t)((uint32_t)pstcConfig->enFrtUpCntOddMatchEvenNotMatchOddChRtStatus    << 8);
    *pu32OCSE1 |= (uint32_t)((uint32_t)pstcConfig->enFrtUpCntOddNotMatchEvenMatchOddChRtStatus    << 18);
    *pu32OCSE1 |= (uint32_t)((uint32_t)pstcConfig->enFrtPeakOddMatchEvenMatchOddChRtStatus        << 22);
    *pu32OCSE1 |= (uint32_t)((uint32_t)pstcConfig->enFrtPeakOddMatchEvenNotMatchOddChRtStatus     << 6) ;
    *pu32OCSE1 |= (uint32_t)((uint32_t)pstcConfig->enFrtPeakOddNotMatchEvenMatchOddChRtStatus     << 28);
    *pu32OCSE1 |= (uint32_t)((uint32_t)pstcConfig->enFrtPeakOddNotMatchEvenNotMatchOddChRtStatus  << 12);
    *pu32OCSE1 |= (uint32_t)((uint32_t)pstcConfig->enFrtDownOddMatchEvenMatchOddChRtStatus        << 20);
    *pu32OCSE1 |= (uint32_t)((uint32_t)pstcConfig->enFrtDownOddMatchEvenNotMatchOddChRtStatus     << 4);
    *pu32OCSE1 |= (uint32_t)((uint32_t)pstcConfig->enFrtDownOddNotMatchEvenMatchOddChRtStatus     << 16); 
    *pu32OCSE1 |= (uint32_t)((uint32_t)pstcConfig->enIopFlagWhenFrtZeroOddMatch                   << 3);
    *pu32OCSE1 |= (uint32_t)((uint32_t)pstcConfig->enIopFlagWhenFrtUpCntOddMatch                  << 2);
    *pu32OCSE1 |= (uint32_t)((uint32_t)pstcConfig->enIopFlagWhenFrtPeakOddMatch                   << 1);
    *pu32OCSE1 |= (uint32_t)((uint32_t)pstcConfig->enIopFlagWhenFrtDownCntOddMatch                << 0);
    
    return Ok;
}

/*!
 ******************************************************************************
 ** \brief  enable ocu operation
 **
 ** \param [in] pstcOcu Pointer to OCU instance
 ** \param [in]  u8Ch  channel of OCU
 ** \arg    MFT_OCU_CH0 ~ MFT_OCU_CH5
 **
 ** \retval Ok                    OCU operation enabled
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8Ch out of range
 **                               - Other invalid configuration
 **
 *****************************************************************************
*/
en_result_t Mft_Ocu_EnableOperation(volatile stc_mftn_ocu_t* pstcOcu, uint8_t u8Ch)
{
    volatile stc_mft_ocu_ocsa10_field_t* pstcOCSA;

    // Check for NULL pointer
    if ( (pstcOcu == NULL)
         || (u8Ch >= MFT_OCU_MAXCH))
    {
        return ErrorInvalidParameter ;
    }

    //Get pointer of current channel OCU register address
    pstcOCSA = (volatile stc_mft_ocu_ocsa10_field_t*)((volatile uint8_t*)pstcOcu + 0x18UL + (u8Ch/2)*4);

    //set register
    if(0 == (u8Ch%2)) //channel 0,2,4
    {
    	pstcOCSA->CST0 = 1;
    }
    else //channel 1,3,5
    {
    	pstcOCSA->CST1 = 1;
    }

    return Ok;
}

/*!
 ******************************************************************************
 ** \brief  Disable ocu operation
 **
 ** \param [in] pstcOcu Pointer to OCU instance
 ** \param [in]  u8Ch  channel of OCU
 ** \arg    MFT_OCU_CH0 ~ MFT_OCU_CH5
 **
 ** \retval Ok                    OCU operation disabled
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8Ch out of range
 **                               - Other invalid configuration
 **
 *****************************************************************************
*/
en_result_t Mft_Ocu_DisableOperation(volatile stc_mftn_ocu_t* pstcOcu, uint8_t u8Ch)
{
    volatile stc_mft_ocu_ocsa10_field_t* pstcOCSA;

    // Check for NULL pointer
    if ( (pstcOcu == NULL)
         || (u8Ch >= MFT_OCU_MAXCH))
    {
        return ErrorInvalidParameter ;
    }

    //Get pointer of current channel OCU register address
    pstcOCSA = (volatile stc_mft_ocu_ocsa10_field_t*)((volatile uint8_t*)pstcOcu + 0x18UL + (u8Ch/2)*4);

    //set register
    if(0 == (u8Ch%2)) //channel 0,2,4
    {
    	pstcOCSA->CST0 = 0;
    }
    else //channel 1,3,5
    {
    	pstcOCSA->CST1 = 0;
    }

    return Ok;
}

#if (PDL_INTERRUPT_ENABLE_MFT0_OCU == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT1_OCU == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT2_OCU == PDL_ON)
/*!
 ******************************************************************************
 ** \brief  enable ocu interrupt
 **
 ** \param [in] pstcOcu Pointer to OCU instance
 ** \param [in]  u8Ch  channel of OCU
 ** \arg    MFT_OCU_CH0 ~ MFT_OCU_CH5
 ** \param [in]  pfnCallback  callback function
 **
 ** \retval Ok                    OCU interrupt enabled
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8Ch out of range
 **                               - Other invalid configuration
 **
 *****************************************************************************
*/
en_result_t Mft_Ocu_EnableInt(volatile stc_mftn_ocu_t* pstcOcu,
                              uint8_t u8Ch, func_ptr_t  pfnCallback)
{
    stc_mft_ocu_intern_data_t* pstcOcuInternData;
    volatile stc_mft_ocu_ocsa10_field_t* pstcOCSA;
    func_ptr_t* fnCallback;
    // Check for NULL pointer
    if ( (pstcOcu == NULL)
         || (u8Ch >= MFT_OCU_MAXCH))
    {
        return ErrorInvalidParameter ;
    }

    // Get pointer to internal data structure ...
    pstcOcuInternData = MftGetInternDataPtr( pstcOcu ) ;

    //Get pointer of current channel OCU register address
    pstcOCSA = (volatile stc_mft_ocu_ocsa10_field_t*)((volatile uint8_t*)pstcOcu + 0x18UL + (u8Ch/2)*4);

    //set register
    if(0 == (u8Ch%2)) //channel 0,2,4
    {
        pstcOCSA->IOE0 = 1;
    }
    else //channel 1,3,5
    {
        pstcOCSA->IOE1 = 1;
    }

    //set callback function
    fnCallback = (func_ptr_t*)((uint8_t*)&(pstcOcuInternData->pfnOcu0Callback) 
                               + (sizeof(func_ptr_t) * u8Ch));
    *fnCallback = pfnCallback;
      
    Mft_Ocu_InitIrq(pstcOcu);

    return Ok;
}

/*!
 ******************************************************************************
 ** \brief  Disable ocu interrupt
 **
 ** \param [in] pstcOcu Pointer to OCU instance
 ** \param [in]  u8Ch  channel of OCU
 ** \arg    MFT_OCU_CH0 ~ MFT_OCU_CH5
 **
 ** \retval Ok                    OCU interrupt disabled
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8Ch out of range
 **                               - Other invalid configuration
 **
 *****************************************************************************
*/
en_result_t Mft_Ocu_DisableInt(volatile stc_mftn_ocu_t* pstcOcu,uint8_t u8Ch)
{
    stc_mft_ocu_intern_data_t* pstcOcuInternData;
    volatile stc_mft_ocu_ocsa10_field_t* pstcOCSA;

    // Check for NULL pointer
    if ( (pstcOcu == NULL)
         || (u8Ch >= MFT_OCU_MAXCH))
    {
        return ErrorInvalidParameter ;
    }

    // Get pointer to internal data structure ...
    pstcOcuInternData = MftGetInternDataPtr( pstcOcu ) ;

    //Get pointer of current channel OCU register address
    pstcOCSA = (volatile stc_mft_ocu_ocsa10_field_t*)((volatile uint8_t*)pstcOcu + 0x18UL + (u8Ch/2)*4);
    pstcOcuInternData = pstcOcuInternData + (sizeof(pstcOcuInternData)) * u8Ch;

    //set register
    if(0 == (u8Ch%2)) //channel 0,2,4
    {
        pstcOCSA->IOE0 = 0;
    }
    else //channel 1,3,5
    {
        pstcOCSA->IOE1 = 0;
    }

    //set callback function
    pstcOcuInternData = NULL;
    
    Mft_Ocu_DeInitIrq(pstcOcu);
    
    return Ok;
}

#endif

/*!
 ******************************************************************************
 ** \brief  get ocu interrupt flag
 **
 ** \param [in] pstcOcu Pointer to OCU instance
 ** \param [in]  u8Ch  channel of OCU
 ** \arg    MFT_OCU_CH0 ~ MFT_OCU_CH5
 **
 ** \retval PdlClr  OCU interrupt flag is clear
 ** \retval PdlSet  OCU interrupt flag is set
 **
 *****************************************************************************
*/
en_int_flag_t Mft_Ocu_GetIntFlag(volatile stc_mftn_ocu_t* pstcOcu, uint8_t u8Ch)
{
    volatile stc_mft_ocu_ocsa10_field_t* pstcOCSA;
    en_int_flag_t retval = PdlClr;

    //Get pointer of current channel OCU register address
    pstcOCSA = (volatile stc_mft_ocu_ocsa10_field_t*)((volatile uint8_t*)pstcOcu + 0x18UL + (u8Ch/2)*4);

    //set return value
    if(0 == (u8Ch%2)) //channel 0,2,4
    {   
        (pstcOCSA->IOP0 == 1) ? (retval = PdlSet) : (retval = PdlClr);
    }
    else //channel 1,3,5
    {
        (pstcOCSA->IOP1 == 1) ? (retval = PdlSet) : (retval = PdlClr);
    }

    return retval;
}

/*!
 ******************************************************************************
 ** \brief  clear ocu interrupt flag
 **
 ** \param [in] pstcOcu Pointer to OCU instance
 ** \param [in]  u8Ch  channel of OCU
 ** \arg    MFT_OCU_CH0 ~ MFT_OCU_CH5
 **
 ** \retval Ok                    OCU interrupt flag is clear
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8Ch out of range
 **                               - Other invalid configuration
 **
 *****************************************************************************
*/
en_result_t Mft_Ocu_ClrIntFlag(volatile stc_mftn_ocu_t* pstcOcu, uint8_t u8Ch)
{
    volatile stc_mft_ocu_ocsa10_field_t* pstcOCSA;

    // Check for NULL pointer
    if ( (pstcOcu == NULL)
         || (u8Ch >= MFT_OCU_MAXCH))
    {
        return ErrorInvalidParameter ;
    }

    //Get pointer of current channel OCU register address
    pstcOCSA = (volatile stc_mft_ocu_ocsa10_field_t*)((volatile uint8_t*)pstcOcu + 0x18UL + (u8Ch/2)*4);

    //clear register
    if(0 == (u8Ch%2)) //channel 0,2,4
    {
        pstcOCSA->IOP0 = 0;
    }
    else //channel 1,3,5
    {
        pstcOCSA->IOP1 = 0;
    }

    return Ok;
}

/*!
 ******************************************************************************
 ** \brief  Get RT pin level of OCU
 **
 ** \param [in] pstcOcu Pointer to OCU instance
 ** \param [in]  u8Ch  channel of OCU
 ** \arg    MFT_OCU_CH0 ~ MFT_OCU_CH5
 **
 ** \retval RtLowlevel  RT pin is low level
 ** \retval RtHighlevel RT pin is high level
 **
 *****************************************************************************
*/
en_ocu_rt_out_state_t Mft_Ocu_GetRtPinLevel(volatile stc_mftn_ocu_t* pstcOcu, 
                                            uint8_t u8Ch)
{
    volatile stc_mft_ocu_ocsb10_field_t* pstcOCSB;
    en_ocu_rt_out_state_t enLevel = RtLowLevel;
    
    // Get the OCSB address
    pstcOCSB = (volatile stc_mft_ocu_ocsb10_field_t*)((volatile uint8_t*)pstcOcu + 0x19UL + (u8Ch/2)*4);
    if((u8Ch%2) == 0)
    {
        (pstcOCSB->OTD0 == 1) ? (enLevel = RtHighLevel) : (enLevel = RtLowLevel);
    }
    else
    {
        (pstcOCSB->OTD1 == 1) ? (enLevel = RtHighLevel) : (enLevel = RtLowLevel);
    }
    
    return enLevel;
}

/*!
 ******************************************************************************
 ** \brief  writ occp register
 **
 ** \param [in] pstcOcu Pointer to OCU instance
 ** \param [in]  u8Ch  channel of OCU
 ** \arg    MFT_OCU_CH0 ~ MFT_OCU_CH5
 ** \param [in]  u16Occp  the value of occp
 ** \arg    16bit value
 **
 ** \retval Ok                    OCCP written
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8Ch out of range
 **                               - Other invalid configuration
 **
 *****************************************************************************
*/
en_result_t Mft_Ocu_WriteOccp(volatile stc_mftn_ocu_t* pstcOcu,
                              uint8_t u8Ch, uint16_t u16Occp)
{
	volatile uint16_t* p16OCCP;

    // Check for NULL pointer
    if ( (pstcOcu == NULL)
         || (u8Ch >= MFT_OCU_MAXCH))
    {
        return ErrorInvalidParameter ;
    }
    //Get pointer of current channel OCU register address
    p16OCCP = (volatile uint16_t*)((volatile uint8_t*)pstcOcu + 0x02 + u8Ch*4);

    //set register
    *p16OCCP = u16Occp;

    return Ok;
}

/*!
 ******************************************************************************
 ** \brief  get occp register value
 **
 ** \param [in] pstcOcu Pointer to OCU instance
 ** \param [in]  u8Ch  channel of OCU
 ** \arg    MFT_OCU_CH0 ~ MFT_OCU_CH5
 **
 ** \retval 16bit occp value
 **
 *****************************************************************************
*/
uint16_t Mft_Ocu_ReadOccp(volatile stc_mftn_ocu_t* pstcOcu, uint8_t u8Ch)
{
    volatile uint16_t* p16OCCP;

    // Check for NULL pointer
    if ( (pstcOcu == NULL)
         || (u8Ch >= MFT_OCU_MAXCH))
    {
        return 0 ;
    }
    //Get pointer of current channel OCU register address
    p16OCCP = (volatile uint16_t*)((volatile uint8_t*)pstcOcu + 0x02 + u8Ch*4);

    return (uint16_t)((*p16OCCP) & 0xFFFFu);
}

//@}

#endif // #if (defined(PDL_PERIPHERAL_$$X_ACTIVE))

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
