/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file pdl.h
 **
 ** Headerfile for FM0P Peripheral Driver Library 
 **
 ** History:  
 **   - 2013-03-21  0.0.1  MWi        First version.
 **   - 2013-11-25  0.0.2  EZ         Replace mb9abxxx.h with mcu.h
 **
 ******************************************************************************/

#ifndef __PDL_H__
#define __PDL_H__

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "base_types.h"

/* C binding of definitions if building with C++ compiler                     */
#ifdef __cplusplus
extern "C"
{
#endif

/******************************************************************************/
/* Global pre-processor symbols/macros ('#define')                            */
/* Macro for initializing local structures to zero                            */  
/******************************************************************************/ 
#define PDL_ZERO_STRUCT(x) pdl_memclr((uint8_t*)&(x), (uint32_t)(sizeof(x)))
   
/**
 ******************************************************************************
 ** All definitions needed for pdl_user.h are stated here 
 ******************************************************************************/ 
#define PDL_ON            1u    ///< Switches a feature on
#define PDL_OFF           0u    ///< Switches a feature off

/**
 ******************************************************************************
 ** Global Device type definitions
 ** Note that an existing definition does not implicitly means full device
 ** type support of this library!
 ******************************************************************************/  
#define PDL_TYPE1 1u  ///< FM0+ device type1
#define PDL_TYPE2 2u  ///< FM0+ device type2
#define PDL_TYPE3 3u  ///< FM0+ device type3
#define PDL_TYPE4 4u  ///< FM0+ device type4
#define PDL_TYPE5 5u  ///< FM0+ device type5

/**
 ******************************************************************************
 ** Global Device Series List
 ******************************************************************************/
#define PDL_DEVICE_SERIES_S6E1A1  10u

/**
 ******************************************************************************
 ** Global Device Package List
 ******************************************************************************/ 
#define PDL_DEVICE_PACKAGE_B  10u
#define PDL_DEVICE_PACKAGE_C  20u

/******************************************************************************/
/* User Device Setting Include file                                           */
/******************************************************************************/
#include "pdl_device.h"  // MUST be included here!

/**
 ******************************************************************************
 ** Device type extraction
 ******************************************************************************/
#if (PDL_MCU_SERIES == PDL_DEVICE_SERIES_S6E1A1)
  #if (PDL_PACKAGE == PDL_DEVICE_PACKAGE_B) || \
      (PDL_PACKAGE == PDL_DEVICE_PACKAGE_C)
    #define PDL_DEVICE_TYPE PDL_TYPE1
  #else
    #error Device Package not defined!
  #endif
#else
  #error Device Series not found!
#endif

/**
 ******************************************************************************
 ** Default Interrupt Level (lowest priority, used for De-Init functions)
 ******************************************************************************/
#define PDL_DEFAULT_INTERRUPT_LEVEL 0x0Fu

/**
 ******************************************************************************
 ** PDL resource availability check
 ******************************************************************************/ 
 
// ADC
#define PDL_PERIPHERAL_ADC_AVAILABLE          PDL_ON   

// Base Timer
#define PDL_PERIPHERAL_BT_AVAILABLE           PDL_ON   

// Clock
#define PDL_PERIPHERAL_CLK_AVAILABLE          PDL_ON      

// Dual Timer      
#define PDL_PERIPHERAL_DT_AVAILABLE           PDL_ON 

// DMA     
#define PDL_PERIPHERAL_DMA_AVAILABLE          PDL_ON

// CR Trimming  
#define PDL_PERIPHERAL_CRTRIM_AVAILABLE       PDL_ON   

// CSV      
#define PDL_PERIPHERAL_CSV_AVAILABLE          PDL_ON

// External interrupt       
#define PDL_PERIPHERAL_EXINT_AVAILABLE        PDL_ON  
      
// Flash     
#define PDL_PERIPHERAL_FLASH_AVAILABLE        PDL_ON 
            			
// LVD    
#define PDL_PERIPHERAL_LVD_AVAILABLE          PDL_ON

// MFS        
#define PDL_PERIPHERAL_MFS_AVAILABLE          PDL_ON

// QPRC 
#define PDL_PERIPHERAL_QPRC_AVAILABLE         PDL_ON  

// RESET       
#define PDL_PERIPHERAL_RESET_AVAILABLE        PDL_ON 

// Watch counter   
#define PDL_PERIPHERAL_WC_AVAILABLE           PDL_ON   

// Hardware watchdog        
#define PDL_PERIPHERAL_HWWDG_AVAILABLE        PDL_ON

// Software watchdog        
#define PDL_PERIPHERAL_SWWDG_AVAILABLE        PDL_ON

// RTC          
#define PDL_PERIPHERAL_RTC_AVAILABLE          PDL_ON
 
// LCD 
#define PDL_PERIPHERAL_LCD_AVAILABLE          PDL_OFF

// Unique ID  
#define PDL_PERIPHERAL_UID_AVAILABLE          PDL_ON


/**
 ******************************************************************************
 ** \brief Interrupt Type extraction 
 **
 ******************************************************************************
 */
/// \brief FM0P MCU interrupt Type A
#define PDL_INT_TYPE_A                      0u
/// \brief FM0P MCU interrupt Type B 
#define PDL_INT_TYPE_B                      1u
/// \brief FM0P MCU interrupt Type C
#define PDL_INT_TYPE_C                      2u

#if (PDL_DEVICE_TYPE == PDL_TYPE1)
    #define PDL_MCU_INT_TYPE       PDL_INT_TYPE_A
#else
    #error MCU Interrupt Type not found!
#endif

/**
 ******************************************************************************
 ** \brief IRQ name definition for all type MCUs
 ******************************************************************************/
#if (PDL_MCU_INT_TYPE == PDL_INT_TYPE_A)
    #define CSV_IRQHandler(void)               IRQ000_Handler(void) ///< CSV
    #define SWDT_IRQHandler(void)              IRQ001_Handler(void) ///< SW watchdog
    #define LVD_IRQHandler(void)               IRQ002_Handler(void) ///< LVD       
    #define MFT_WFG_IRQHandler(void)           IRQ003_Handler(void) ///< Interrupt Source Selection 3
    #define INT0_7_Handler(void)               IRQ004_Handler(void) ///< Interrupt Source Selection 4
    #define INT8_31_Handler(void)              IRQ005_Handler(void) ///< Interrupt Source Selection 5
    #define DT_QPRC_Handler(void)              IRQ006_Handler(void) ///< Interrupt Source Selection 6
    #define MFS0_8_RX_IRQHandler(void)         IRQ007_Handler(void) ///< Interrupt Source Selection 7
    #define MFS0_8_TX_IRQHandler(void)         IRQ008_Handler(void) ///< Interrupt Source Selection 8
    #define MFS1_9_RX_IRQHandler(void)         IRQ009_Handler(void) ///< Interrupt Source Selection 9
    #define MFS1_9_TX_IRQHandler(void)         IRQ010_Handler(void) ///< Interrupt Source Selection 10
    #define MFS2_10_RX_IRQHandler(void)        IRQ011_Handler(void) ///< External Pin Interrupt Ch. 0
    #define MFS2_10_TX_IRQHandler(void)        IRQ012_Handler(void) ///< External Pin Interrupt Ch. 1
    #define MFS3_11_RX_IRQHandler(void)        IRQ013_Handler(void) ///< External Pin Interrupt Ch. 2
    #define MFS3_11_TX_IRQHandler(void)        IRQ014_Handler(void) ///< External Pin Interrupt Ch. 3
    #define MFS4_12_RX_IRQHandler(void)        IRQ015_Handler(void) ///< External Pin Interrupt Ch. 4
    #define MFS4_12_TX_IRQHandler(void)        IRQ016_Handler(void) ///< External Pin Interrupt Ch. 5
    #define MFS5_13_RX_IRQHandler(void)        IRQ017_Handler(void) ///< External Pin Interrupt Ch. 6
    #define MFS5_13_TX_IRQHandler(void)        IRQ018_Handler(void) ///< External Pin Interrupt Ch. 7
    #define MFS6_14_RX_DMA0_IRQHandler(void)   IRQ019_Handler(void) ///< Quad Position & Revolution Counter Ch. 0
    #define MFS6_14_TX_DMA1_IRQHandler(void)   IRQ020_Handler(void) ///< Quad Position & Revolution Counter Ch. 1
    #define MFS7_15_RX_DMA2_IRQHandler(void)   IRQ021_Handler(void) ///< MFT Unit 0 Waveform Generator / DTIF Ch. 0
    #define MFS7_15_TX_DMA3_IRQHandler(void)   IRQ022_Handler(void) ///< MFT Unit 1 Waveform Generator / DTIF Ch. 1
    #define PPG_IRQHandler(void)               IRQ023_Handler(void) ///< PPG 
    #define TIM_IRQHandler(void)               IRQ024_Handler(void) ///< Watch Counter/Real Time Counter
    #define ADC0_IRQHandler(void)              IRQ025_Handler(void) ///< ADC Unit 0 
    #define ADC1_IRQHandler(void)              IRQ026_Handler(void) ///< ADC Unit 1
    #define ADC2_LCD_IRQHandler(void)          IRQ027_Handler(void) ///< ADC Unit 2/LCD
    #define MFT_FRT_IRQHandler(void)           IRQ028_Handler(void) ///< MFT Unit 0/1/2 Free-Run Timer
    #define MFT_IPC_IRQHandler(void)           IRQ029_Handler(void) ///< MFT Unit 0/1/2 Input Capture Unit
    #define MFT_OPC_IRQHandler(void)           IRQ030_Handler(void) ///< MFT Unit 0/1/2 Output Compare Unit  
    #define BT0_7_FLASH_IRQHandler(void)       IRQ031_Handler(void) ///< Base Timer ch.0~ch.7   
#else
    #error Interrupt Type not found!
#endif  
        
/******************************************************************************/
/* Global type definitions ('typedef')                                        */
/******************************************************************************/
/**
 ******************************************************************************
 ** \brief Level
 ** 
 ** Specifies levels.
 **
 ******************************************************************************/
typedef enum en_level
{
    PDLLow      = 0,  ///< Low level '0'
    PDLHigh     = 1   ///< High level '1'
} en_level_t;

/**
 ******************************************************************************
 ** \brief Generic Flag Code
 ** 
 ** Specifies flags.
 **
 ******************************************************************************/
typedef enum en_flag
{
    PdlClr = 0,       ///< Flag clr '0'
    PdlSet = 1        ///< Flag set '1' 
} en_stat_flag_t, en_int_flag_t;   
/*****************************************************************************/
/* Global variable declarations ('extern', definition in C source)           */
/*****************************************************************************/

/*****************************************************************************/
/* Global function prototypes ('extern', definition in C source)             */
/*****************************************************************************/

/******************************************************************************
 * Global function prototypes
 ******************************************************************************/
extern void pdl_memclr(uint8_t* pu32Address, uint32_t u32Count);

/**
 ******************************************************************************
 ** This hook is part of wait loops.
 ******************************************************************************/
extern void PDL_WAIT_LOOP_HOOK(void);

#ifdef __cplusplus
}
#endif

#endif /* __PDL_H__ */

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/

