/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved.
*
* This software is owned and published by:
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion
* components. This software is licensed by Spansion to be adapted only
* for use in systems utilizing Spansion components. Spansion shall not be
* responsible for misuse or illegal use of this software for devices not
* supported herein.  Spansion is providing this software "AS IS" and will
* not be responsible for issues arising from incorrect user implementation
* of the software.
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS),
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING,
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED
* WARRANTY OF NONINFRINGEMENT.
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT,
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION,
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA,
* SAVINGS OR PROFITS,
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED
* FROM, THE SOFTWARE.
*
* This software may be replicated in part or whole for the licensed use,
* with the restriction that this Disclaimer and Copyright notice must be
* included with each copy of this software, whether used in part or whole,
* at all times.
*/
/******************************************************************************/
/** \file csio_int_s25fl127s.c
 **
 ** A detailed description is available at 
 ** @link CsioIntS25fl127sGroup Module description @endlink
 **
 ** History:
 **   - 2014-03-17  1.0  Edison Zhang         First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "csio_int_s25fl127s.h"

#if (PDL_UTILITY_ENABLE_CSIO_INT_S25FL127S == PDL_ON)

/*****************************************************************************/
/* Local pre-processor symbols/macros ('#define')                            */
/*****************************************************************************/
/*****************************************************************************/
/* Global variable definitions (declared in header file with 'extern')       */
/*****************************************************************************/
/*****************************************************************************/
/* Local type definitions ('typedef')                                        */
/*****************************************************************************/
/**
 ******************************************************************************
 ** \brief Part of SPI Flash command list
 ******************************************************************************/
typedef enum en_flash_cmd
{
    CmdReadId           = 0x90u,
    CmdReadStatusReg1   = 0x05u,
    CmdWriteEnable      = 0x06u,
    CmdWriteDisable     = 0x04u,
    CmdBulkErase        = 0x60u,
    Cmd4kSectorErase    = 0x20u,
    CmdSectorErase      = 0xD8u,
    CmdPagePgm          = 0x02u,
    CmdRead             = 0x03u,
    CmdFastRead         = 0x0Bu,
    
}en_flash_cmd_t;

/**
 ******************************************************************************
 ** \brief CSIO information
 ******************************************************************************/
typedef struct stc_csio_info
{
    uint8_t*  pu8TxData;
    uint32_t  u8TxSize;
    uint8_t*  pu8RxData;
    uint32_t  u8RxSize;
    uint32_t  u32Cnt;
    boolean_t bTxFinish;
    boolean_t bRxFinish;
    boolean_t bRwSyncFlag;
    
}stc_csio_info_t;

/*****************************************************************************/
/* Local function prototypes ('static')                                      */
/*****************************************************************************/
static void Delay(uint32_t u32Cnt);
static void CsioMasterTxIntCallback(void);
static void CsioMasterRxIntCallback(void);
static en_result_t CsioTxData(uint8_t u8Cmd,    uint8_t u8CmdSize,
                              uint32_t u32Addr,  uint8_t u8AddrSize,
                              uint8_t* pu8Data, uint32_t u32DataSize);
static en_result_t CsioRxData(uint8_t u8Cmd,    uint8_t u8CmdSize,
                              uint32_t u32Addr,  uint8_t u8AddrSize,
                              uint8_t* pu8Data, uint32_t u32DataSize);
static en_result_t CsioRxReg(uint8_t u8Cmd,      uint8_t u8CmdSize, 
                             uint8_t* pRegValue, uint8_t u8RegSize);
static en_result_t CsioTxCmd(uint8_t u8Cmd,      uint8_t u8CmdSize);
static en_result_t CsioTxCmdAddr(uint8_t u8Cmd,      uint8_t u8CmdSize,
                                 uint32_t u32Addr,   uint8_t u8AddrSize);


/*****************************************************************************/
/* Local variable definitions ('static')                                     */
/*****************************************************************************/
stc_csio_int_sel_t stcCsio1IntSel;
stc_csio_int_cb_t stcCsio1IntCb;
stc_csio_info_t stcCsioInfo;

/**
 ******************************************************************************
 ** \brief  Time delay
 ******************************************************************************/
static void Delay(uint32_t u32Cnt)
{
    while(u32Cnt--);
}

/**
 ******************************************************************************
 ** \brief  CSIO master transfer interrupt callback function
 ******************************************************************************/
static void CsioMasterTxIntCallback(void)
{
    if(stcCsioInfo.u32Cnt >= stcCsioInfo.u8TxSize)
    {
        /* Disable interrupt */
        PDL_ZERO_STRUCT(stcCsio1IntSel);
        stcCsio1IntSel.bTxInt = TRUE;
        Mfs_Csio_DisableInt(FLASH_CSIO_CH, &stcCsio1IntSel);
        
        stcCsioInfo.bTxFinish = TRUE;
        
        return;
    }

    Mfs_Csio_SendData(FLASH_CSIO_CH, *stcCsioInfo.pu8TxData, TRUE);
    stcCsioInfo.u32Cnt++;
    stcCsioInfo.pu8TxData++;
    
    stcCsioInfo.bRwSyncFlag = FALSE;
}


/**
 ******************************************************************************
 ** \brief  CSIO Master receive interrupt callback function
 ******************************************************************************/
static void CsioMasterRxIntCallback(void)
{
    *stcCsioInfo.pu8RxData = Mfs_Csio_ReceiveData(FLASH_CSIO_CH);
    
    stcCsioInfo.u32Cnt++;
    stcCsioInfo.pu8RxData++;
    
    stcCsioInfo.bRwSyncFlag = TRUE;

    if(stcCsioInfo.u32Cnt >= stcCsioInfo.u8RxSize)
    {
        /* Disable interrupt */
        PDL_ZERO_STRUCT(stcCsio1IntSel);
        stcCsio1IntSel.bRxInt = TRUE;
        Mfs_Csio_DisableInt(FLASH_CSIO_CH, &stcCsio1IntSel);
        
        stcCsioInfo.bRxFinish = TRUE;
        
        return;
    }
}

/**
 ******************************************************************************
 ** \brief  Write command, address, data to S25FL127S
 **
 ** \param  u8Cmd Command
 ** \param  u8CmdSize Command size
 ** \param  u32Addr Flash address
 ** \param  u8AddrSize Address size, can be 3 or 4bytes (3 bytes for S25FL127S)
 ** \param  pu8Data Pointer to data array to be sent
 ** \param  u32DataSize Data size
 **
 ** \retval Ok Command, address, data are Writen normally
 ******************************************************************************/
static en_result_t CsioTxData(uint8_t u8Cmd,    uint8_t u8CmdSize,
                              uint32_t u32Addr,  uint8_t u8AddrSize,
                              uint8_t* pu8Data, uint32_t u32DataSize)
{
    uint8_t u8Times = 0, u8Offset = 0;
    uint8_t u8i;
  
    /* Enable TX function of CSIO1   */
    Mfs_Csio_EnableFunc(FLASH_CSIO_CH, CsioTx);
    
    Mfs_Csio_SetCsHoldStatus(FLASH_CSIO_CH, TRUE);
    Mfs_Csio_SetCsTransferByteCount(FLASH_CSIO_CH, CsPinScs0, u8CmdSize);
    
    /* Send command */
    stcCsioInfo.pu8TxData = &u8Cmd;
    stcCsioInfo.u8TxSize = u8CmdSize;
    stcCsioInfo.u32Cnt = 0;
    stcCsioInfo.bTxFinish = FALSE;
    
    PDL_ZERO_STRUCT(stcCsio1IntSel);
    stcCsio1IntSel.bTxInt = TRUE;
    stcCsio1IntCb.pfnTxIntCb = CsioMasterTxIntCallback;
    Mfs_Csio_EnableInt(FLASH_CSIO_CH, &stcCsio1IntSel, &stcCsio1IntCb);

    /* Wait until receive finish */
    while(stcCsioInfo.bTxFinish != TRUE)
    {
        ;
    }
    
    Mfs_Csio_SetCsTransferByteCount(FLASH_CSIO_CH, CsPinScs0, u8AddrSize);
    
    /* Send address */
    stcCsioInfo.pu8TxData = (uint8_t*)&u32Addr;
    stcCsioInfo.u8TxSize = u8AddrSize;
    stcCsioInfo.u32Cnt = 0;
    stcCsioInfo.bTxFinish = FALSE;
    
    PDL_ZERO_STRUCT(stcCsio1IntSel);
    stcCsio1IntSel.bTxInt = TRUE;
    stcCsio1IntCb.pfnTxIntCb = CsioMasterTxIntCallback;
    Mfs_Csio_EnableInt(FLASH_CSIO_CH, &stcCsio1IntSel, &stcCsio1IntCb);

    /* Wait until receive finish */
    while(stcCsioInfo.bTxFinish != TRUE)
    {
        ;
    }
    
    /* Send data */
    u8Times = u32DataSize/255u;
    u8Offset = u32DataSize%255u;
    u8i = 0;
    
    if(u8Times > 0)
    {
        while(u8i != u8Times)
        {
            Mfs_Csio_SetCsTransferByteCount(FLASH_CSIO_CH, CsPinScs0, 255u);
            
            stcCsioInfo.pu8TxData = pu8Data + 255u*u8i;
            stcCsioInfo.u8TxSize = 255u;
            stcCsioInfo.u32Cnt = 0;
            stcCsioInfo.bTxFinish = FALSE;
            
            PDL_ZERO_STRUCT(stcCsio1IntSel);
            stcCsio1IntSel.bTxInt = TRUE;
            stcCsio1IntCb.pfnTxIntCb = CsioMasterTxIntCallback;
            Mfs_Csio_EnableInt(FLASH_CSIO_CH, &stcCsio1IntSel, &stcCsio1IntCb);

            /* Wait until receive finish */
            while(stcCsioInfo.bTxFinish != TRUE)
            {
                ;
            }
            
            u8i++;
        }
    }
    
    if(u8Offset > 0)
    {
        Mfs_Csio_SetCsTransferByteCount(FLASH_CSIO_CH, CsPinScs0, u8Offset);
                
        stcCsioInfo.pu8TxData = pu8Data + 255u*u8i;
        stcCsioInfo.u8TxSize = u8Offset;
        stcCsioInfo.u32Cnt = 0;
        stcCsioInfo.bTxFinish = FALSE;
        
        PDL_ZERO_STRUCT(stcCsio1IntSel);
        stcCsio1IntSel.bTxInt = TRUE;
        stcCsio1IntCb.pfnTxIntCb = CsioMasterTxIntCallback;
        Mfs_Csio_EnableInt(FLASH_CSIO_CH, &stcCsio1IntSel, &stcCsio1IntCb);

        /* Wait until receive finish */
        while(stcCsioInfo.bTxFinish != TRUE)
        {
            ;
        }
    }
    
    Mfs_Csio_SetCsHoldStatus(FLASH_CSIO_CH, FALSE);
    while(TRUE != Mfs_Csio_GetStatus(FLASH_CSIO_CH, CsioTxIdle));

    /* Disable TX function of CSIO0   */
    Mfs_Csio_DisableFunc(FLASH_CSIO_CH, CsioTx);
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief  Read data from S25FL127S
 **
 ** \param  u8Cmd Command
 ** \param  u8CmdSize Command size
 ** \param  u32Addr Flash address
 ** \param  u8AddrSize Address size, can be 3 or 4bytes (3 bytes for S25FL127S)
 ** \param  pu8Data Pointer to data array to be read
 ** \param  u32DataSize Data size
 **
 ** \retval Ok Command, address, data are sent normally
 ******************************************************************************/
static en_result_t CsioRxData(uint8_t u8Cmd,    uint8_t u8CmdSize,
                              uint32_t u32Addr,  uint8_t u8AddrSize,
                              uint8_t* pu8Data, uint32_t u32DataSize)
{
    uint8_t u8Times = 0, u8Offset = 0;
    uint8_t u8i,u8j;
  
    /* Enable TX function of CSIO1   */
    Mfs_Csio_EnableFunc(FLASH_CSIO_CH, CsioTx);
    
    Mfs_Csio_SetCsHoldStatus(FLASH_CSIO_CH, TRUE);
    Mfs_Csio_SetCsTransferByteCount(FLASH_CSIO_CH, CsPinScs0, u8CmdSize);

    /* Send command */
    stcCsioInfo.pu8TxData = &u8Cmd;
    stcCsioInfo.u8TxSize = u8CmdSize;    
    stcCsioInfo.bTxFinish = FALSE;
    stcCsioInfo.u32Cnt = 0;

    PDL_ZERO_STRUCT(stcCsio1IntSel);
    stcCsio1IntSel.bTxInt = TRUE;
    stcCsio1IntCb.pfnTxIntCb = CsioMasterTxIntCallback;
    Mfs_Csio_EnableInt(FLASH_CSIO_CH, &stcCsio1IntSel, &stcCsio1IntCb);
    
    /* Wait until send finish */
    while(stcCsioInfo.bTxFinish != TRUE)
    {
        ;
    }

    Mfs_Csio_SetCsTransferByteCount(FLASH_CSIO_CH, CsPinScs0, u8AddrSize);
    /* Send address */
    stcCsioInfo.pu8TxData = (uint8_t*)&u32Addr;
    stcCsioInfo.u8TxSize = u8AddrSize;    
    stcCsioInfo.bTxFinish = FALSE;
    stcCsioInfo.u32Cnt = 0;

    PDL_ZERO_STRUCT(stcCsio1IntSel);
    stcCsio1IntSel.bTxInt = TRUE;
    stcCsio1IntCb.pfnTxIntCb = CsioMasterTxIntCallback;
    Mfs_Csio_EnableInt(FLASH_CSIO_CH, &stcCsio1IntSel, &stcCsio1IntCb);
    
    /* Wait until send finish */
    while(stcCsioInfo.bTxFinish != TRUE)
    {
        ;
    }
    
    Delay(100);
    
    /* Read data */
    u8Times = u32DataSize/255u;
    u8Offset = u32DataSize%255u;
    u8i = 0;
    
    if(u8Times > 0)
    {
        while(u8i < u8Times)
        {
            stcCsioInfo.pu8RxData = pu8Data + 255*u8i;
            stcCsioInfo.u8RxSize = 255u;
            stcCsioInfo.bRxFinish = FALSE;
            stcCsioInfo.u32Cnt = 0;
            
            Mfs_Csio_EnableFunc(FLASH_CSIO_CH, CsioRx);
            
            Mfs_Csio_SetCsTransferByteCount(FLASH_CSIO_CH, CsPinScs0, 255u);
            
            PDL_ZERO_STRUCT(stcCsio1IntSel);
            stcCsio1IntSel.bRxInt = TRUE;
            stcCsio1IntCb.pfnRxIntCb = CsioMasterRxIntCallback;
            Mfs_Csio_EnableInt(FLASH_CSIO_CH, &stcCsio1IntSel, &stcCsio1IntCb);
            
             /* Send dummy data and wait until receive finish */
            u8j = 255u;
            while(u8j--)
            {
                while(TRUE != Mfs_Csio_GetStatus(FLASH_CSIO_CH, CsioTxEmpty));
                Mfs_Csio_SendData(FLASH_CSIO_CH, 0x00u, FALSE);   /* Dummy write */
            }
            
            while(stcCsioInfo.bRxFinish != TRUE)
            {
                ;
            }
            
            u8i++;
        }
    }
    
    if(u8Offset > 0)
    {
        stcCsioInfo.pu8RxData = pu8Data + 255*u8i;
        stcCsioInfo.u8RxSize = u8Offset;
        stcCsioInfo.bRxFinish = FALSE;
        stcCsioInfo.u32Cnt = 0;
        
        Mfs_Csio_EnableFunc(FLASH_CSIO_CH, CsioRx);
        
        Mfs_Csio_SetCsTransferByteCount(FLASH_CSIO_CH, CsPinScs0, u8Offset);
        
        PDL_ZERO_STRUCT(stcCsio1IntSel);
        stcCsio1IntSel.bRxInt = TRUE;
        stcCsio1IntCb.pfnRxIntCb = CsioMasterRxIntCallback;
        Mfs_Csio_EnableInt(FLASH_CSIO_CH, &stcCsio1IntSel, &stcCsio1IntCb);
        
         /* Send dummy data and wait until receive finish */
        u8j = u8Offset;
        while(u8j--)
        {
            while(TRUE != Mfs_Csio_GetStatus(FLASH_CSIO_CH, CsioTxEmpty));
            Mfs_Csio_SendData(FLASH_CSIO_CH, 0x00u, FALSE);   /* Dummy write */
        }
        
        while(stcCsioInfo.bRxFinish != TRUE)
        {
            ;
        }

    }
    
    Mfs_Csio_SetCsHoldStatus(FLASH_CSIO_CH, FALSE);
    while(TRUE != Mfs_Csio_GetStatus(FLASH_CSIO_CH, CsioTxIdle));
    
    /* Disable TX function of CSIO0   */
    Mfs_Csio_DisableFunc(FLASH_CSIO_CH, CsioTx);
    Mfs_Csio_DisableFunc(FLASH_CSIO_CH, CsioRx);
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief  Read register from S25FL127S
 **
 ** \param  u8Cmd Command
 ** \param  u8CmdSize Command size
 ** \param  pRegValue Poitner to register value
 ** \param  u8RegSize Read times
 **
 ** \retval Ok Command, address, data are sent normally
 ******************************************************************************/
static en_result_t CsioRxReg(uint8_t u8Cmd,      uint8_t u8CmdSize, 
                             uint8_t* pRegValue, uint8_t u8RegSize)
{
    /* Enable TX function of CSIO1   */
    Mfs_Csio_EnableFunc(FLASH_CSIO_CH, CsioTx);
    
    Mfs_Csio_SetCsTransferByteCount(FLASH_CSIO_CH, CsPinScs0, (u8CmdSize+u8RegSize));

    /* Send command */
    stcCsioInfo.pu8TxData = &u8Cmd;
    stcCsioInfo.u8TxSize = u8CmdSize;    
    stcCsioInfo.bTxFinish = FALSE;
    stcCsioInfo.u32Cnt = 0;

    PDL_ZERO_STRUCT(stcCsio1IntSel);
    stcCsio1IntSel.bTxInt = TRUE;
    stcCsio1IntCb.pfnTxIntCb = CsioMasterTxIntCallback;
    Mfs_Csio_EnableInt(FLASH_CSIO_CH, &stcCsio1IntSel, &stcCsio1IntCb);
    
    /* Wait until send finish */
    while(stcCsioInfo.bTxFinish != TRUE)
    {
        ;
    }

    /* Read data */
    stcCsioInfo.pu8RxData = pRegValue;
    stcCsioInfo.u8RxSize = u8RegSize;
    stcCsioInfo.bRxFinish = FALSE;
    stcCsioInfo.u32Cnt = 0;
    
    Mfs_Csio_EnableFunc(FLASH_CSIO_CH, CsioRx);
    
    PDL_ZERO_STRUCT(stcCsio1IntSel);
    stcCsio1IntSel.bRxInt = TRUE;
    stcCsio1IntCb.pfnRxIntCb = CsioMasterRxIntCallback;
    Mfs_Csio_EnableInt(FLASH_CSIO_CH, &stcCsio1IntSel, &stcCsio1IntCb);
    
     /* Send dummy data and wait until receive finish */
    while(u8RegSize--)
    {
        while(TRUE != Mfs_Csio_GetStatus(FLASH_CSIO_CH, CsioTxEmpty));
        Mfs_Csio_SendData(FLASH_CSIO_CH, 0x00u, FALSE);   /* Dummy write */
    }
    
    while(stcCsioInfo.bRxFinish != TRUE)
    {
        ;
    }
    
    /* Disable TX function of CSIO0   */
    Mfs_Csio_DisableFunc(FLASH_CSIO_CH, CsioTx);
    Mfs_Csio_DisableFunc(FLASH_CSIO_CH, CsioRx);
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief  Write command only to S25FL127S
 **
 ** \param  u8Cmd Command
 ** \param  u8CmdSize Command size
 **
 ** \retval Ok Command are writen normally
 ******************************************************************************/
static en_result_t CsioTxCmd(uint8_t u8Cmd,      uint8_t u8CmdSize)
{
    /* Enable TX function of CSIO1   */
    Mfs_Csio_EnableFunc(FLASH_CSIO_CH, CsioTx);
    
    Mfs_Csio_SetCsTransferByteCount(FLASH_CSIO_CH, CsPinScs0, u8CmdSize);
    
    /* Send command */
    stcCsioInfo.pu8TxData = &u8Cmd;
    stcCsioInfo.u8TxSize = u8CmdSize;    
    stcCsioInfo.bTxFinish = FALSE;
    stcCsioInfo.u32Cnt = 0;

    PDL_ZERO_STRUCT(stcCsio1IntSel);
    stcCsio1IntSel.bTxInt = TRUE;
    stcCsio1IntCb.pfnTxIntCb = CsioMasterTxIntCallback;
    Mfs_Csio_EnableInt(FLASH_CSIO_CH, &stcCsio1IntSel, &stcCsio1IntCb);
    
    /* Wait until send finish */
    while(stcCsioInfo.bTxFinish != TRUE)
    {
        ;
    }
    
    while(TRUE != Mfs_Csio_GetStatus(FLASH_CSIO_CH, CsioTxIdle));
    
    /* Disable TX function of CSIO0   */
    Mfs_Csio_DisableFunc(FLASH_CSIO_CH, CsioTx);
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief  Write command and address only to S25FL127S
 **
 ** \param  u8Cmd Command
 ** \param  u8CmdSize Command size
 ** \param  u32Addr    Address
 ** \param  u8AddrSize Address size
 **
 ** \retval Ok Command and address are writen normally
 ******************************************************************************/
static en_result_t CsioTxCmdAddr(uint8_t u8Cmd,      uint8_t u8CmdSize,
                                 uint32_t u32Addr,   uint8_t u8AddrSize)
{
    /* Enable TX function of CSIO1   */
    Mfs_Csio_EnableFunc(FLASH_CSIO_CH, CsioTx);
    
    Mfs_Csio_SetCsTransferByteCount(FLASH_CSIO_CH, CsPinScs0, (u8CmdSize+u8AddrSize));
    
    /* Send command */
    stcCsioInfo.pu8TxData = &u8Cmd;
    stcCsioInfo.u8TxSize = u8CmdSize;    
    stcCsioInfo.bTxFinish = FALSE;
    stcCsioInfo.u32Cnt = 0;

    PDL_ZERO_STRUCT(stcCsio1IntSel);
    stcCsio1IntSel.bTxInt = TRUE;
    stcCsio1IntCb.pfnTxIntCb = CsioMasterTxIntCallback;
    Mfs_Csio_EnableInt(FLASH_CSIO_CH, &stcCsio1IntSel, &stcCsio1IntCb);
    
    /* Wait until send finish */
    while(stcCsioInfo.bTxFinish != TRUE)
    {
        ;
    }
    
    /* Send address */
    stcCsioInfo.pu8TxData = (uint8_t*)&u32Addr;
    stcCsioInfo.u8TxSize = u8AddrSize;    
    stcCsioInfo.bTxFinish = FALSE;
    stcCsioInfo.u32Cnt = 0;

    PDL_ZERO_STRUCT(stcCsio1IntSel);
    stcCsio1IntSel.bTxInt = TRUE;
    stcCsio1IntCb.pfnTxIntCb = CsioMasterTxIntCallback;
    Mfs_Csio_EnableInt(FLASH_CSIO_CH, &stcCsio1IntSel, &stcCsio1IntCb);
    
    /* Wait until send finish */
    while(stcCsioInfo.bTxFinish != TRUE)
    {
        ;
    }
    
    while(TRUE != Mfs_Csio_GetStatus(FLASH_CSIO_CH, CsioTxIdle));
    
    /* Disable TX function of CSIO0   */
    Mfs_Csio_DisableFunc(FLASH_CSIO_CH, CsioTx);
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief  Initialize S25FL127S
 ******************************************************************************/
en_result_t S25fl127s_Init(void)
{
    stc_mfs_csio_config_t stcCsio1Config;

    /* Clear configuration structure */
    PDL_ZERO_STRUCT(stcCsio1Config);
    PDL_ZERO_STRUCT(stcCsio1IntSel);
    PDL_ZERO_STRUCT(stcCsio1IntCb);

    /* Initialize CSIO function I/O */
    InitCsioIo();

    /* Initialize CSIO master  */
    stcCsio1Config.enMsMode = CsioMaster;
    stcCsio1Config.enActMode = CsioActNormalMode;
    stcCsio1Config.bInvertClk = FALSE;
    stcCsio1Config.u32BaudRate = 500000;
    stcCsio1Config.enDataLength = CsioEightBits;
    stcCsio1Config.enBitDirection = CsioDataMsbFirst;
    stcCsio1Config.enSyncWaitTime = CsioSyncWaitZero;
    stcCsio1Config.bEnableFifo = FALSE;

    stcCsio1Config.bEnChipSelection = TRUE;
    stcCsio1Config.stcCsConfig.enCsStartPin = CsPinScs0;
    stcCsio1Config.stcCsConfig.enCsEndPin = CsPinScs0;
    stcCsio1Config.stcCsConfig.enClkDiv = CsClkNoDiv;
    stcCsio1Config.stcCsConfig.bActiveHold = FALSE;
    stcCsio1Config.stcCsConfig.enLevel = CsLowActive;
    stcCsio1Config.stcCsConfig.u8CsSetupDelayTime = 200u;
    stcCsio1Config.stcCsConfig.u8CsHoldDelayTime = 200u;
    stcCsio1Config.stcCsConfig.u16CsDeselectTime = 2000u;
    stcCsio1Config.stcCsConfig.u8Scs0TransferByteCnt = 0;
    stcCsio1Config.stcCsConfig.u8Scs1TransferByteCnt = 0;
    stcCsio1Config.stcCsConfig.u8Scs2TransferByteCnt = 0;
    stcCsio1Config.stcCsConfig.u8Scs3TransferByteCnt = 0;
    stcCsio1Config.stcCsConfig.bScs0En = TRUE;
    stcCsio1Config.stcCsConfig.bScs1En = FALSE;
    stcCsio1Config.stcCsConfig.bScs2En = FALSE;
    stcCsio1Config.stcCsConfig.bScs3En = FALSE;

    stcCsio1Config.bEnSyncTransfer = FALSE;

    if(Ok != Mfs_Csio_Init(FLASH_CSIO_CH, &stcCsio1Config))
    {
        return Error;
    }
    
    return Ok;
}


/**
 ******************************************************************************
 ** \brief  Read ID from S25FL127S
 **
 ** \param  pManufactureId Pointer to manufacture ID
 ** \param  pDeviceId Pointer to device ID
 **
 ** \retval Ok ID are read normally
 ******************************************************************************/
en_result_t S25fl127s_ReadId(uint8_t* pManufactureId, uint8_t* pDeviceId)
{
    uint8_t u8RxData[2];

    CsioRxData(CmdReadId, 1, 0x000000u, 3, u8RxData, 2);
 
    *pManufactureId = u8RxData[0];
    *pDeviceId = u8RxData[1];
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief  Chip erase the whole Flash memory
 **
 ** \retval Ok Chip Erase ok
 ** \retval Error Chip erase error occurs
 ******************************************************************************/
en_result_t S25fl127s_BulkErase(void)
{
    uint8_t u8Status;
    boolean_t bError = FALSE;
      
    CsioTxCmd(CmdWriteEnable, 1);
    
    CsioTxCmd(CmdBulkErase, 1);
    
    while(1)
    {
        CsioRxReg(CmdReadStatusReg1, 1, &u8Status, 1);
        
        if((u8Status & 0x40u) == 0x40u)
        {
            bError = TRUE;
            break;
        }
        
        if((u8Status & 0x01u) == 0x00u) /* Wait until write finish */
        {
            break;
        } 
    }
    
    CsioTxCmd(CmdWriteDisable, 1);
    
    if(bError == TRUE)
    {
        return Error;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief  Sector erase 4KB sector
 **
 ** \param u32Addr The address within the 4k area
 **
 ** \retval Ok Sector Erase ok
 ** \retval Error Sector erase error occurs
 ******************************************************************************/
en_result_t S25fl127s_4kSectorErase(uint32_t u32Addr)
{
    uint8_t u8Status;
    boolean_t bError = FALSE;
    
    CsioTxCmd(CmdWriteEnable, 1);
    
    CsioTxCmdAddr(Cmd4kSectorErase, 1, u32Addr, 3);
    
    while(1)
    {
        CsioRxReg(CmdReadStatusReg1, 1, &u8Status, 1);
        
        if((u8Status & 0x40u) == 0x40u)
        {
            bError = TRUE;
            break;
        }
        
        if((u8Status & 0x01u) == 0x00u) /* Wait until write finish */
        {
            break;
        } 
    }
    
    CsioTxCmd(CmdWriteDisable, 1);
    
    if(bError == TRUE)
    {
        return Error;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief  Sector erase 64kB or 256kB sector
 **
 ** \param u32Addr The address within the 64kB or 256kB area
 **
 ** \retval Ok Sector Erase ok
 ** \retval Error Sector erase error occurs
 ******************************************************************************/
en_result_t S25fl127s_SectorErase(uint32_t u32Addr)
{
    uint8_t u8Status;
    boolean_t bError = FALSE;
    
    CsioTxCmd(CmdWriteEnable, 1);
    
    CsioTxCmdAddr(CmdSectorErase, 1, u32Addr, 3);
    
    while(1)
    {
        CsioRxReg(CmdReadStatusReg1, 1, &u8Status, 1);
        
        if((u8Status & 0x40u) == 0x40u)
        {
            bError = TRUE;
            break;
        }
        
        if((u8Status & 0x01u) == 0x00u) /* Wait until write finish */
        {
            break;
        } 
    }
    
    CsioTxCmd(CmdWriteDisable, 1);
    
    if(bError == TRUE)
    {
        return Error;
    }
    
    return Ok;
}

/**
 ******************************************************************************
 ** \brief  Page program S25FL127S 
 ** 
 ** The default page size is 256Byte, 256bytes or less can be progammed with 
 ** this function. The page buffer size can also be set to 512Byte via 
 ** status register 2, but it is not considered here.
 **
 ** \param u32Addr Address
 ** \param pu8Data Pointer to data address
 ** \param u16Size Data size
 **
 ** \retval Ok Page program OK
 ** \retval ErrorInvalidParameter u16Size > S25FL127S_DEFAULT_PAGE_SIZE
                                  pu8Data == NULL
 ** \retval Error Page program error
 ******************************************************************************/
en_result_t S25fl127s_PageProgram(uint32_t u32Addr, uint8_t* pu8Data, uint16_t u16Size)
{
    uint8_t u8Status;
    boolean_t bError = FALSE;
  
    if((u16Size > S25FL127S_DEFAULT_PAGE_SIZE) || (pu8Data == NULL))
    {
        return ErrorInvalidParameter;
    }
    
    CsioTxCmd(CmdWriteEnable, 1);
    CsioTxData(CmdPagePgm, 1, u32Addr, 3, pu8Data, u16Size);
    
    while(1)
    {
        CsioRxReg(CmdReadStatusReg1, 1, &u8Status, 1);
        
        if((u8Status & 0x40u) == 0x40u)
        {
            bError = TRUE;
            break;
        }
        
        if((u8Status & 0x01u) == 0x00u) /* Wait until write finish */
        {
            break;
        } 
    }
    
    CsioTxCmd(CmdWriteDisable, 1);
    
    if(bError == TRUE)
    {
        return Error;
    }
    
    return Ok;
    
}

/**
 ******************************************************************************
 ** \brief  Read data from S25FL127S 
 **
 ** \param u32Addr Address
 ** \param pu8Data Pointer to data address
 ** \param u32Size Data size
 **
 ** \retval Ok Page program OK
 ** \retval ErrorInvalidParameter   pu8Data == NULL
 ******************************************************************************/
en_result_t S25fl127s_Read(uint32_t u32Addr, uint8_t* pu8Data, uint32_t u32Size)
{  
    if(pu8Data == NULL)
    {
        return ErrorInvalidParameter;
    }
    
    CsioRxData(CmdRead, 1, u32Addr, 3, pu8Data, u32Size);
    
    return Ok;
    
}

#endif

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
