/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved.
*
* This software is owned and published by:
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion
* components. This software is licensed by Spansion to be adapted only
* for use in systems utilizing Spansion components. Spansion shall not be
* responsible for misuse or illegal use of this software for devices not
* supported herein.  Spansion is providing this software "AS IS" and will
* not be responsible for issues arising from incorrect user implementation
* of the software.
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS),
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING,
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED
* WARRANTY OF NONINFRINGEMENT.
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT,
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION,
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA,
* SAVINGS OR PROFITS,
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED
* FROM, THE SOFTWARE.
*
* This software may be replicated in part or whole for the licensed use,
* with the restriction that this Disclaimer and Copyright notice must be
* included with each copy of this software, whether used in part or whole,
* at all times.
*/
/******************************************************************************/
/** \file csio_int_s25fl127s.h
 **
 ** S25FL127S driver header file
 **
 ** The S25fl127S is tested in the example, make following pin connection first.
 **
 ** ---------------------------------------------------------
 **| MCU             S25fl127S                               |
 **|--------------------------------------------------------- 
 **| Pin Name        Pin Count     Pin Name                  |
 **| SCS10_1         1             CS#    (pull-up)          |
 **| SIN1_1          2             SO/IO1 (pull-up)          |
 **| -               3             WP#/IO2 (pull-up)         |
 **| GND             4             GND                       |
 **| SOT1_1          5             SI/IO0 (pull-up)          |
 **| SCK1_1          6             CLK    (pull-up)          |
 **| -               7             HOLD#/IO3/RESET# (pull-up)|
 **| VCC             8             VCC                       |
 ** ---------------------------------------------------------
 **
 ** This driver provides basic operation APIs to access to S25FL127S, based on 
 ** default memory map as following:
 ** -------- 0x00FFFFFF 
 **| ...    |           
 ** --------          
 **| 64k    |            
 ** -------- 0x00010000  
 **| ...    |          
 ** --------             
 **| 4k     |          
 ** -------- 0x00000000  
 **
 ** The memory map can be configured to other types, which is not implemented 
 ** and considered in this driver file.
 **
 ** History:
 **   - 2014-03-10  1.0  Edison Zhang        First release version.
 **
 ******************************************************************************/
#ifndef _CSIO_INT_S25FL127S_H_
#define _CSIO_INT_S25FL127S_H_

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_user.h"
#include "mfs\mfs.h"
#include "gpio\gpio.h"

#if (PDL_UTILITY_ENABLE_CSIO_INT_S25FL127S == PDL_ON)

/**
 ******************************************************************************
 ** \defgroup CsioIntS25fl127sGroup Access S25FL127 with CSIO interrupt mode
 **
 ** Provided functions of CsioIntS25fl127sGroup module:
 **
 ** - S25fl127s_Init()
 ** - S25fl127s_ReadId()
 ** - At24cxx_ByteWrite()
 ** - S25fl127s_BulkErase()
 ** - S25fl127s_4kSectorErase()
 ** - S25fl127s_SectorErase()
 ** - S25fl127s_PageProgram()  
 ** - S25fl127s_Read()
 **
 ** S25fl127s_Init() initializes the CSIO channel to access AT24CXX.
 **
 ** S25fl127s_ReadId() reads the device ID and manufacture ID.
 **
 ** At24cxx_ByteWrite() write 1 byte at a romdon address. 
 **
 ** S25fl127s_BulkErase() erases whole memory of S25FL127S
 **
 ** S25fl127s_4kSectorErase() can erase 4kB sector of S25FL127S.
 **
 ** S25fl127s_SectorErase() can erase 64kB sector or 256kB sector.
 **
 ** S25fl127s_PageProgram() can program up to 1 page bytes, at the default 
 ** 1 page contains 256bytes, to change the page size is possible, but it is 
 ** not implemented and considered in this driver.
 **
 ** S25fl127s_Read() can read as many bytes as whole memory from the given
 ** address.
 **
 ******************************************************************************/ 

/******************************************************************************
 * Global definitions
 ******************************************************************************/
///< CSIO function initialization
#define InitCsioIo(void)  {SetPinFunc_SIN1_1(); SetPinFunc_SOT1_1();SetPinFunc_SCK1_1();SetPinFunc_SCS10_1();}

///< CSIO channel used for S25FL127S
#define FLASH_CSIO_CH        &CSIO1
   
///< Flash default page size   
#define S25FL127S_DEFAULT_PAGE_SIZE     (256u)
   
/*---------------------------------------------------------------------------*/
/* function prototypes                                                       */
/*---------------------------------------------------------------------------*/
#ifdef __cplusplus
extern "C" {
#endif
en_result_t S25fl127s_Init(void);
en_result_t S25fl127s_ReadId(uint8_t* pManufactureId, uint8_t* pDeviceId);
en_result_t S25fl127s_BulkErase(void);
en_result_t S25fl127s_4kSectorErase(uint32_t u32Addr);
en_result_t S25fl127s_SectorErase(uint32_t u32Addr);
en_result_t S25fl127s_PageProgram(uint32_t u32Addr, uint8_t* pu8Data, uint16_t u16Size);
en_result_t S25fl127s_Read(uint32_t u32Addr, uint8_t* pu8Data, uint32_t u32Size);

#ifdef __cplusplus
}
#endif

#endif // #if (PDL_UTILITY_ENABLE_CSIO_INT_S25FL127S == PDL_ON)

#endif //  _CSIO_INT_S25FL127S_H_

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
